#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

#include <axis/IWrapperSoapSerializer.hpp>

#include "../soap/SoapSerializer.h"

AXIS_CPP_NAMESPACE_USE

extern "C" {

#include <axis/GDefine.h>

#include <axis/AxisUserAPI.h>

#include <axis/BasicNode.h>

#include <axis/SoapEnvVersions.h>

#include <axis/TypeMapping.h>

#include <axis/WSDDDefines.h>

#include <axis/IWrapperSoapSerializer.h>

AXISC_STORAGE_CLASS_INFO void axiscDestroyIWrapperSoapSerializer(AXISCHANDLE wrapperSoapSerializer) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscDestroyIWrapperSoapSerializer", NULL, 1,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
	delete sz;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscDestroyIWrapperSoapSerializer", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO int axiscCreateSoapMethod(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * sLocalName, 
	const AxiscChar * sURI) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscCreateSoapMethod", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&sLocalName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&sURI));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->createSoapMethod(sLocalName,sURI));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscCreateSoapMethod", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->createSoapMethod(sLocalName,sURI);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscCreateSoapFault(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * sLocalName, 
	const AxiscChar * sURI, const AxiscChar * sFaultCode, 
	const AxiscChar * sFaultString) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscCreateSoapFault", NULL, 5,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&sLocalName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&sURI),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&sFaultCode),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&sFaultString));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->createSoapFault(sLocalName,sURI,sFaultCode,sFaultString));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscCreateSoapFault", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->createSoapFault(sLocalName,sURI,sFaultCode,sFaultString);
		#endif
	}

}

//AXISC_STORAGE_CLASS_INFO const AxiscChar * axiscGetNamespacePrefixIWrapperSoapSerializer(

//	AXISCHANDLE wrapperSoapSerializer, const AxiscChar * pNamespace) {

//	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;

//	return sz->getNamespacePrefix(pNamespace);

//}

AXISC_STORAGE_CLASS_INFO const AxiscChar * axiscGetNamespacePrefixIWrapperSoapSerializer(
    AXISCHANDLE wrapperSoapSerializer, const AxiscChar * pNamespace,
    AxiscBool * blnIsNewPrefix) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetNamespacePrefixIWrapperSoapSerializer", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace),
					TRACETYPE_POINTER, sizeof(AxiscBool), ((void*)&blnIsNewPrefix));	  /* AUTOINSERTED TRACE */
	#endif

    IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
    
    bool isNewPrefix = false;
    const AxiscChar * returnValue = sz->getNamespacePrefix(pNamespace, isNewPrefix);
    *(blnIsNewPrefix) = (AxiscBool) isNewPrefix;
    	{
		#ifdef ENABLE_AXISTRACE
			const AxiscChar* traceRet = (returnValue);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetNamespacePrefixIWrapperSoapSerializer", NULL, 0,
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return returnValue;
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void axiscRemoveNamespacePrefix(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscRemoveNamespacePrefix", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
	sz->removeNamespacePrefix(pNamespace);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscRemoveNamespacePrefix", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO int axiscAddOutputParam(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * pchName, 
	void * pValue, AXISC_XSDTYPE type) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddOutputParam", NULL, 4,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pchName),
					TRACETYPE_POINTER, 1, ((void*)&pValue),
					TRACETYPE_DATA, sizeof(AXISC_XSDTYPE), ((void*)&type));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->addOutputParam(pchName,pValue,(XSDTYPE)type));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddOutputParam", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->addOutputParam(pchName,pValue,(XSDTYPE)type);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscAddOutputCmplxArrayParam(AXISCHANDLE wrapperSoapSerializer, const Axisc_Array * pArray, 
	void * pSZFunct, void * pDelFunct, void * pSizeFunct, 
	const AxiscChar * pName, const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddOutputCmplxArrayParam", NULL, 7,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(Axisc_Array), ((void*)&pArray),
					TRACETYPE_POINTER, 1, ((void*)&pSZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, 1, ((void*)&pSizeFunct),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->addOutputCmplxArrayParam((const Axis_Array*)pArray,pSZFunct,pDelFunct,pSizeFunct,pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddOutputCmplxArrayParam", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->addOutputCmplxArrayParam((const Axis_Array*)pArray,pSZFunct,pDelFunct,pSizeFunct,pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscAddOutputBasicArrayParam(AXISCHANDLE wrapperSoapSerializer, const Axisc_Array * pArray, 
	AXISC_XSDTYPE nType, const AxiscChar * pName) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddOutputBasicArrayParam", NULL, 4,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(Axisc_Array), ((void*)&pArray),
					TRACETYPE_DATA, sizeof(AXISC_XSDTYPE), ((void*)&nType),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->addOutputBasicArrayParam((const Axis_Array*)pArray,(XSDTYPE)nType,pName));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddOutputBasicArrayParam", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->addOutputBasicArrayParam((const Axis_Array*)pArray,(XSDTYPE)nType,pName);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscAddOutputCmplxParam(AXISCHANDLE wrapperSoapSerializer, void * pObject, 
	void * pSZFunct, void * pDelFunct, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddOutputCmplxParam", NULL, 6,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, 1, ((void*)&pObject),
					TRACETYPE_POINTER, 1, ((void*)&pSZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->addOutputCmplxParam(pObject,pSZFunct,pDelFunct,pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddOutputCmplxParam", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->addOutputCmplxParam(pObject,pSZFunct,pDelFunct,pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscAddFaultDetail(AXISCHANDLE wrapperSoapSerializer, void * pObject, 
	void * pSZFunct, void * pDelFunct, const AxiscChar * pName, 
	const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddFaultDetail", NULL, 6,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, 1, ((void*)&pObject),
					TRACETYPE_POINTER, 1, ((void*)&pSZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->addFaultDetail(pObject,pSZFunct,pDelFunct,pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddFaultDetail", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->addFaultDetail(pObject,pSZFunct,pDelFunct,pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscSerializeCmplxArray(AXISCHANDLE wrapperSoapSerializer, const Axisc_Array * pArray, 
	void * pSZFunct, void * pDelFunct, void * pSizeFunct, 
	const AxiscChar * pName, const AxiscChar * pNamespace) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSerializeCmplxArray", NULL, 7,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(Axisc_Array), ((void*)&pArray),
					TRACETYPE_POINTER, 1, ((void*)&pSZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, 1, ((void*)&pSizeFunct),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->serializeCmplxArray((const Axis_Array*)pArray,pSZFunct,pDelFunct,pSizeFunct,pName,pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSerializeCmplxArray", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->serializeCmplxArray((const Axis_Array*)pArray,pSZFunct,pDelFunct,pSizeFunct,pName,pNamespace);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscSerializeBasicArray(AXISCHANDLE wrapperSoapSerializer, const Axisc_Array * pArray, 
	const AxiscChar * pNamespace,AXISC_XSDTYPE nType, const AxiscChar * pName) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSerializeBasicArray", NULL, 5,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(Axisc_Array), ((void*)&pArray),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace),
					TRACETYPE_DATA, sizeof(AXISC_XSDTYPE), ((void*)&nType),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->serializeBasicArray((const Axis_Array*)pArray,pNamespace,(XSDTYPE)nType,pName));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSerializeBasicArray", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->serializeBasicArray((const Axis_Array*)pArray,pNamespace,(XSDTYPE)nType,pName);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscSerializeAsElement(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * sName, 
	const AxiscChar * pNamespace, void * pValue, AXISC_XSDTYPE type) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSerializeAsElement", NULL, 5,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&sName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace),
					TRACETYPE_POINTER, 1, ((void*)&pValue),
					TRACETYPE_DATA, sizeof(AXISC_XSDTYPE), ((void*)&type));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
	if (pNamespace)
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->serializeAsElement(sName,pNamespace,pValue,(XSDTYPE)type));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSerializeAsElement", NULL, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->serializeAsElement(sName,pNamespace,pValue,(XSDTYPE)type);
		#endif
	}

		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->serializeAsElement(sName,pValue,(XSDTYPE)type));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSerializeAsElement", NULL, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->serializeAsElement(sName,pValue,(XSDTYPE)type);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscSerializeAsAttribute(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * sName, 
	const AxiscChar * pNamespace, void * pValue, AXISC_XSDTYPE type) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSerializeAsAttribute", NULL, 5,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&sName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace),
					TRACETYPE_POINTER, 1, ((void*)&pValue),
					TRACETYPE_DATA, sizeof(AXISC_XSDTYPE), ((void*)&type));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->serializeAsAttribute(sName,pNamespace,pValue,(XSDTYPE)type));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSerializeAsAttribute", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->serializeAsAttribute(sName,pNamespace,pValue,(XSDTYPE)type);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void axiscSerializeStartElementOfType(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * pName, 
	const AxiscChar * pNamespace, const AxiscChar * pPrefix) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSerializeStartElementOfType", NULL, 4,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pNamespace),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pPrefix));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
	sz->serializeStartElementOfType(pName,pNamespace,pPrefix);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSerializeStartElementOfType", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO void axiscSerializeEndElementOfType(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * pName) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSerializeEndElementOfType", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&pName));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
	sz->serializeEndElementOfType(pName);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSerializeEndElementOfType", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO AXISC_PROVIDERTYPE axiscGetCurrentProviderType(AXISCHANDLE wrapperSoapSerializer) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscGetCurrentProviderType", NULL, 1,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			AXISC_PROVIDERTYPE traceRet = ((AXISC_PROVIDERTYPE)(sz->getCurrentProviderType()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscGetCurrentProviderType", NULL, 0,
					TRACETYPE_DATA, sizeof(AXISC_PROVIDERTYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (AXISC_PROVIDERTYPE)(sz->getCurrentProviderType());
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void axiscSetCurrentProviderType(AXISCHANDLE wrapperSoapSerializer, AXISC_PROVIDERTYPE nType) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSetCurrentProviderType", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_DATA, sizeof(AXISC_PROVIDERTYPE), ((void*)&nType));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
	sz->setCurrentProviderType((PROVIDERTYPE)nType);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSetCurrentProviderType", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO int axiscAddOutputAnyObject(AXISCHANDLE wrapperSoapSerializer, AxiscAnyType * pAnyObject) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddOutputAnyObject", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscAnyType), ((void*)&pAnyObject));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->addOutputAnyObject((AnyType*)pAnyObject));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddOutputAnyObject", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->addOutputAnyObject((AnyType*)pAnyObject);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscSerializeAnyObject(AXISCHANDLE wrapperSoapSerializer, AxiscAnyType * pAnyObject) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSerializeAnyObject", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscAnyType), ((void*)&pAnyObject));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->serializeAnyObject((AnyType*)pAnyObject));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSerializeAnyObject", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->serializeAnyObject((AnyType*)pAnyObject);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO int axiscSerializeAsChardata(AXISCHANDLE wrapperSoapSerializer, void * pValue, 
	AXISC_XSDTYPE type) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSerializeAsChardata", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, 1, ((void*)&pValue),
					TRACETYPE_DATA, sizeof(AXISC_XSDTYPE), ((void*)&type));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (sz->serializeAsChardata(pValue,(XSDTYPE)type));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSerializeAsChardata", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return sz->serializeAsChardata(pValue,(XSDTYPE)type);
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void axiscAddAttachment(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * achId, 
	AXISCHANDLE objAttach) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddAttachment", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&achId),
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&objAttach));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
	sz->addAttachment(achId,(ISoapAttachment*)objAttach);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddAttachment", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO void axiscAddAttachmentBody(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * achId, 
	xsdc__base64Binary * pAttchBody) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddAttachmentBody", NULL, 3,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&achId),
					TRACETYPE_POINTER, sizeof(xsdc__base64Binary), ((void*)&pAttchBody));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
	sz->addAttachmentBody(achId,(xsd__base64Binary*)pAttchBody);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddAttachmentBody", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO void axiscAddAttachmentHeader(AXISCHANDLE wrapperSoapSerializer, const AxiscChar * achId, 
	const AxiscChar * achHeaderName, const AxiscChar * achHeaderValue) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscAddAttachmentHeader", NULL, 4,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&achId),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&achHeaderName),
					TRACETYPE_POINTER, sizeof(AxiscChar), ((void*)&achHeaderValue));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
	sz->addAttachmentHeader(achId,achHeaderName,achHeaderValue);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscAddAttachmentHeader", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXISC_STORAGE_CLASS_INFO AXISCHANDLE axiscCreateSoapAttachmentSoapAttachment(AXISCHANDLE wrapperSoapSerializer) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscCreateSoapAttachmentSoapAttachment", NULL, 1,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer));	  /* AUTOINSERTED TRACE */
	#endif

	IWrapperSoapSerializer *sz = (IWrapperSoapSerializer*)wrapperSoapSerializer;
		{
		#ifdef ENABLE_AXISTRACE
			AXISCHANDLE traceRet = ((AXISCHANDLE)(sz->createSoapAttachment()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscCreateSoapAttachmentSoapAttachment", NULL, 0,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (AXISCHANDLE)(sz->createSoapAttachment());
		#endif
	}

}

AXISC_STORAGE_CLASS_INFO void axiscSerializeIWrapperSoapSerializer(
	AXISCHANDLE wrapperSoapSerializer, const char *pFirst, ...) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axiscSerializeIWrapperSoapSerializer", NULL, 2,
					TRACETYPE_DATA, sizeof(AXISCHANDLE), ((void*)&wrapperSoapSerializer),
					TRACETYPE_STRING, 0, ((void*)&pFirst) );	  /* AUTOINSERTED TRACE */
	#endif

	SoapSerializer *sz = (SoapSerializer*)wrapperSoapSerializer;
	// TODO: There were problems getting the ... to work properly, so serializeVargs was
	// introduced. Also wsdl2ws only ever calls this method with one parameter so
	// ignore the extra parameters here for the moment. This should be fixed properly
	// later. I think SoapSerializer::serialize is wrong to assume that the variable
	// list of parameters will be terminated by a null pointer. I think we should always
	// pass in an explicit parameter count here.
	sz->serializeVargs(1,&pFirst);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axiscSerializeIWrapperSoapSerializer", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

}

