#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 *
 */

#include "../common/MessageData.h"

#include "IDeployerUtils.h"

#include "../wsdd/WSDDDeployment.h"

extern AXIS_CPP_NAMESPACE_PREFIX WSDDDeployment* g_pWSDDDeployment;

AXIS_CPP_NAMESPACE_START

const AxisChar* MessageData::m_pachBlankPropertyValue = "";

AdminUtils MessageData::m_AdminUtil;

MessageData::MessageData ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "MessageData", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_pSZ = NULL;
    m_pDZ = NULL;
    m_bPastPivotState = false;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "MessageData", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

MessageData::~MessageData ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "~MessageData", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    map < const AxisChar*, const AxisChar*, ltstr >::const_iterator itCurrentItem =
        m_Properties.begin ();
    while (itCurrentItem != m_Properties.end ())
    {
        delete [] (char*)(*itCurrentItem).first;
		delete [] (char*)(*itCurrentItem).second;
        itCurrentItem++;
    }
    m_Properties.clear ();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "~MessageData", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void MessageData::setSerializer (IWrapperSoapSerializer* pSZ)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "setSerializer", this, 1,
					TRACETYPE_POINTER, sizeof(IWrapperSoapSerializer), ((void*)&pSZ));	  /* AUTOINSERTED TRACE */
	#endif

    m_pSZ = pSZ;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "setSerializer", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void MessageData::setDeSerializer (IWrapperSoapDeSerializer* pDZ)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "setDeSerializer", this, 1,
					TRACETYPE_POINTER, sizeof(IWrapperSoapDeSerializer), ((void*)&pDZ));	  /* AUTOINSERTED TRACE */
	#endif

    m_pDZ = pDZ;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "setDeSerializer", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void MessageData::setOperationName(const AxisChar* pchOperation)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "setOperationName", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pchOperation));	  /* AUTOINSERTED TRACE */
	#endif

	m_sOperationName = pchOperation;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "setOperationName", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const AxisChar* MessageData::getOperationName()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "getOperationName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			const AxisChar* traceRet = (m_sOperationName.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getOperationName", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_sOperationName.c_str();
		#endif
	}

}

void MessageData::setUserName (string &sUserName)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "setUserName", this, 1,
					TRACETYPE_STLSTRING, 0, ((void*)&sUserName));	  /* AUTOINSERTED TRACE */
	#endif

    m_sUserName = sUserName;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "setUserName", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

string &MessageData::getUserName ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "getUserName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			string& traceRet = (m_sUserName);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getUserName", this, 0,
					TRACETYPE_STLSTRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_sUserName;
		#endif
	}

    // return "damitha kumarage"; 
}

/*
 *    comm on 25Jul2003 at 10.00am
 *    ISoapSerializer* MessageData::getSoapSerializer()
 *    {
 *        return m_pSZ;
 *    }
 */

IWrapperSoapDeSerializer* MessageData::getSoapDeserializer ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "getSoapDeserializer", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			IWrapperSoapDeSerializer* traceRet = (m_pDZ);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getSoapDeserializer", this, 0,
					TRACETYPE_POINTER, sizeof(IWrapperSoapDeSerializer), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pDZ;
		#endif
	}

}

bool MessageData::isPastPivot ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "isPastPivot", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			bool traceRet = (m_bPastPivotState);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "isPastPivot", this, 0,
					TRACETYPE_BOOL, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_bPastPivotState;
		#endif
	}

}

int MessageData::setPastPivotState (bool bState)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "setPastPivotState", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&bState));	  /* AUTOINSERTED TRACE */
	#endif

    m_bPastPivotState = bState;
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "setPastPivotState", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

void MessageData::getSoapSerializer 
    (IHandlerSoapSerializer** pIHandlerSoapSerializer)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "getSoapSerializer", this, 1,
					TRACETYPE_POINTER, sizeof(IHandlerSoapSerializer*), ((void*)&pIHandlerSoapSerializer));	  /* AUTOINSERTED TRACE */
	#endif

    *pIHandlerSoapSerializer = static_cast < IHandlerSoapSerializer* >(m_pSZ);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getSoapSerializer", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void MessageData::getSoapSerializer (IWrapperSoapSerializer**
    pIWrapperSoapSerializer)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "getSoapSerializer", this, 1,
					TRACETYPE_POINTER, sizeof(IWrapperSoapSerializer*), ((void*)&pIWrapperSoapSerializer));	  /* AUTOINSERTED TRACE */
	#endif

    *pIWrapperSoapSerializer = static_cast < IWrapperSoapSerializer* >(m_pSZ);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getSoapSerializer", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void MessageData::getSoapDeSerializer (IWrapperSoapDeSerializer**
    pIWrapperSoapDeSerializer)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "getSoapDeSerializer", this, 1,
					TRACETYPE_POINTER, sizeof(IWrapperSoapDeSerializer*), ((void*)&pIWrapperSoapDeSerializer));	  /* AUTOINSERTED TRACE */
	#endif

    *pIWrapperSoapDeSerializer = 
        static_cast < IWrapperSoapDeSerializer* >(m_pDZ);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getSoapDeSerializer", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void MessageData::getSoapDeSerializer (IHandlerSoapDeSerializer**
    pIHandlerSoapDeSerializer)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "getSoapDeSerializer", this, 1,
					TRACETYPE_POINTER, sizeof(IHandlerSoapDeSerializer*), ((void*)&pIHandlerSoapDeSerializer));	  /* AUTOINSERTED TRACE */
	#endif

    *pIHandlerSoapDeSerializer = 
        static_cast < IHandlerSoapDeSerializer * >(m_pDZ);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getSoapDeSerializer", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int MessageData::setProperty (const AxisChar* pachName, const AxisChar* pachValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "setProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pachName),
					TRACETYPE_STRING, 0, ((void*)&pachValue));	  /* AUTOINSERTED TRACE */
	#endif

    AxisChar* pachTmpName = new AxisChar[strlen (pachName) + 1];
    strcpy (pachTmpName, pachName);
    AxisChar* pachTmpValue = new AxisChar[strlen (pachValue) + 1];
    strcpy (pachTmpValue, pachValue);
    m_Properties[pachTmpName] = pachTmpValue;
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "setProperty", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int MessageData::setProperty (const AxisChar* pachName, const void* pachValue, int len)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "setProperty", this, 3,
					TRACETYPE_STRING, 0, ((void*)&pachName),
					TRACETYPE_POINTER, 1, ((void*)&pachValue),
					TRACETYPE_INT, 0, ((void*)&len));	  /* AUTOINSERTED TRACE */
	#endif

     AxisChar* pachTmpName = new AxisChar[strlen (pachName) + 1];
     strcpy (pachTmpName, pachName);
     AxisChar* pachTmpValue = new AxisChar[len];
     memcpy (pachTmpValue, pachValue, len);
 
     m_Properties[pachTmpName] = pachTmpValue;
 
     	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "setProperty", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

void MessageData::setComplexProperty(const AxisChar* pachName, void* pValue, int iObjectSize)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "setComplexProperty", this, 3,
					TRACETYPE_STRING, 0, ((void*)&pachName),
					TRACETYPE_POINTER, 1, ((void*)&pValue),
					TRACETYPE_INT, 0, ((void*)&iObjectSize));	  /* AUTOINSERTED TRACE */
	#endif

    AxisChar* pachTmpName = new AxisChar[strlen (pachName) + 1];
    strcpy (pachTmpName, pachName);
    void* pchTmpObject = malloc(iObjectSize);
    pchTmpObject = pValue;
    m_ComplexProperties[pachTmpName] = pchTmpObject;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "setComplexProperty", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/*
 *    The method caller has to check whether the returned string is empty or
 *    not. If it is empty then the idea is that the property is not 
 *    available.
 */

const void* MessageData::getProperty (const AxisChar* pachName)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "getProperty", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pachName));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_Properties.find (pachName) != m_Properties.end ())
    {
        	{
		#ifdef ENABLE_AXISTRACE
			const void* traceRet = (m_Properties[pachName]);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getProperty", this, 1,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_Properties[pachName];
		#endif
	}

    }
    	{
		#ifdef ENABLE_AXISTRACE
			const void* traceRet = (m_pachBlankPropertyValue);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getProperty", this, 2,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pachBlankPropertyValue;
		#endif
	}

}

void* MessageData::getComplexProperty(const AxisChar* pachName)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "getComplexProperty", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pachName));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_ComplexProperties.find (pachName) != m_ComplexProperties.end ())
    {
        	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (m_ComplexProperties[pachName]);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getComplexProperty", this, 1,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_ComplexProperties[pachName];
		#endif
	}

    }
                                                                                                                                                                            
    	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getComplexProperty", this, 2,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}
	
}

void MessageData::getAdminUtils (IAdminUtils** pIAdminUtils)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("MessageData", "getAdminUtils", this, 1,
					TRACETYPE_POINTER, sizeof(IAdminUtils*), ((void*)&pIAdminUtils));	  /* AUTOINSERTED TRACE */
	#endif

    *pIAdminUtils = &m_AdminUtil;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("MessageData", "getAdminUtils", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXIS_CPP_NAMESPACE_END

