#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 *
 */

#include "AxisEngine.h"

#include <stdio.h>

#include "HandlerPool.h"

#include "DeserializerPool.h"

#include "SerializerPool.h"

#include "../common/AxisTrace.h"

extern AXIS_CPP_NAMESPACE_PREFIX DeserializerPool* g_pDeserializerPool;

extern AXIS_CPP_NAMESPACE_PREFIX SerializerPool* g_pSerializerPool;

extern AXIS_CPP_NAMESPACE_PREFIX HandlerPool* g_pHandlerPool;

// extern WSDDDeployment* g_pWSDDDeployment;

AXIS_CPP_NAMESPACE_START

AxisEngine::AxisEngine ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisEngine", "AxisEngine", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_pSZ = NULL;
    m_pDZ = NULL;
    m_pGReqFChain = NULL;
    m_pGResFChain = NULL;
    m_pTReqFChain = NULL;
    m_pTResFChain = NULL;
    m_pSReqFChain = NULL;
    m_pSResFChain = NULL;
    m_pMsgData = NULL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "AxisEngine", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AxisEngine::~AxisEngine ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisEngine", "~AxisEngine", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pDZ)
        g_pDeserializerPool->putInstance (m_pDZ);
    if (m_pSZ)
        g_pSerializerPool->putInstance (m_pSZ);
    if (m_pMsgData)
        delete m_pMsgData;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "~AxisEngine", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int AxisEngine::initialize ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisEngine", "initialize", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    int Status;
    m_pMsgData = new MessageData ();
    if (!m_pMsgData)
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "initialize", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

    // Create and initialize Serializer and Deserializer objects
    if (AXIS_SUCCESS != (Status = g_pSerializerPool->getInstance ((IWrapperSoapSerializer**)&m_pSZ)))
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "initialize", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

    if (AXIS_SUCCESS != (Status = g_pDeserializerPool->getInstance ((IWrapperSoapDeSerializer**)&m_pDZ)))
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "initialize", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

    m_pMsgData->setSerializer (m_pSZ);
    m_pMsgData->setDeSerializer (m_pDZ);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "initialize", this, 4,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

void AxisEngine::unInitialize ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisEngine", "unInitialize", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pMsgData)
    {
        delete m_pMsgData;
        m_pMsgData = NULL;
    }
    /* nothing to do with m_pWSDD because its destructor deletes its objects
     * nothing to do with m_pHandlerPool because its destructor deletes its 
     * objects
     */ 
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "unInitialize", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int AxisEngine::initializeHandlers (string & sSessionId, 
    AXIS_PROTOCOL_TYPE protocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("AxisEngine", "initializeHandlers", this, 2,
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId),
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&protocol));	  /* AUTOINSERTED TRACE */
	#endif

    int Status = AXIS_SUCCESS;
    // Get Global Handlers from the pool if configured any
    if (AXIS_SUCCESS !=
        (Status =
        g_pHandlerPool->getGlobalRequestFlowHandlerChain (&m_pGReqFChain,
        sSessionId)))
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "initializeHandlers", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

    if (AXIS_SUCCESS !=
        (Status =
        g_pHandlerPool->getGlobalResponseFlowHandlerChain (&m_pGResFChain,
        sSessionId)))
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "initializeHandlers", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

    // Get Transport Handlers from the pool if configured any
    if (AXIS_SUCCESS !=
        (Status =
        g_pHandlerPool->getTransportRequestFlowHandlerChain (&m_pTReqFChain,
        sSessionId, protocol)))
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "initializeHandlers", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

    if (AXIS_SUCCESS !=
        (Status =
        g_pHandlerPool->getTransportResponseFlowHandlerChain (&m_pTResFChain,
        sSessionId, protocol)))
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "initializeHandlers", this, 4,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("AxisEngine", "initializeHandlers", this, 5,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

