#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 *
 */

#include "HandlerChain.h"

#include "../common/AxisTrace.h"

AXIS_CPP_NAMESPACE_START

HandlerChain::HandlerChain ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerChain", "HandlerChain", this, 0);	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerChain", "HandlerChain", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

HandlerChain::~HandlerChain ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerChain", "~HandlerChain", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    /* Nothing to do because the list keeps only pointers to objects that 
     * belongs to the Handler pool;
     */
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerChain", "~HandlerChain", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int HandlerChain::invoke (void* pMsg)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerChain", "invoke", this, 1,
					TRACETYPE_POINTER, 1, ((void*)&pMsg));	  /* AUTOINSERTED TRACE */
	#endif

    m_itCurrHandler = m_HandlerList.begin ();
    while (m_itCurrHandler != m_HandlerList.end ())
    {
        BasicHandler* pHandler = (*m_itCurrHandler).m_pHandler;
        /*
         * if (AXIS_SUCCESS == pHandler->_functions->Invoke(pHandler->_object, 
	 *     pMsg))
         * {            
         *     m_itCurrHandler++;
         * }
         * else
         * {
         *     AXISTRACE1("Handler invoke not successful", WARN);            
         *     OnFault(pMsg);
         *     return AXIS_FAIL;
         * }
         */
        
	//----------change by roshan on 12Feb2004-------------
        if (0 != pHandler->_functions)
        {
                /* This is a C Handler */
        }
        else if (0 != pHandler->_object)
        {
            if (AXIS_SUCCESS ==
                ((Handler *) (pHandler->_object))->invoke (pMsg))
            {
                m_itCurrHandler++;
            }
            else
            {
                AXISTRACE1 ("Handler invoke not successful", WARN);
                onFault (pMsg);
                	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerChain", "invoke", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

            }
        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerChain", "invoke", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

void HandlerChain::onFault (void* pMsg)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerChain", "onFault", this, 1,
					TRACETYPE_POINTER, 1, ((void*)&pMsg));	  /* AUTOINSERTED TRACE */
	#endif

    while (m_itCurrHandler != m_HandlerList.begin ())
    {
        BasicHandler* pHandler = (*m_itCurrHandler).m_pHandler;
        pHandler->_functions->onFault (pHandler->_object, pMsg);
        m_itCurrHandler--;
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerChain", "onFault", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int HandlerChain::addHandler (BasicHandler* pHandler, int nScope, int nLibId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerChain", "addHandler", this, 3,
					TRACETYPE_POINTER, sizeof(BasicHandler), ((void*)&pHandler),
					TRACETYPE_INT, 0, ((void*)&nScope),
					TRACETYPE_INT, 0, ((void*)&nLibId));	  /* AUTOINSERTED TRACE */
	#endif

    ChainItem item;
    item.m_pHandler = pHandler;
    item.m_nScope = nScope;
    item.m_nLibId = nLibId;
    m_HandlerList.push_back (item);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerChain", "addHandler", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int HandlerChain::init ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerChain", "init", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_HandlerList.clear ();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerChain", "init", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int HandlerChain::fini ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerChain", "fini", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_HandlerList.clear ();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerChain", "fini", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

