#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 * @author Damitha Kumarage (damitha@opensource.lk, damitha@jkcsworld.com)
 *
 */

#ifdef WIN32

#pragma warning (disable : 4390)        // empty controlled statement found;

#pragma warning (disable : 4786)

#endif

#include "../platforms/PlatformAutoSense.hpp"

#include "HandlerLoader.h"

#include <stdio.h>

#include "../common/AxisUtils.h"

#include "../wsdd/WSDDDeployment.h"

#include "../common/AxisTrace.h"

extern AXIS_CPP_NAMESPACE_PREFIX WSDDDeployment* g_pWSDDDeployment;

AXIS_CPP_NAMESPACE_START

HandlerLoader::HandlerLoader ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerLoader", "HandlerLoader", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

    PLATFORM_LOADLIBINIT();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerLoader", "HandlerLoader", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

HandlerLoader::~HandlerLoader ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerLoader", "~HandlerLoader", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

    //lock ();
	Lock l(this);
    HandlerInformation* pHandlerInfo = NULL;
    for (map < int, HandlerInformation* >::iterator it =
         m_HandlerInfoList.begin (); it != m_HandlerInfoList.end (); it++)
    {
        pHandlerInfo = (*it).second;
        if (pHandlerInfo->m_nObjCount != 0);  /* It seems that some objects
                                               * created have not been deleted
					                           * - unexpected
					                           */
            unloadLib (pHandlerInfo);
        delete pHandlerInfo;
    }
    //unlock ();
	l.unlock ();
    PLATFORM_LOADLIBEXIT()
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerLoader", "~HandlerLoader", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int HandlerLoader::deleteHandler (BasicHandler* pHandler, int nLibId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerLoader", "deleteHandler", NULL, 2,
					TRACETYPE_POINTER, sizeof(BasicHandler), ((void*)&pHandler),
					TRACETYPE_INT, 0, ((void*)&nLibId));	  /* AUTOINSERTED TRACE */
	#endif

    //lock ();
	Lock l(this);
    if (m_HandlerInfoList.find (nLibId) != m_HandlerInfoList.end ())
    {
        HandlerInformation* pHandlerInfo = m_HandlerInfoList[nLibId];
        pHandlerInfo->m_nObjCount--;
        pHandlerInfo->m_Delete (pHandler);
        //if (pHandlerInfo->m_nObjCount == 0);  //time to unload the DLL
          //  unlock ();
    }
    else
    {
        //unlock ();
        throw AxisEngineException(SERVER_ENGINE_HANDLER_NOT_LOADED);
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerLoader", "deleteHandler", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int HandlerLoader::loadLib (HandlerInformation* pHandlerInfo)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerLoader", "loadLib", NULL, 1,
					TRACETYPE_POINTER, sizeof(HandlerInformation), ((void*)&pHandlerInfo));	  /* AUTOINSERTED TRACE */
	#endif

    pHandlerInfo->m_Handler = PLATFORM_LOADLIB(pHandlerInfo->m_sLib.c_str());
    if (!pHandlerInfo->m_Handler)
    {
        AXISTRACE1("SERVER_ENGINE_LIBRARY_LOADING_FAILED", CRITICAL);
        long dwError = GETLASTERROR
        string *    message = PLATFORM_GET_ERROR_MESSAGE( dwError);
        char        fullMessage[1024];
        sprintf(fullMessage,
                "Failed to load handler within server engine: \n \
                Error Message='%s'\
                Error Code='%d'\n \
                Load lib error='%s' \n",
                message->c_str(), (int) dwError, PLATFORM_LOADLIB_ERROR);
        delete( message);
        throw AxisEngineException(SERVER_ENGINE_LIBRARY_LOADING_FAILED, fullMessage);
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerLoader", "loadLib", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int HandlerLoader::unloadLib (HandlerInformation* pHandlerInfo)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerLoader", "unloadLib", NULL, 1,
					TRACETYPE_POINTER, sizeof(HandlerInformation), ((void*)&pHandlerInfo));	  /* AUTOINSERTED TRACE */
	#endif

    PLATFORM_UNLOADLIB(pHandlerInfo->m_Handler);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerLoader", "unloadLib", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int HandlerLoader::createHandler (BasicHandler** pHandler, int nLibId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerLoader", "createHandler", NULL, 2,
					TRACETYPE_POINTER, sizeof(BasicHandler*), ((void*)&pHandler),
					TRACETYPE_INT, 0, ((void*)&nLibId));	  /* AUTOINSERTED TRACE */
	#endif

    //lock ();
	Lock l(this);
    *pHandler = NULL;
    HandlerInformation* pHandlerInfo = NULL;
    if (m_HandlerInfoList.find (nLibId) == m_HandlerInfoList.end ())
    {
        pHandlerInfo = new HandlerInformation ();
        pHandlerInfo->m_sLib = g_pWSDDDeployment->getLibName (nLibId);
        if (pHandlerInfo->m_sLib.empty ())
        {
            delete pHandlerInfo;
            //unlock ();
            AXISTRACE1("SERVER_CONFIG_LIBRARY_PATH_EMPTY", CRITICAL);
            throw AxisConfigException(SERVER_CONFIG_LIBRARY_PATH_EMPTY);
        }
        if (AXIS_SUCCESS == loadLib (pHandlerInfo))
        {
            pHandlerInfo->m_Create =
                (CREATE_OBJECT) PLATFORM_GETPROCADDR(pHandlerInfo->m_Handler,
                CREATE_FUNCTION);
            pHandlerInfo->m_Delete =
                (DELETE_OBJECT) PLATFORM_GETPROCADDR(pHandlerInfo->m_Handler,
                DELETE_FUNCTION);
            if (!pHandlerInfo->m_Create || !pHandlerInfo->m_Delete)
            {
                unloadLib (pHandlerInfo);
                delete pHandlerInfo;
                //unlock ();
                AXISTRACE1 ("Library loading failed", CRITICAL);
                throw AxisEngineException(SERVER_ENGINE_LIBRARY_LOADING_FAILED);
            }
            else // success
            {
                m_HandlerInfoList[nLibId] = pHandlerInfo;
            }
        }
        else
        {
            //unlock ();
            AXISTRACE1 ("Library loading failed", CRITICAL);
            throw AxisEngineException(SERVER_ENGINE_LIBRARY_LOADING_FAILED);
        }
    }
    pHandlerInfo = m_HandlerInfoList[nLibId];
    BasicHandler* pBH = NULL;
    pHandlerInfo->m_Create (&pBH);
    if (pBH)
    {
        if (0 != pBH->_functions)
        /* this is a C service or handler */
        {
            if (AXIS_SUCCESS == pBH->_functions->init (pBH->_object))
            {
                pHandlerInfo->m_nObjCount++;
                *pHandler = pBH;
            }
            else
            {
                pBH->_functions->fini (pBH->_object);
                pHandlerInfo->m_Delete (pBH);
                //unlock ();
                AXISTRACE1("SERVER_ENGINE_HANDLER_INIT_FAILED", CRITICAL);
                throw AxisEngineException(SERVER_ENGINE_HANDLER_INIT_FAILED);
            }
        }
        else if (0 == pBH->_object)
        {
            AXISTRACE1("SERVER_ENGINE_HANDLER_CREATION_FAILED", CRITICAL);
            throw AxisEngineException(SERVER_ENGINE_HANDLER_CREATION_FAILED);
        }
        else
        /* C++ service or handler */
        {
            if (AXIS_SUCCESS == ((HandlerBase*) pBH->_object)->init ())
            {
                pHandlerInfo->m_nObjCount++;
                *pHandler = pBH;
            }
            else
            {
                ((HandlerBase*) pBH->_object)->fini ();
                pHandlerInfo->m_Delete (pBH);
                //unlock ();
                AXISTRACE1("SERVER_ENGINE_HANDLER_INIT_FAILED", CRITICAL);
                throw AxisEngineException(SERVER_ENGINE_HANDLER_INIT_FAILED);
            }
        }
    }
    else
    {
        //unlock ();
        AXISTRACE1("SERVER_ENGINE_HANDLER_CREATION_FAILED", CRITICAL);
        throw AxisEngineException(SERVER_ENGINE_HANDLER_CREATION_FAILED);
    }
    unlock ();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerLoader", "createHandler", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

