#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 *
 */

#include "SharedObject.h"

#include "../platforms/PlatformAutoSense.hpp"

AXIS_CPP_NAMESPACE_START

SharedObject::SharedObject ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SharedObject", "SharedObject", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_bLocked = false;
#ifdef WIN32
    mut = CreateMutex(NULL, FALSE, NULL);
#else //Linux
    mut = new pthread_mutex_t;
    pthread_mutex_init (mut, NULL);
#endif
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SharedObject", "SharedObject", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

SharedObject::~SharedObject ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SharedObject", "~SharedObject", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

#ifdef WIN32
    CloseHandle(mut);
#else //Linux
    pthread_mutex_destroy (mut);
    delete mut;
#endif
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SharedObject", "~SharedObject", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/* Following functions should be improved to avoid chances of failure using 
 * platform specific mechanisms
 */ 
int SharedObject::lock ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SharedObject", "lock", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

#ifdef WIN32
    WaitForSingleObject(mut, INFINITE);
#else //Linux
    pthread_mutex_lock (mut);
#endif
    m_bLocked = true;
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SharedObject", "lock", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

int SharedObject::unlock ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SharedObject", "unlock", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

#ifdef WIN32
    ReleaseMutex(mut);
#else //Linux
    pthread_mutex_unlock (mut);
#endif
    m_bLocked = false;
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SharedObject", "unlock", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

