#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 * 
 * @author Sanjaya Singharage (sanjayas@opensource.lk)
 * @author Susantha Kumara (susantha@opensource.lk, skumara@virtusa.com)
 * @author Samisa Abeysinghe (sabeysinghe@virtusa.com)
 *
 */

#include <axis/client/Call.hpp>

#include <axis/AxisException.hpp>

#include "../../common/AxisConfig.h"

#include "ClientAxisEngine.h"

#include "../SOAPTransportFactory.h"

#include "../../transport/SOAPTransport.h"

#include "../../soap/SoapSerializer.h"

#include "../../soap/SoapDeSerializer.h"

#include "../../soap/HeaderBlock.h"

#include "../../common/AxisGenException.h"

#include "../../soap/Attribute.h"

extern AXIS_CPP_NAMESPACE_PREFIX AxisConfig* g_pConfig;

extern "C" int initialize_module (int bServer);

extern "C" int uninitialize_module ();

AXIS_CPP_NAMESPACE_USE

Call::Call ()
:m_pcEndPointUri(NULL), m_strProxyHost(""), m_uiProxyPort(0), m_bUseProxy(false),
m_bCallInitialized(false), m_pContentIdSet(NULL)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "Call", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_pAxisEngine = NULL;
    m_pIWSSZ = NULL;
    m_pIWSDZ = NULL;
    initialize_module (0);
    
    m_pTransport = NULL;
    m_nTransportType = APTHTTP1_1;
    
    m_nStatus = AXIS_SUCCESS;
    m_pchSessionID = NULL;
	m_pContentIdSet = new ContentIdSet();
    // Setup Transport
    try
    {
        // Get a transport object from factory
        if( !m_pTransport)
		{
            m_pTransport = SOAPTransportFactory::getTransportObject(m_nTransportType);
			if( !m_pTransport)
			{
				m_nStatus = AXIS_FAIL;
			}
		}
        // Engine initialization
        m_pAxisEngine = new ClientAxisEngine ();
        if (!m_pAxisEngine) 
        {
        	m_nStatus = AXIS_FAIL;
        }
        m_nStatus = m_pAxisEngine->initialize ();
    }
    catch( AxisException& e)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("Call", "Call", this, 1,
					TRACETYPE_AXISEXCEPTION, sizeof(AxisException&), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

        cleanup();
		throw AxisGenException( e.getExceptionCode(), e.what());
    }
    catch(...)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("Call", "Call", this, 2 );	  /* AUTOINSERTED TRACE */
	#endif

        cleanup();
        throw;
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "Call", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

Call::~Call ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "~Call", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_pAxisEngine->unInitialize ();
    
    cleanup();
    uninitialize_module();
    
	list<ISoapAttachment*>::iterator it = m_attachments.begin();
	while (it != m_attachments.end())
	{
		delete *it;
		it++;
	}
	m_attachments.clear();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "~Call", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void Call::cleanup()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "cleanup", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pContentIdSet)
        delete m_pContentIdSet;
    m_pContentIdSet = NULL;
    if (m_pTransport)
        SOAPTransportFactory::destroyTransportObject(m_pTransport);
    m_pTransport = NULL;
    if (m_pAxisEngine)
        delete m_pAxisEngine;
    m_pAxisEngine = NULL;
    if (m_pcEndPointUri)
        delete [] m_pcEndPointUri;  
	m_pcEndPointUri = NULL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "cleanup", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int Call::setEndpointURI (const char* pchEndpointURI)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "setEndpointURI", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pchEndpointURI));	  /* AUTOINSERTED TRACE */
	#endif

    m_pTransport->setEndpointUri(pchEndpointURI);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "setEndpointURI", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

void Call::setOperation (const char* pchOperation, const char* pchNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "setOperation", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pchOperation),
					TRACETYPE_STRING, 0, ((void*)&pchNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    m_pIWSSZ->createSoapMethod (pchOperation, pchNamespace);
    m_pAxisEngine->getMessageData()->setOperationName(pchOperation);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "setOperation", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void Call::addParameter (void* pValue, const char* pchName, XSDTYPE nType)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "addParameter", this, 3,
					TRACETYPE_POINTER, 1, ((void*)&pValue),
					TRACETYPE_STRING, 0, ((void*)&pchName),
					TRACETYPE_DATA, sizeof(XSDTYPE), ((void*)&nType));	  /* AUTOINSERTED TRACE */
	#endif

     m_nStatus = m_pIWSSZ->addOutputParam (pchName, pValue, nType);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "addParameter", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/*
 * Method used to add arrays of basic types as parameters
 */

void Call::addBasicArrayParameter (Axis_Array* pArray, XSDTYPE nType,
    const AxisChar* pName)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "addBasicArrayParameter", this, 3,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&pArray),
					TRACETYPE_DATA, sizeof(XSDTYPE), ((void*)&nType),
					TRACETYPE_STRING, 0, ((void*)&pName));	  /* AUTOINSERTED TRACE */
	#endif

    m_nStatus = m_pIWSSZ->addOutputBasicArrayParam (pArray, nType, pName);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "addBasicArrayParameter", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void Call::addCmplxArrayParameter (Axis_Array* pArray, void* pSZFunct,
    void* pDelFunct, void* pSizeFunct, const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "addCmplxArrayParameter", this, 6,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&pArray),
					TRACETYPE_POINTER, 1, ((void*)&pSZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, 1, ((void*)&pSizeFunct),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

     m_nStatus = m_pIWSSZ->addOutputCmplxArrayParam (pArray, pSZFunct, pDelFunct, pSizeFunct,
        pName, pNamespace);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "addCmplxArrayParameter", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void Call::addCmplxParameter (void* pObject, void* pSZFunct, void* pDelFunct,
    const AxisChar* pName, const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "addCmplxParameter", this, 5,
					TRACETYPE_POINTER, 1, ((void*)&pObject),
					TRACETYPE_POINTER, 1, ((void*)&pSZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

     m_nStatus = m_pIWSSZ->addOutputCmplxParam (pObject, pSZFunct, pDelFunct, pName,
        pNamespace);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "addCmplxParameter", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int Call::invoke ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "invoke", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

        m_nStatus =  m_pAxisEngine->process(m_pTransport);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "invoke", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

}

class HandlerProperty
{

public:
	HandlerProperty(AxisChar *n, void *v, int l) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerProperty", "HandlerProperty", NULL, 3,
					TRACETYPE_STRING, 0, ((void*)&n),
					TRACETYPE_POINTER, 1, ((void*)&v),
					TRACETYPE_INT, 0, ((void*)&l));	  /* AUTOINSERTED TRACE */
	#endif
name=n; value=v; len=l;	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerProperty", "HandlerProperty", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}
	// The storage pointed at by name and value is not owned by HandlerProperty
	AxisChar *name;
	void *value;
	int len;

};

int Call::initialize(PROVIDERTYPE nStyle)
/* Does this mean that the stub that uses this Call object as well as all 
 * client side handlers have the same PROVIDERTYPE ? 
 */
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "initialize", this, 1,
					TRACETYPE_DATA, sizeof(PROVIDERTYPE), ((void*)&nStyle));	  /* AUTOINSERTED TRACE */
	#endif

    m_bCallInitialized = true;
    /* Initialize re-usable objects of this instance (objects may have been 
     * populated by the previous call.)
     */
    try
    {
        m_nStatus = AXIS_SUCCESS;
            MessageData *msgData = m_pAxisEngine->getMessageData ();
            if (msgData)
            {
				list<void*>::iterator it = m_handlerProperties.begin();
				while (it != m_handlerProperties.end())
				{
					HandlerProperty *pHp = (HandlerProperty*)(*it);
					msgData->setProperty(pHp->name, pHp->value, pHp->len);
					delete pHp;
					it++;
				}
				m_handlerProperties.clear();
                msgData->getSoapSerializer ((IWrapperSoapSerializer**) 
                    (&m_pIWSSZ));
                msgData->getSoapDeSerializer ((IWrapperSoapDeSerializer**) 
                    (&m_pIWSDZ));
                if (m_pIWSSZ && m_pIWSDZ)
                {
                    m_pIWSSZ->setCurrentProviderType (nStyle);
                    m_pIWSDZ->setCurrentProviderType (nStyle);
                    m_pIWSSZ->reset();
                    m_pIWSDZ->init();
                    
                    switch (nStyle)
                    {
                        case C_RPC_PROVIDER:
                        case CPP_RPC_PROVIDER:
                            m_pIWSSZ->setStyle (RPC_ENCODED);
                            m_pIWSDZ->setStyle (RPC_ENCODED);
                            break;
                        case C_DOC_PROVIDER:
                        case CPP_DOC_PROVIDER:
                            m_pIWSSZ->setStyle (DOC_LITERAL);
                            m_pIWSDZ->setStyle (DOC_LITERAL);
                            break;
                        case COM_PROVIDER:
                            // TODO: ??
                            break;
                        default:;
                                //TODO: ??
                    }
                    if(m_pchSessionID)
                    {
                        msgData->setProperty("sessionid", m_pchSessionID);
                    }
					m_pIWSSZ->setContentIdSet(m_pContentIdSet);
					list<ISoapAttachment*>::iterator itAtt = m_attachments.begin();
					while (itAtt != m_attachments.end())
					{
						m_pIWSSZ->addAttachment((*itAtt)->getHeader(AXIS_CONTENT_ID),*itAtt);
						itAtt++;
					}
					m_attachments.clear();
                    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "initialize", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

                }
            }
            m_nStatus = AXIS_FAIL;
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "initialize", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

        m_nStatus = AXIS_FAIL;        
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "initialize", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

    }
    catch (AxisException& e)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("Call", "initialize", this, 1,
					TRACETYPE_AXISEXCEPTION, sizeof(AxisException&), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

        e = e;
        m_nStatus = AXIS_FAIL;
        throw;
    }
    catch (...)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("Call", "initialize", this, 2 );	  /* AUTOINSERTED TRACE */
	#endif

        m_nStatus = AXIS_FAIL;        
        throw;
    }
}

int Call::unInitialize ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "unInitialize", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_bCallInitialized = false;
    if (m_pAxisEngine)
    {
		//Initialization,serialization, invokation or check message success 
		if ( m_nStatus == AXIS_SUCCESS &&  m_pIWSDZ != NULL ) 
		{
			// Test if deserialization failed 
			m_nStatus = m_pIWSDZ->getStatus();
		}
		MessageData *msgData = m_pAxisEngine->getMessageData();	
        AxisChar * pachTemp = (AxisChar *)msgData->getProperty("sessionid");
        int len = strlen(pachTemp);
        if ( len > 0 ) // Samisa: check if there is a session key
        {
            if (m_pchSessionID) // Samisa: deallocate before allocation
            {
                delete [] m_pchSessionID;
                m_pchSessionID = NULL;
            }
            m_pchSessionID = new char[len + 1];  // Samisa: should have space for terminating char
            strcpy(m_pchSessionID, pachTemp);
        }
        else //Samisa: there is no session key
        {
            if (m_pchSessionID) 
                delete [] m_pchSessionID;
            m_pchSessionID = NULL;
        }
    }
    closeConnection ();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "unInitialize", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int Call::setProtocol (AXIS_PROTOCOL_TYPE protocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "setProtocol", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&protocol));	  /* AUTOINSERTED TRACE */
	#endif

	int success = AXIS_FAIL;
    if (m_pTransport)
    {
    	if( m_pTransport->setProtocol(protocol)==AXIS_SUCCESS)
    	{
    		m_nTransportType=protocol;
    		success = AXIS_SUCCESS;
    	}
    }
    else
    {
    		m_nTransportType=protocol;
    		success = AXIS_SUCCESS;
    }
   		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (success);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "setProtocol", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return success;
		#endif
	}

}

AXIS_PROTOCOL_TYPE Call::getProtocol ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getProtocol", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if(m_pTransport)
	{
			{
		#ifdef ENABLE_AXISTRACE
			AXIS_PROTOCOL_TYPE traceRet = (m_pTransport->getProtocol());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getProtocol", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pTransport->getProtocol();
		#endif
	}

	}
	else
	{
			{
		#ifdef ENABLE_AXISTRACE
			AXIS_PROTOCOL_TYPE traceRet = (m_nTransportType);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getProtocol", this, 2,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nTransportType;
		#endif
	}

	}
}

int Call::setTransportProperty( AXIS_TRANSPORT_INFORMATION_TYPE type, const char* value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "setTransportProperty", this, 2,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_INFORMATION_TYPE), ((void*)&type),
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

	int	iSuccess = AXIS_SUCCESS;
    // Samisa - if SOAPAction is being set add extra "" to value
    if (type == SOAPACTION_HEADER)
    {
        char* tempvalue = new char[strlen(value) + 3];
        sprintf( tempvalue, "\"%s\"", value);
        m_pTransport->setTransportProperty(type, tempvalue);
		if( tempvalue != NULL)
		{
        delete [] tempvalue;
			tempvalue = NULL;
		}
    }
    else
	{
		try
		{
			iSuccess = m_pTransport->setTransportProperty( type, value);
		}
		catch( AxisException& e)
		{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("Call", "setTransportProperty", this, 0,
					TRACETYPE_AXISEXCEPTION, sizeof(AxisException&), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

			throw AxisGenException(e.getExceptionCode(), e.what());
		}
	}
	if( iSuccess < 0)
	{
		throw AxisGenException( -iSuccess, m_pTransport->getLastChannelError());
	}
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (iSuccess);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "setTransportProperty", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return iSuccess;
		#endif
	}

}

const char* Call::getTransportProperty(const char *key, bool response) 
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&key),
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

	if (m_pTransport) 	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pTransport->getTransportProperty(key,response));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getTransportProperty", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pTransport->getTransportProperty(key,response);
		#endif
	}

	else 	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

int Call::setHandlerProperty(AxisChar* name, void* value, int len)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "setHandlerProperty", this, 3,
					TRACETYPE_STRING, 0, ((void*)&name),
					TRACETYPE_POINTER, 1, ((void*)&value),
					TRACETYPE_INT, 0, ((void*)&len));	  /* AUTOINSERTED TRACE */
	#endif

	// Unfortunately we have to cache the handler properties here
	// in the Call object since the m_pMsgData is not set up
	// until Call::initialize which doesn't happen until the actual
	// web service is invoked.
	m_handlerProperties.push_back(new HandlerProperty(name,value,len));
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "setHandlerProperty", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

/*
 * This method closes the connection of this object to the server
 */

void Call::closeConnection()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "closeConnection", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (m_pTransport) {
        m_pTransport->closeConnection();
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "closeConnection", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void Call::setSOAPVersion (SOAP_VERSION version)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "setSOAPVersion", this, 1,
					TRACETYPE_DATA, sizeof(SOAP_VERSION), ((void*)&version));	  /* AUTOINSERTED TRACE */
	#endif

    m_pIWSSZ->setSoapVersion (version);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "setSOAPVersion", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

Axis_Array* Call::getBasicArray (XSDTYPE nType, const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getBasicArray", this, 3,
					TRACETYPE_DATA, sizeof(XSDTYPE), ((void*)&nType),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (m_pIWSDZ->getBasicArray (nType, pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getBasicArray", this, 0,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getBasicArray (nType, pName, pNamespace);
		#endif
	}

}

Axis_Array* Call::getCmplxArray (Axis_Array * pArray, void* pDZFunct, void* pCreFunct, 
    void* pDelFunct, void* pSizeFunct, const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getCmplxArray", this, 7,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&pArray),
					TRACETYPE_POINTER, 1, ((void*)&pDZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pCreFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, 1, ((void*)&pSizeFunct),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (m_pIWSDZ->getCmplxArray (pArray, pDZFunct, pCreFunct, pDelFunct, pSizeFunct,
        pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getCmplxArray", this, 0,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getCmplxArray (pArray, pDZFunct, pCreFunct, pDelFunct, pSizeFunct,
        pName, pNamespace);
		#endif
	}

}

xsd__int * Call::getElementAsInt (const AxisChar* pName, const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsInt", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__int* traceRet = (m_pIWSDZ->getElementAsInt (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsInt", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__int), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsInt (pName, pNamespace);
		#endif
	}

}

xsd__boolean * Call::getElementAsBoolean (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsBoolean", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__boolean* traceRet = (m_pIWSDZ->getElementAsBoolean (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsBoolean", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__boolean), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsBoolean (pName, pNamespace);
		#endif
	}

}

xsd__unsignedInt * Call::getElementAsUnsignedInt (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsUnsignedInt", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedInt* traceRet = (m_pIWSDZ->getElementAsUnsignedInt (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsUnsignedInt", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedInt), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsUnsignedInt (pName, pNamespace);
		#endif
	}

}

xsd__short * Call::getElementAsShort (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsShort", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__short* traceRet = (m_pIWSDZ->getElementAsShort (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsShort", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__short), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsShort (pName, pNamespace);
		#endif
	}

}

xsd__unsignedShort * Call::getElementAsUnsignedShort (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsUnsignedShort", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedShort* traceRet = (m_pIWSDZ->getElementAsUnsignedShort (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsUnsignedShort", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedShort), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsUnsignedShort (pName, pNamespace);
		#endif
	}

}

xsd__byte * Call::getElementAsByte (const AxisChar* pName, const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsByte", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__byte* traceRet = (m_pIWSDZ->getElementAsByte (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsByte", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__byte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsByte (pName, pNamespace);
		#endif
	}

}

xsd__unsignedByte * Call::getElementAsUnsignedByte (const AxisChar* pName,
    const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsUnsignedByte", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedByte* traceRet = (m_pIWSDZ->getElementAsUnsignedByte (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsUnsignedByte", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedByte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsUnsignedByte (pName, pNamespace);
		#endif
	}

}

xsd__long * Call::getElementAsLong (const AxisChar* pName, const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsLong", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__long* traceRet = (m_pIWSDZ->getElementAsLong (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsLong", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__long), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsLong (pName, pNamespace);
		#endif
	}

}

xsd__integer * Call::getElementAsInteger (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__integer* traceRet = (m_pIWSDZ->getElementAsInteger (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__integer), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsInteger (pName, pNamespace);
		#endif
	}

}

xsd__unsignedLong * Call::getElementAsUnsignedLong (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsUnsignedLong", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedLong* traceRet = (m_pIWSDZ->getElementAsUnsignedLong (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsUnsignedLong", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedLong), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsUnsignedLong (pName, pNamespace);
		#endif
	}

}

xsd__float * Call::getElementAsFloat (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsFloat", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__float* traceRet = (m_pIWSDZ->getElementAsFloat (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsFloat", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__float), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsFloat (pName, pNamespace);
		#endif
	}

}

xsd__double * Call::getElementAsDouble (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsDouble", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__double* traceRet = (m_pIWSDZ->getElementAsDouble (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsDouble", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__double), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsDouble (pName, pNamespace);
		#endif
	}

}

xsd__decimal * Call::getElementAsDecimal (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsDecimal", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__decimal* traceRet = (m_pIWSDZ->getElementAsDecimal (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsDecimal", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__decimal), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsDecimal (pName, pNamespace);
		#endif
	}

}

xsd__string Call::getElementAsString (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsString", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__string traceRet = (m_pIWSDZ->getElementAsString (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsString", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsString (pName, pNamespace);
		#endif
	}

}

xsd__anyURI Call::getElementAsAnyURI (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsAnyURI", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__anyURI traceRet = (m_pIWSDZ->getElementAsAnyURI (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsAnyURI", this, 0,
					TRACETYPE_DATA, sizeof(xsd__anyURI), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsAnyURI (pName, pNamespace);
		#endif
	}

}

xsd__QName Call::getElementAsQName (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsQName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__QName traceRet = (m_pIWSDZ->getElementAsQName (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsQName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__QName), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsQName (pName, pNamespace);
		#endif
	}

}

xsd__hexBinary * Call::getElementAsHexBinary (const AxisChar* pName, 
    const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsHexBinary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__hexBinary* traceRet = (m_pIWSDZ->getElementAsHexBinary (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsHexBinary", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__hexBinary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsHexBinary (pName, pNamespace);
		#endif
	}

}

xsd__base64Binary * Call::getElementAsBase64Binary (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsBase64Binary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__base64Binary* traceRet = (m_pIWSDZ->getElementAsBase64Binary (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsBase64Binary", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__base64Binary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsBase64Binary (pName, pNamespace);
		#endif
	}

}

xsd__dateTime * Call::getElementAsDateTime (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsDateTime", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__dateTime* traceRet = (m_pIWSDZ->getElementAsDateTime (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsDateTime", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__dateTime), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsDateTime (pName, pNamespace);
		#endif
	}

}

xsd__date * Call::getElementAsDate (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsDate", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__date* traceRet = (m_pIWSDZ->getElementAsDate (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsDate", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__date), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsDate (pName, pNamespace);
		#endif
	}

}

xsd__time * Call::getElementAsTime (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsTime", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__time* traceRet = (m_pIWSDZ->getElementAsTime (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsTime", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__time), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsTime (pName, pNamespace);
		#endif
	}

}

xsd__duration * Call::getElementAsDuration (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsDuration", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__duration* traceRet = (m_pIWSDZ->getElementAsDuration (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsDuration", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__duration), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsDuration (pName, pNamespace);
		#endif
	}

}

xsd__gYearMonth * Call::getElementAsGYearMonth (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsGYearMonth", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gYearMonth* traceRet = (m_pIWSDZ->getElementAsGYearMonth (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsGYearMonth", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gYearMonth), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsGYearMonth (pName, pNamespace);
		#endif
	}

}

xsd__gYear * Call::getElementAsGYear (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsGYear", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gYear* traceRet = (m_pIWSDZ->getElementAsGYear (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsGYear", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gYear), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsGYear (pName, pNamespace);
		#endif
	}

}

xsd__gMonthDay * Call::getElementAsGMonthDay (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsGMonthDay", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gMonthDay* traceRet = (m_pIWSDZ->getElementAsGMonthDay (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsGMonthDay", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gMonthDay), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsGMonthDay (pName, pNamespace);
		#endif
	}

}

xsd__gDay * Call::getElementAsGDay (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsGDay", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gDay* traceRet = (m_pIWSDZ->getElementAsGDay (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsGDay", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gDay), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsGDay (pName, pNamespace);
		#endif
	}

}

xsd__gMonth * Call::getElementAsGMonth (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsGMonth", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gMonth* traceRet = (m_pIWSDZ->getElementAsGMonth (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsGMonth", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gMonth), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsGMonth (pName, pNamespace);
		#endif
	}

}

xsd__nonPositiveInteger * Call::getElementAsNonPositiveInteger (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsNonPositiveInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__nonPositiveInteger* traceRet = (m_pIWSDZ->getElementAsNonPositiveInteger (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsNonPositiveInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__nonPositiveInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsNonPositiveInteger (pName, pNamespace);
		#endif
	}

}

xsd__negativeInteger * Call::getElementAsNegativeInteger (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsNegativeInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__negativeInteger* traceRet = (m_pIWSDZ->getElementAsNegativeInteger (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsNegativeInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__negativeInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsNegativeInteger (pName, pNamespace);
		#endif
	}

}

xsd__nonNegativeInteger * Call::getElementAsNonNegativeInteger (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsNonNegativeInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__nonNegativeInteger* traceRet = (m_pIWSDZ->getElementAsNonNegativeInteger (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsNonNegativeInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__nonNegativeInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsNonNegativeInteger (pName, pNamespace);
		#endif
	}

}

xsd__positiveInteger * Call::getElementAsPositiveInteger (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsPositiveInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__positiveInteger* traceRet = (m_pIWSDZ->getElementAsPositiveInteger (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsPositiveInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__positiveInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsPositiveInteger (pName, pNamespace);
		#endif
	}

}

xsd__normalizedString Call::getElementAsNormalizedString (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsNormalizedString", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__normalizedString traceRet = (m_pIWSDZ->getElementAsNormalizedString (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsNormalizedString", this, 0,
					TRACETYPE_DATA, sizeof(xsd__normalizedString), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsNormalizedString (pName, pNamespace);
		#endif
	}

}

xsd__token Call::getElementAsToken (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsToken", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__token traceRet = (m_pIWSDZ->getElementAsToken (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsToken", this, 0,
					TRACETYPE_DATA, sizeof(xsd__token), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsToken (pName, pNamespace);
		#endif
	}

}

xsd__language Call::getElementAsLanguage (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsLanguage", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__language traceRet = (m_pIWSDZ->getElementAsLanguage (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsLanguage", this, 0,
					TRACETYPE_DATA, sizeof(xsd__language), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsLanguage (pName, pNamespace);
		#endif
	}

}

xsd__Name Call::getElementAsName (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__Name traceRet = (m_pIWSDZ->getElementAsName (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__Name), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsName (pName, pNamespace);
		#endif
	}

}

xsd__NCName Call::getElementAsNCName (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsNCName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NCName traceRet = (m_pIWSDZ->getElementAsNCName (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsNCName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NCName), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsNCName (pName, pNamespace);
		#endif
	}

}

xsd__ID Call::getElementAsID (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsID", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ID traceRet = (m_pIWSDZ->getElementAsID (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsID", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ID), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsID (pName, pNamespace);
		#endif
	}

}

xsd__IDREF Call::getElementAsIDREF (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsIDREF", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__IDREF traceRet = (m_pIWSDZ->getElementAsIDREF (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsIDREF", this, 0,
					TRACETYPE_DATA, sizeof(xsd__IDREF), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsIDREF (pName, pNamespace);
		#endif
	}

}

xsd__IDREFS Call::getElementAsIDREFS (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsIDREFS", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__IDREFS traceRet = (m_pIWSDZ->getElementAsIDREFS (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsIDREFS", this, 0,
					TRACETYPE_DATA, sizeof(xsd__IDREFS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsIDREFS (pName, pNamespace);
		#endif
	}

}

xsd__ENTITY Call::getElementAsENTITY (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsENTITY", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ENTITY traceRet = (m_pIWSDZ->getElementAsENTITY (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsENTITY", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ENTITY), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsENTITY (pName, pNamespace);
		#endif
	}

}

xsd__ENTITIES Call::getElementAsENTITIES (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsENTITIES", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ENTITIES traceRet = (m_pIWSDZ->getElementAsENTITIES (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsENTITIES", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ENTITIES), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsENTITIES (pName, pNamespace);
		#endif
	}

}

xsd__NMTOKEN Call::getElementAsNMTOKEN (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsNMTOKEN", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NMTOKEN traceRet = (m_pIWSDZ->getElementAsNMTOKEN (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsNMTOKEN", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NMTOKEN), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsNMTOKEN (pName, pNamespace);
		#endif
	}

}

xsd__NMTOKENS Call::getElementAsNMTOKENS (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsNMTOKENS", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NMTOKENS traceRet = (m_pIWSDZ->getElementAsNMTOKENS (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsNMTOKENS", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NMTOKENS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsNMTOKENS (pName, pNamespace);
		#endif
	}

}

xsd__NOTATION Call::getElementAsNOTATION (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getElementAsNOTATION", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NOTATION traceRet = (m_pIWSDZ->getElementAsNOTATION (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getElementAsNOTATION", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NOTATION), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getElementAsNOTATION (pName, pNamespace);
		#endif
	}

}

xsd__int * Call::getAttributeAsInt (const AxisChar* pName, const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsInt", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__int* traceRet = (m_pIWSDZ->getAttributeAsInt (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsInt", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__int), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsInt (pName, pNamespace);
		#endif
	}

}

xsd__boolean * Call::getAttributeAsBoolean (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsBoolean", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__boolean* traceRet = (m_pIWSDZ->getAttributeAsBoolean (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsBoolean", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__boolean), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsBoolean (pName, pNamespace);
		#endif
	}

}

xsd__unsignedInt * Call::getAttributeAsUnsignedInt (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsUnsignedInt", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedInt* traceRet = (m_pIWSDZ->getAttributeAsUnsignedInt (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsUnsignedInt", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedInt), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsUnsignedInt (pName, pNamespace);
		#endif
	}

}

xsd__short * Call::getAttributeAsShort (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsShort", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__short* traceRet = (m_pIWSDZ->getAttributeAsShort (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsShort", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__short), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsShort (pName, pNamespace);
		#endif
	}

}

xsd__unsignedShort * Call::getAttributeAsUnsignedShort (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsUnsignedShort", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedShort* traceRet = (m_pIWSDZ->getAttributeAsUnsignedShort (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsUnsignedShort", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedShort), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsUnsignedShort (pName, pNamespace);
		#endif
	}

}

xsd__byte * Call::getAttributeAsByte (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsByte", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__byte* traceRet = (m_pIWSDZ->getAttributeAsByte (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsByte", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__byte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsByte (pName, pNamespace);
		#endif
	}

}

xsd__unsignedByte * Call::getAttributeAsUnsignedByte (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsUnsignedByte", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedByte* traceRet = (m_pIWSDZ->getAttributeAsUnsignedByte (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsUnsignedByte", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedByte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsUnsignedByte (pName, pNamespace);
		#endif
	}

}

xsd__long * Call::getAttributeAsLong (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsLong", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__long* traceRet = (m_pIWSDZ->getAttributeAsLong (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsLong", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__long), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsLong (pName, pNamespace);
		#endif
	}

}

xsd__integer * Call::getAttributeAsInteger (const AxisChar* pName,
    const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__integer* traceRet = (m_pIWSDZ->getAttributeAsInteger (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__integer), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsInteger (pName, pNamespace);
		#endif
	}

}

xsd__unsignedLong * Call::getAttributeAsUnsignedLong (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsUnsignedLong", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedLong* traceRet = (m_pIWSDZ->getAttributeAsUnsignedLong (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsUnsignedLong", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedLong), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsUnsignedLong (pName, pNamespace);
		#endif
	}

}

xsd__float * Call::getAttributeAsFloat (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsFloat", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__float* traceRet = (m_pIWSDZ->getAttributeAsFloat (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsFloat", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__float), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsFloat (pName, pNamespace);
		#endif
	}

}

xsd__double * Call::getAttributeAsDouble (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsDouble", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__double* traceRet = (m_pIWSDZ->getAttributeAsDouble (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsDouble", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__double), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsDouble (pName, pNamespace);
		#endif
	}

}

xsd__decimal * Call::getAttributeAsDecimal (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsDecimal", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__decimal* traceRet = (m_pIWSDZ->getAttributeAsDecimal (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsDecimal", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__decimal), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsDecimal (pName, pNamespace);
		#endif
	}

}

xsd__string Call::getAttributeAsString (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsString", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__string traceRet = (m_pIWSDZ->getAttributeAsString (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsString", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsString (pName, pNamespace);
		#endif
	}

}

xsd__anyURI Call::getAttributeAsAnyURI (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsAnyURI", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__anyURI traceRet = (m_pIWSDZ->getAttributeAsAnyURI (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsAnyURI", this, 0,
					TRACETYPE_DATA, sizeof(xsd__anyURI), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsAnyURI (pName, pNamespace);
		#endif
	}

}

xsd__QName Call::getAttributeAsQName (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsQName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__QName traceRet = (m_pIWSDZ->getAttributeAsQName (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsQName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__QName), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsQName (pName, pNamespace);
		#endif
	}

}

xsd__hexBinary * Call::getAttributeAsHexBinary (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsHexBinary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__hexBinary* traceRet = (m_pIWSDZ->getAttributeAsHexBinary (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsHexBinary", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__hexBinary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsHexBinary (pName, pNamespace);
		#endif
	}

}

xsd__base64Binary * Call::getAttributeAsBase64Binary (const AxisChar* pName,
    const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsBase64Binary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__base64Binary* traceRet = (m_pIWSDZ->getAttributeAsBase64Binary (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsBase64Binary", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__base64Binary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsBase64Binary (pName, pNamespace);
		#endif
	}

}

xsd__dateTime * Call::getAttributeAsDateTime (const AxisChar* pName,
    const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsDateTime", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__dateTime* traceRet = (m_pIWSDZ->getAttributeAsDateTime (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsDateTime", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__dateTime), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsDateTime (pName, pNamespace);
		#endif
	}

}

xsd__date * Call::getAttributeAsDate (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsDate", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__date* traceRet = (m_pIWSDZ->getAttributeAsDate (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsDate", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__date), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsDate (pName, pNamespace);
		#endif
	}

}

xsd__time * Call::getAttributeAsTime (const AxisChar* pName, 
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsTime", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__time* traceRet = (m_pIWSDZ->getAttributeAsTime (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsTime", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__time), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsTime (pName, pNamespace);
		#endif
	}

}

xsd__duration * Call::getAttributeAsDuration (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsDuration", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__duration* traceRet = (m_pIWSDZ->getAttributeAsDuration (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsDuration", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__duration), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsDuration (pName, pNamespace);
		#endif
	}

}

xsd__gYearMonth * Call::getAttributeAsGYearMonth (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsGYearMonth", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gYearMonth* traceRet = (m_pIWSDZ->getAttributeAsGYearMonth (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsGYearMonth", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gYearMonth), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsGYearMonth (pName, pNamespace);
		#endif
	}

}

xsd__gYear * Call::getAttributeAsGYear (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsGYear", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gYear* traceRet = (m_pIWSDZ->getAttributeAsGYear (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsGYear", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gYear), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsGYear (pName, pNamespace);
		#endif
	}

}

xsd__gMonthDay * Call::getAttributeAsGMonthDay (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsGMonthDay", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gMonthDay* traceRet = (m_pIWSDZ->getAttributeAsGMonthDay (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsGMonthDay", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gMonthDay), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsGMonthDay (pName, pNamespace);
		#endif
	}

}

xsd__gDay * Call::getAttributeAsGDay (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsGDay", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gDay* traceRet = (m_pIWSDZ->getAttributeAsGDay (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsGDay", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gDay), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsGDay (pName, pNamespace);
		#endif
	}

}

xsd__gMonth * Call::getAttributeAsGMonth (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsGMonth", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gMonth* traceRet = (m_pIWSDZ->getAttributeAsGMonth (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsGMonth", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gMonth), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsGMonth (pName, pNamespace);
		#endif
	}

}

xsd__NOTATION Call::getAttributeAsNOTATION (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsNOTATION", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NOTATION traceRet = (m_pIWSDZ->getAttributeAsNOTATION (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsNOTATION", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NOTATION), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsNOTATION (pName, pNamespace);
		#endif
	}

}

xsd__normalizedString Call::getAttributeAsNormalizedString (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsNormalizedString", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__normalizedString traceRet = (m_pIWSDZ->getAttributeAsNormalizedString (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsNormalizedString", this, 0,
					TRACETYPE_DATA, sizeof(xsd__normalizedString), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsNormalizedString (pName, pNamespace);
		#endif
	}

}

xsd__token Call::getAttributeAsToken (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsToken", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__token traceRet = (m_pIWSDZ->getAttributeAsToken (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsToken", this, 0,
					TRACETYPE_DATA, sizeof(xsd__token), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsToken (pName, pNamespace);
		#endif
	}

}

xsd__language Call::getAttributeAsLanguage (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsLanguage", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__language traceRet = (m_pIWSDZ->getAttributeAsLanguage (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsLanguage", this, 0,
					TRACETYPE_DATA, sizeof(xsd__language), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsLanguage (pName, pNamespace);
		#endif
	}

}

xsd__Name Call::getAttributeAsName (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__Name traceRet = (m_pIWSDZ->getAttributeAsName (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__Name), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsName (pName, pNamespace);
		#endif
	}

}

xsd__NCName Call::getAttributeAsNCName (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsNCName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NCName traceRet = (m_pIWSDZ->getAttributeAsNCName (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsNCName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NCName), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsNCName (pName, pNamespace);
		#endif
	}

}

xsd__ID Call::getAttributeAsID (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsID", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ID traceRet = (m_pIWSDZ->getAttributeAsID (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsID", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ID), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsID (pName, pNamespace);
		#endif
	}

}

xsd__IDREF Call::getAttributeAsIDREF (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsIDREF", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__IDREF traceRet = (m_pIWSDZ->getAttributeAsIDREF (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsIDREF", this, 0,
					TRACETYPE_DATA, sizeof(xsd__IDREF), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsIDREF (pName, pNamespace);
		#endif
	}

}

xsd__IDREFS Call::getAttributeAsIDREFS (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsIDREFS", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__IDREFS traceRet = (m_pIWSDZ->getAttributeAsIDREFS (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsIDREFS", this, 0,
					TRACETYPE_DATA, sizeof(xsd__IDREFS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsIDREFS (pName, pNamespace);
		#endif
	}

}

xsd__ENTITY Call::getAttributeAsENTITY (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsENTITY", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ENTITY traceRet = (m_pIWSDZ->getAttributeAsENTITY (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsENTITY", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ENTITY), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsENTITY (pName, pNamespace);
		#endif
	}

}

xsd__ENTITIES Call::getAttributeAsENTITIES (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsENTITIES", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ENTITIES traceRet = (m_pIWSDZ->getAttributeAsENTITIES (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsENTITIES", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ENTITIES), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsENTITIES (pName, pNamespace);
		#endif
	}

}

xsd__NMTOKEN Call::getAttributeAsNMTOKEN (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsNMTOKEN", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NMTOKEN traceRet = (m_pIWSDZ->getAttributeAsNMTOKEN (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsNMTOKEN", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NMTOKEN), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsNMTOKEN (pName, pNamespace);
		#endif
	}

}

xsd__NMTOKENS Call::getAttributeAsNMTOKENS (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsNMTOKENS", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NMTOKENS traceRet = (m_pIWSDZ->getAttributeAsNMTOKENS (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsNMTOKENS", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NMTOKENS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsNMTOKENS (pName, pNamespace);
		#endif
	}

}

xsd__nonPositiveInteger * Call::getAttributeAsNonPositiveInteger (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsNonPositiveInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__nonPositiveInteger* traceRet = (m_pIWSDZ->getAttributeAsNonPositiveInteger (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsNonPositiveInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__nonPositiveInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsNonPositiveInteger (pName, pNamespace);
		#endif
	}

}

xsd__negativeInteger * Call::getAttributeAsNegativeInteger (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsNegativeInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__negativeInteger* traceRet = (m_pIWSDZ->getAttributeAsNegativeInteger (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsNegativeInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__negativeInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsNegativeInteger (pName, pNamespace);
		#endif
	}

}

xsd__nonNegativeInteger * Call::getAttributeAsNonNegativeInteger (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsNonNegativeInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__nonNegativeInteger* traceRet = (m_pIWSDZ->getAttributeAsNonNegativeInteger (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsNonNegativeInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__nonNegativeInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsNonNegativeInteger (pName, pNamespace);
		#endif
	}

}

xsd__positiveInteger * Call::getAttributeAsPositiveInteger (const AxisChar* pName,
    const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAttributeAsPositiveInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			xsd__positiveInteger* traceRet = (m_pIWSDZ->getAttributeAsPositiveInteger (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAttributeAsPositiveInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__positiveInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAttributeAsPositiveInteger (pName, pNamespace);
		#endif
	}

}

int Call::checkMessage (const AxisChar* pName, const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "checkMessage", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	 m_nStatus = m_pIWSDZ->checkMessageBody (pName, pNamespace);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "checkMessage", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

}

void* Call::checkFault (const AxisChar* pName, const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "checkFault", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

	 	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (m_pIWSDZ->checkForFault (pName, pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "checkFault", this, 0,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->checkForFault (pName, pNamespace);
		#endif
	}

}

void* Call::getCmplxObject (void* pDZFunct, void* pCreFunct, void* pDelFunct,
    const AxisChar* pName, const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getCmplxObject", this, 5,
					TRACETYPE_POINTER, 1, ((void*)&pDZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pCreFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (m_pIWSDZ->getCmplxObject (pDZFunct, pCreFunct, pDelFunct, pName,
        pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getCmplxObject", this, 0,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getCmplxObject (pDZFunct, pCreFunct, pDelFunct, pName,
        pNamespace);
		#endif
	}

}

IHeaderBlock* Call::createHeaderBlock ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "createHeaderBlock", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = ((m_pIWSSZ->createHeaderBlock ()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "createHeaderBlock", this, 0,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (m_pIWSSZ->createHeaderBlock ());
		#endif
	}

}

IHeaderBlock* Call::createHeaderBlock(AxisChar* pachLocalName, 
                                       AxisChar* pachUri)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "createHeaderBlock", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pachLocalName),
					TRACETYPE_STRING, 0, ((void*)&pachUri));	  /* AUTOINSERTED TRACE */
	#endif

 		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (new HeaderBlock(pachLocalName, pachUri));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "createHeaderBlock", this, 0,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return new HeaderBlock(pachLocalName, pachUri);
		#endif
	}

}

IHeaderBlock* Call::createHeaderBlock(AxisChar* pachLocalName, 
                                       AxisChar* pachUri, AxisChar* pachPrefix)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "createHeaderBlock", this, 3,
					TRACETYPE_STRING, 0, ((void*)&pachLocalName),
					TRACETYPE_STRING, 0, ((void*)&pachUri),
					TRACETYPE_STRING, 0, ((void*)&pachPrefix));	  /* AUTOINSERTED TRACE */
	#endif

 		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (new HeaderBlock(pachLocalName, pachUri, pachPrefix));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "createHeaderBlock", this, 0,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return new HeaderBlock(pachLocalName, pachUri, pachPrefix);
		#endif
	}

}

int Call::getStatus() 
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getStatus", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getStatus", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

}

void Call::setProxy(const char* pcProxyHost, unsigned int uiProxyPort)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "setProxy", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcProxyHost),
					TRACETYPE_UINT, 0, ((void*)&uiProxyPort));	  /* AUTOINSERTED TRACE */
	#endif

    m_strProxyHost = pcProxyHost;
    m_uiProxyPort = uiProxyPort;
    m_bUseProxy = true;    
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "setProxy", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AnyType* Call::getAnyObject()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getAnyObject", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			AnyType* traceRet = (m_pIWSDZ->getAnyObject());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getAnyObject", this, 0,
					TRACETYPE_POINTER, sizeof(AnyType), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getAnyObject();
		#endif
	}

}

int Call::addAnyObject(AnyType* pAnyObject)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "addAnyObject", this, 1,
					TRACETYPE_POINTER, sizeof(AnyType), ((void*)&pAnyObject));	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_pIWSSZ->addOutputAnyObject(pAnyObject));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "addAnyObject", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSSZ->addOutputAnyObject(pAnyObject);
		#endif
	}

}

const AxisChar* Call::getNamespacePrefix(const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getNamespacePrefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const AxisChar* traceRet = (m_pIWSSZ->getNamespacePrefix(pNamespace));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getNamespacePrefix", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSSZ->getNamespacePrefix(pNamespace);
		#endif
	}

}

void Call::setSOAPMethodAttribute(const AxisChar *pLocalname, const AxisChar *pPrefix, const AxisChar *pValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "setSOAPMethodAttribute", this, 3,
					TRACETYPE_STRING, 0, ((void*)&pLocalname),
					TRACETYPE_STRING, 0, ((void*)&pPrefix),
					TRACETYPE_STRING, 0, ((void*)&pValue));	  /* AUTOINSERTED TRACE */
	#endif

    setSOAPMethodAttribute(pLocalname,pPrefix,NULL,pValue);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "setSOAPMethodAttribute", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void Call::setSOAPMethodAttribute(const AxisChar *pLocalname, const AxisChar *pPrefix, 
                                  const AxisChar *pUri, const AxisChar *pValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "setSOAPMethodAttribute", this, 4,
					TRACETYPE_STRING, 0, ((void*)&pLocalname),
					TRACETYPE_STRING, 0, ((void*)&pPrefix),
					TRACETYPE_STRING, 0, ((void*)&pUri),
					TRACETYPE_STRING, 0, ((void*)&pValue));	  /* AUTOINSERTED TRACE */
	#endif

    IAttribute* pAttribute;
	if (pValue==NULL) pValue = "";
    if (NULL!=pUri)
    {
        std::list<Attribute*> attributeList;
        pAttribute = new Attribute(attributeList, pLocalname, pPrefix, pUri, pValue);
    }
    else
    {
        std::list<Attribute*> attributeList;
        pAttribute = new Attribute(attributeList, pLocalname, pPrefix, pValue);
    }
    m_pIWSSZ->setSOAPMethodAttribute(((Attribute*)pAttribute)->clone());
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "setSOAPMethodAttribute", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const xsd__string Call::getFaultAsXMLString()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "getFaultAsXMLString", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			const xsd__string traceRet = (m_pIWSDZ->getFaultAsXMLString());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "getFaultAsXMLString", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pIWSDZ->getFaultAsXMLString();
		#endif
	}

}

void Call::addAttachment(ISoapAttachment* att)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "addAttachment", this, 1,
					TRACETYPE_POINTER, sizeof(ISoapAttachment), ((void*)&att));	  /* AUTOINSERTED TRACE */
	#endif

	if (NULL==m_pIWSSZ)
		m_attachments.push_back(att);
	else
		m_pIWSSZ->addAttachment(att->getHeader(AXIS_CONTENT_ID),att);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "addAttachment", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

ISoapAttachment* Call::createSoapAttachment()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "createSoapAttachment", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			ISoapAttachment* traceRet = (new SoapAttachment(m_pContentIdSet));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "createSoapAttachment", this, 0,
					TRACETYPE_POINTER, sizeof(ISoapAttachment), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return new SoapAttachment(m_pContentIdSet);
		#endif
	}

}

void Call::addAttachmentParameter(ISoapAttachment* att, const char* pName, IAttribute **attributes, int nAttributes)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "addAttachmentParameter", this, 4,
					TRACETYPE_POINTER, sizeof(ISoapAttachment), ((void*)&att),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_POINTER, sizeof(IAttribute*), ((void*)&attributes),
					TRACETYPE_INT, 0, ((void*)&nAttributes));	  /* AUTOINSERTED TRACE */
	#endif

	m_pIWSSZ->addAttachmentParameter(att,pName,attributes,nAttributes);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "addAttachmentParameter", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

IAttribute* Call::createAttribute(const AxisChar *pLocalname, const AxisChar *pPrefix, const AxisChar *pValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Call", "createAttribute", this, 3,
					TRACETYPE_STRING, 0, ((void*)&pLocalname),
					TRACETYPE_STRING, 0, ((void*)&pPrefix),
					TRACETYPE_STRING, 0, ((void*)&pValue));	  /* AUTOINSERTED TRACE */
	#endif

    std::list<Attribute*> attributeList;
    	{
		#ifdef ENABLE_AXISTRACE
			IAttribute* traceRet = (new Attribute(attributeList, pLocalname, pPrefix, pValue));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Call", "createAttribute", this, 0,
					TRACETYPE_POINTER, sizeof(IAttribute), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return new Attribute(attributeList, pLocalname, pPrefix, pValue);
		#endif
	}

}

