#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 * @author Dasarath Weerathunga (dasarath@opensource.lk)
 * @author Dinesh Premalal (xydinesh@gmail.com
                            premalal@opensource.lk)
 *
  *@author  Susantha Kumara  ( susantha@opensource.lk )
 */

#include "XmlPullParser.hpp"

void
XmlPullParser::relocateTokens (int offset)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "relocateTokens", this, 1,
					TRACETYPE_INT, 0, ((void*)&offset));	  /* AUTOINSERTED TRACE */
	#endif

  int size = token->count ();
  for (int i = 0; i < size; i++)
    token->get (i)->relocate (offset);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "relocateTokens", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
XmlPullParser::shift ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "shift", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  memmove (buffer->get (), buffer->get () + offset, _next - offset);
  _next -= offset;
  last -= offset;
  offset = 0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "shift", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int
XmlPullParser::read ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "read", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  if (_next == buffer->getSize ())
    {
      if (offset > 0)
	{
	  relocateTokens (offset);
	  shift ();
	}
      else
	{
	  char *p = buffer->get ();
	  buffer->grow ();
	  relocateTokens (p - buffer->get ());
	}
    }
  int c = s->read (buffer->get (), _next, buffer->getSize () - _next);
  last += c;
  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (!c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "read", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return !c;
		#endif
	}

}

/*! check with BOM
   *This may or may not be presant with utf8 documents 
   *But UTF-16 document must have BOM 0xff 0xfe in any order, 
   * for LE = 0xff 0xfe, 
   * for BE = 0xfe 0xff,  
   * for UTF-8 = 0xef 0xbb 0xbf ( not necessary) */

int
XmlPullParser::isUtf8 ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "isUtf8", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  UTF8_char utf[3] = { 0, 0, 0 };
  for (int i = 0; i < 3; i++)
    utf[i] = (UTF8_char) buffer->get ()[_next++];
  
  if (0xef == utf[0] && 0xbb == utf[1] && 0xbf == utf[2])
    {
      unicodeState = None;
      	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (_next);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "isUtf8", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return _next;
		#endif
	}

    }
  else
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (_next = 0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "isUtf8", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return _next = 0;
		#endif
	}

}

/*! check for BOM */

int
XmlPullParser::isUtf16 ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "isUtf16", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  if (_next == 0)
    {
      UTF8_char utf[2];
      int i;
      for (i = 0; i < 2; i++)
	utf[i] = (UTF8_char) buffer->get ()[_next++];
      if (0xff == utf[0] && 0xfe == utf[1])
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (LE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "isUtf16", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return LE;
		#endif
	}

      else if (0xfe == utf[0] && 0xff == utf[1])
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (BE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "isUtf16", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return BE;
		#endif
	}

      else
	{
	  _next = 0;
	  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (None);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "isUtf16", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return None;
		#endif
	}

	}
    }
  else
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (None);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "isUtf16", this, 4,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return None;
		#endif
	}

}

/*!since we are interested in special charactes such as "<",
 *  "/",">","?","!",""",... nextCh() method suppose to return those 
 * values. Those values are retrieved form UTF-16 characters
 * ,Here we need not to do a convertion. Because those values are
 * < 128 */

int
XmlPullParser::fromUtf16 (int eof)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "fromUtf16", this, 1,
					TRACETYPE_INT, 0, ((void*)&eof));	  /* AUTOINSERTED TRACE */
	#endif

  int c = buffer->get ()[_next++];
  
  if (_next > last && read ())
    if (eof)
      {
	term = 1;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "fromUtf16", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

      }
    else
      throw new XmlPullParserException ();
  /*UTF16_char d = *((UTF16_char *) & buffer->get ()[_next]);
  if (!d)
  {
  term = 1;
  return -1;
  }*/
  while (!c)
    c = buffer->get ()[_next++];
  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "fromUtf16", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c;
		#endif
	}

}

int
XmlPullParser::nextCh (int eof)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "nextCh", this, 1,
					TRACETYPE_INT, 0, ((void*)&eof));	  /* AUTOINSERTED TRACE */
	#endif

  if (_next > last && read ())
    if (eof)
      {
	term = 1;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "nextCh", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

      }
    else
      throw new XmlPullParserException ();
  if (_next == 0)
    {
      if (isUtf8 ());
      else
	(unicodeState = isUtf16 ());
    }
  if (unicodeState == None)
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (buffer->get ()[_next++]);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "nextCh", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return buffer->get ()[_next++];
		#endif
	}

  else
      	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (fromUtf16 (eof));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "nextCh", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return fromUtf16 (eof);
		#endif
	}

}

char *
XmlPullParser::lastCh ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "lastCh", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  int i = 1;
  char c = buffer->get ()[_next - 1];
  /*! If term has set to true then that is a buffer termination */
  if (term)
    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (buffer->get () + _next - 1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "lastCh", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return buffer->get () + _next - 1;
		#endif
	}

  else
    {
      while (!c)
	c = buffer->get ()[_next - (++i)];
      	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (buffer->get () + _next - i);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "lastCh", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return buffer->get () + _next - i;
		#endif
	}

    }
}

void
XmlPullParser::reset (int del)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "reset", this, 1,
					TRACETYPE_INT, 0, ((void*)&del));	  /* AUTOINSERTED TRACE */
	#endif

  offset = _next;
  name = NULL;
  prefix = NULL;
  namesp = NULL;
  term = 0;
  token->trunc (0);
  attr->trunc (0);
  if (!del)
    {
      if (ETag == event || EmptyElemTag == event)
	closeElement ();
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "reset", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
XmlPullParser::openToken ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "openToken", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  token_t *t = token->append ();
  t->type = token_t::Unknown;
  t->start = lastCh ();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "openToken", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
XmlPullParser::closeToken (int type)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "closeToken", this, 1,
					TRACETYPE_INT, 0, ((void*)&type));	  /* AUTOINSERTED TRACE */
	#endif

  token_t *t = token->last ();
  t->type = type;
  t->end = lastCh () - 1;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "closeToken", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

XmlPullParser::XmlPullParser (InputStream * s)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "XmlPullParser", this, 1,
					TRACETYPE_POINTER, sizeof(InputStream), ((void*)&s));	  /* AUTOINSERTED TRACE */
	#endif

  this->s = s;
  state = S_1;
  offset = 0;
  last = -1;
  _next = 0;
  buffer = new Buffer < char >(1024);
  token = new Array < token_t > (64);
  namespa = new Array < namespace_t > (64);
  depth = new Array < depth_t > (16);
  attr = new Array < attr_t > (64);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "XmlPullParser", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

XmlPullParser::~XmlPullParser ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "~XmlPullParser", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  reset (-1);
  delete buffer;
  delete token;
  delete namespa;
  delete depth;
  delete attr;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "~XmlPullParser", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int
XmlPullParser::isS (int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "isS", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0x9 == c || 0xA == c || 0xD == c || 0x20 == c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "isS", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0x9 == c || 0xA == c || 0xD == c || 0x20 == c;
		#endif
	}

}

int
XmlPullParser::parseS (int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseS", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

  while (isS (c))
    c = nextCh ();
  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseS", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c;
		#endif
	}

}

int
XmlPullParser::parseEq (int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseEq", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

  if ('=' == parseS (c))
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (parseS (nextCh ()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseEq", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return parseS (nextCh ());
		#endif
	}

  else
    throw new XmlPullParserException ();
}

int
XmlPullParser::parseNameToken (int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseNameToken", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

  openToken ();
  while (!(isS (c) || '/' == c || '?' == c || '=' == c || '>' == c))
    {
      if (':' == c)
	{
	  closeToken (token_t::Prefix);
	  c = nextCh ();
	  openToken ();
	}
      else
	c = nextCh ();
    }
  closeToken (token_t::Name);
  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseNameToken", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c;
		#endif
	}

}

int
XmlPullParser::parseAttValueToken (int quote)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseAttValueToken", this, 1,
					TRACETYPE_INT, 0, ((void*)&quote));	  /* AUTOINSERTED TRACE */
	#endif

  if ('\'' == quote || '"' == quote)
    {
      int c = nextCh ();
      openToken ();
      while ('&' != c && '<' != c)
	{
	  if (c == quote)
	    {
	      closeToken (token_t::AttValue);
	      	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (nextCh ());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseAttValueToken", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return nextCh ();
		#endif
	}

	    }
	  c = nextCh ();
	}
    }
  throw new XmlPullParserException ();
}

int
XmlPullParser::parseAttribute (int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseAttribute", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

  c = parseEq (parseNameToken (c));
  c = parseAttValueToken (c);
  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (parseS (c));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseAttribute", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return parseS (c);
		#endif
	}

}

void
XmlPullParser::parseXMLDecl ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseXMLDecl", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  if (nextCh () == 'x' && nextCh () == 'm' && nextCh () == 'l')
    {
      int c = parseAttribute (parseS (nextCh ()));
      if (c == 'e')
	c = parseAttribute (c);
      if (c == 's')
	c = parseAttribute (c);
      if (c == '?' && nextCh () == '>')
	{
	  event = XMLDecl;
	  	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseXMLDecl", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

	}
    }
  throw new XmlPullParserException ();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseXMLDecl", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
XmlPullParser::parseSTagOrEmptyElemTag (int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseSTagOrEmptyElemTag", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

  c = parseS (parseNameToken (c));
  event = STag;
  for (;;)
    {
      if (c == '/')
	{
	  event = EmptyElemTag;
	  if (nextCh () == '>')
	    	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseSTagOrEmptyElemTag", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

	  else
	    throw new XmlPullParserException ();
	}
      else if (c == '>')
		{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseSTagOrEmptyElemTag", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

      else
	c = parseAttribute (c);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseSTagOrEmptyElemTag", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
XmlPullParser::parseETag ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseETag", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  if (parseS (parseNameToken (nextCh ())) == '>')
    event = ETag;
  else
    throw new XmlPullParserException ();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseETag", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int
XmlPullParser::parseCharData ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseCharData", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  int c;
  event = Content;
  openToken ();
  do
    {
      c = nextCh (-1);
      if (-1 == c)
	{
	  closeToken (token_t::CharData);
	  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseCharData", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

	}
    }
  while ('<' != c);
  closeToken (token_t::CharData);
  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseCharData", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c;
		#endif
	}

}

int
XmlPullParser::tokenize ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "tokenize", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  do
    {
      int c = nextCh (-1);
      if (-1 == c)
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "tokenize", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

      switch (state)
	{
	case S_1:
	  if ('<' == c)
	    state = S_2;
	  else
	    state = S_0;
	  break;
	case S_2:
	  if ('?' == c)
	    parseXMLDecl ();
	  else
	    parseSTagOrEmptyElemTag (c);
	  state = S_3;
	  break;
	case S_3:
	  if ('<' == c)
	    state = S_4;
	  else
	    {
	      c = parseCharData ();
	      if ('<' == c)
		{
		  state = S_4;
		  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (event);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "tokenize", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return event;
		#endif
	}

		}
	      else if ('\0' == c)
		state = S_3;
	      else
		state = S_0;
	    }
	  break;
	case S_4:
	  if ('/' == c)
	    {
	      parseETag ();
	      state = S_3;
	    }
	  else if ('!' == c)
	    state = S_0;
	  else if ('?' == c)
	    state = S_0;
	  else
	    {
	      parseSTagOrEmptyElemTag (c);
	      state = S_3;
	    }
	}
      if (state == S_0)
	throw new XmlPullParserException ();
    }
  while (state != S_3);
  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (event);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "tokenize", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return event;
		#endif
	}

}

int
XmlPullParser::addNamespace (token_t * uri)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "addNamespace", this, 1,
					TRACETYPE_POINTER, sizeof(token_t), ((void*)&uri));	  /* AUTOINSERTED TRACE */
	#endif

  namespace_t *r = namespa->append ();
  r->prefix = NULL;
  r->length = 0;
  r->uri = uri->toString (unicodeState);
  r->lengthUri = strlen(r->uri);
}

int
XmlPullParser::addNamespace (token_t * prefix, token_t * uri)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "addNamespace", this, 2,
					TRACETYPE_POINTER, sizeof(token_t), ((void*)&prefix),
					TRACETYPE_POINTER, sizeof(token_t), ((void*)&uri));	  /* AUTOINSERTED TRACE */
	#endif

  namespace_t *r = namespa->append ();
  r->prefix = prefix->toString (unicodeState);
  r->length = strlen (r->prefix);
  r->uri = uri->toString (unicodeState);
  r->lengthUri = strlen(r->uri);
}

int
XmlPullParser::addAttribute (token_t * prefix, token_t * name,
			     token_t * value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "addAttribute", this, 3,
					TRACETYPE_POINTER, sizeof(token_t), ((void*)&prefix),
					TRACETYPE_POINTER, sizeof(token_t), ((void*)&name),
					TRACETYPE_POINTER, sizeof(token_t), ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

  attr_t *r = attr->append ();
  r->prefix = prefix;
  r->name = name;
  r->value = value;
  r->namesp = NULL;
}

void
XmlPullParser::openElement ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "openElement", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  if (depth->count ())
    {
      depth_t *l = depth->last ();
      depth_t *m = depth->append ();
      m->first = l->first + l->count;
      m->total = namespa->count ();
      m->count = m->total - l->total;
    }
  else
    {
      depth_t *m = depth->append ();
      m->first = 0;
      m->total = namespa->count ();
      m->count = m->total;
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "openElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
XmlPullParser::closeElement ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "closeElement", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  depth_t *d = depth->last ();
  if (d->count)
    {
      namespace_t *p = namespa->get (d->first);
      for (int i = 0; i < d->count; i++, p++)
	{
	  if (p->prefix)
	    free (p->prefix);
	  if(p->uri)
	    free(p->uri);
	}
      namespa->trunc (d->first);
    }
  depth->chip ();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "closeElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int
XmlPullParser::next ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "next", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  token_t *p;
  reset ();
  switch (tokenize ())
    {
    case -1:
      	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "next", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

    case XMLDecl:
    case STag:
    case EmptyElemTag:
      p = token->last ();
      while (p->isOfType (token_t::AttValue))
	{
	  token_t *value = p--;
	  token_t *name = p--;
	  if (p->isOfType (token_t::Prefix))
	    {
	      if (!p->compare ("xmlns", 5))
		addNamespace (name, value);
	      else
		addAttribute (p, name, value);
	      p--;
	    }
	  else if (!name->compare ("xmlns", 5))
	    addNamespace (value);
	  else
	    addAttribute (NULL, name, value);
	}
      openElement ();
    case ETag:
      p = token->get ();
      if (p->isOfType (token_t::Prefix))
	prefix = p++;
      name = p;
    case Content:
    defualt:
      	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (event);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "next", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return event;
		#endif
	}

    }
}

namespace_t *
XmlPullParser::getNamespace (token_t * prefix)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespace", this, 1,
					TRACETYPE_POINTER, sizeof(token_t), ((void*)&prefix));	  /* AUTOINSERTED TRACE */
	#endif

  namespace_t *p = namespa->last (), *q = namespa->get ();
  while (1)
    {
      if (prefix == NULL)
	{
	  if (p->prefix == NULL)
	    	{
		#ifdef ENABLE_AXISTRACE
			namespace_t* traceRet = (p);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespace", this, 1,
					TRACETYPE_POINTER, sizeof(namespace_t), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return p;
		#endif
	}

	}
      else
	{
	  if (p->prefix && !prefix->compare (p->prefix, p->length))
	    	{
		#ifdef ENABLE_AXISTRACE
			namespace_t* traceRet = (p);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespace", this, 2,
					TRACETYPE_POINTER, sizeof(namespace_t), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return p;
		#endif
	}

	}
      if (p == q)
	throw new XmlPullParserException ();
      p--;
    }
}

char *
XmlPullParser::getName ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  if (name)
    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (name->toString (unicodeState));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getName", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return name->toString (unicodeState);
		#endif
	}

  throw new XmlPullParserException ();
}
 
char *
XmlPullParser::getPrefix ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getPrefix", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  if (prefix)
    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (prefix->toString (unicodeState));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getPrefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return prefix->toString (unicodeState);
		#endif
	}

  	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getPrefix", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

char*
XmlPullParser::getNamespaceUri ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespaceUri", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  if (!namesp)
    namesp = getNamespace (prefix);
  	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (strndup (namesp->uri, namesp->lengthUri));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespaceUri", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strndup (namesp->uri, namesp->lengthUri);
		#endif
	}

}

char*
XmlPullParser::getValue ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getValue", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (token->get ()->toString (unicodeState));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getValue", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return token->get ()->toString (unicodeState);
		#endif
	}

}

int
XmlPullParser::getDepth ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getDepth", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (depth->count ());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getDepth", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return depth->count ();
		#endif
	}

}

int
XmlPullParser::getAttributeCount ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getAttributeCount", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (attr->count ());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributeCount", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return attr->count ();
		#endif
	}

}

char*
XmlPullParser::getAttributeName (int i)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getAttributeName", this, 1,
					TRACETYPE_INT, 0, ((void*)&i));	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (attr->get (i)->name->toString (unicodeState));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributeName", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return attr->get (i)->name->toString (unicodeState);
		#endif
	}

}

char*
XmlPullParser::getAttributeValue (int i)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getAttributeValue", this, 1,
					TRACETYPE_INT, 0, ((void*)&i));	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (attr->get (i)->value->toString (unicodeState));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributeValue", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return attr->get (i)->value->toString (unicodeState);
		#endif
	}

}

char*
XmlPullParser::getAttributePrefix (int i)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getAttributePrefix", this, 1,
					TRACETYPE_INT, 0, ((void*)&i));	  /* AUTOINSERTED TRACE */
	#endif

  attr_t *p = attr->get (i);
  if (p->prefix)
    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (p->prefix->toString (unicodeState));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributePrefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return p->prefix->toString (unicodeState);
		#endif
	}

  	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributePrefix", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

char*
XmlPullParser::getAttributeNamespaceUri (int i)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getAttributeNamespaceUri", this, 1,
					TRACETYPE_INT, 0, ((void*)&i));	  /* AUTOINSERTED TRACE */
	#endif

  attr_t *p = attr->get (i);
  if (!p->namesp)
    p->namesp = getNamespace (p->prefix);
  	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (strndup(p->namesp->uri, p->namesp->lengthUri));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributeNamespaceUri", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strndup(p->namesp->uri, p->namesp->lengthUri);
		#endif
	}

}

int
XmlPullParser::getNamespaceCount (int i)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespaceCount", this, 1,
					TRACETYPE_INT, 0, ((void*)&i));	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (depth->get (i - 1)->total);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespaceCount", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return depth->get (i - 1)->total;
		#endif
	}

}

char *
XmlPullParser::getNamespacePrefix (int i)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespacePrefix", this, 1,
					TRACETYPE_INT, 0, ((void*)&i));	  /* AUTOINSERTED TRACE */
	#endif

  namespace_t *p = namespa->get (i);
  if (p->prefix)
   		{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (strndup(p->prefix, p->length));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespacePrefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strndup(p->prefix, p->length);
		#endif
	}

  	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespacePrefix", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

char *
XmlPullParser::getNamespaceUri (int i)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespaceUri", this, 1,
					TRACETYPE_INT, 0, ((void*)&i));	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (strndup(namespa->get(i)->uri, namespa->get(i)->lengthUri));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespaceUri", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strndup(namespa->get(i)->uri, namespa->get(i)->lengthUri);
		#endif
	}

}

char *
XmlPullParser::getNamespaceForPrefix (char *prefix)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespaceForPrefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&prefix));	  /* AUTOINSERTED TRACE */
	#endif

  namespace_t *p = namespa->last (), *q = namespa->get ();
  while (1)
    {
      if (prefix == NULL)
	{
	  if (p->prefix == NULL)
	    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (strndup (p->uri, p->lengthUri));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespaceForPrefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strndup (p->uri, p->lengthUri);
		#endif
	}

	}
      else 
	{
	  if (p->prefix && !strncmp (p->prefix, prefix, p->length))
	    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (strndup (p->uri, p->lengthUri));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespaceForPrefix", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strndup (p->uri, p->lengthUri);
		#endif
	}

	}
      if (p == q)
	throw new XmlPullParserException ();
      p--;
    }
}

char *
XmlPullParser::getPrefixForNamespace (char *ns)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getPrefixForNamespace", this, 1,
					TRACETYPE_STRING, 0, ((void*)&ns));	  /* AUTOINSERTED TRACE */
	#endif

  namespace_t *p = namespa->last (), *q = namespa->get ();
  while (1)
    {
      if (ns == NULL)
	throw new XmlPullParserException ();
      /* No Namespace exsist with NULL URI*/
      else 
	{
	  if (p->uri && !strncmp (p->uri, ns, p->lengthUri))
	    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (strndup (p->prefix, p->length));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getPrefixForNamespace", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strndup (p->prefix, p->length);
		#endif
	}

	}
      if (p == q)
	throw new XmlPullParserException ();
      p--;
    }
}
      
