#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Susantha Kumara (susantha@opensource.lk, skumara@virtusa.com)
 * @author Samisa Abeysinghe (sabeysinghe@virtusa.com)
 */

/*
 * Revision 1.1  2004/06/08 samisa
 * Added setTimeout
 */

#if !defined(AXIS_APACHE2TRANSPORT_H__OF_AXIS_INCLUDED_)

#define AXIS_APACHE2TRANSPORT_H__OF_AXIS_INCLUDED_

#include <string>

#ifdef WIN32

#include <apache2_0/httpd.h>

#include <apache2_0/http_config.h>

#include <apache2_0/http_core.h>

#include <apache2_0/http_protocol.h>

#include <apache2_0/http_log.h>

#include <apache2_0/ap_compat.h>

#include <apache2_0/apr_tables.h>

#include "../../transport/SOAPTransport.h"

#else

#include <httpd.h>

#include <http_config.h>

#include <http_core.h>

#include <http_protocol.h>

#include <http_log.h>

#include <ap_compat.h>

#include <apr_tables.h>

#include "../../transport/SOAPTransport.h"

//#include <string.h>

#endif

#include "AttachmentHelper.h"

#include <axis/ISoapAttachment.hpp>

AXIS_CPP_NAMESPACE_USE

class Apache2Transport : public SOAPTransport  
{

public:
	char* getIncomingSOAPMimeHeaders();
	Apache2Transport(void* pContext);
	virtual ~Apache2Transport();
    int openConnection(){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "openConnection", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "openConnection", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}
};
    void closeConnection(){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "closeConnection", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "closeConnection", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
};
    AXIS_TRANSPORT_STATUS sendBytes(const char* pcSendBuffer, const void* pBufferId);
	void registerReleaseBufferCallback(AXIS_ENGINE_CALLBACK_RELEASE_SEND_BUFFER pFunct)
	{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "registerReleaseBufferCallback", NULL, 1,
					TRACETYPE_DATA, sizeof(AXIS_ENGINE_CALLBACK_RELEASE_SEND_BUFFER), ((void*)&pFunct));	  /* AUTOINSERTED TRACE */
	#endif
 m_pReleaseBufferCallback = pFunct; 	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "registerReleaseBufferCallback", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
};
    AXIS_TRANSPORT_STATUS getBytes(char* pBuffer, int* piSize);
	void releaseBuffer(const char* pBuffer);
    int setTransportProperty(AXIS_TRANSPORT_INFORMATION_TYPE type, const char* value);
    const char* getTransportProperty(AXIS_TRANSPORT_INFORMATION_TYPE eType);
    int setTransportProperty(const char* pcKey, const char* pcValue);
    const char* getTransportProperty(const char* pcKey, bool response=true);
	void setAttachment(const char* pcAttachmentid, const char* pcAttachment){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "setAttachment", NULL, 2,
					TRACETYPE_STRING, 0, ((void*)&pcAttachmentid),
					TRACETYPE_STRING, 0, ((void*)&pcAttachment));	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "setAttachment", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
};
	ISoapAttachment* getAttachment(const char* pcAttachmentid);
    ISoapAttachment** getAllAttachments(int *pAttchArraySize);
	void setEndpointUri(const char* pcEndpointUri)
	{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "setEndpointUri", NULL, 1,
					TRACETYPE_STRING, 0, ((void*)&pcEndpointUri));	  /* AUTOINSERTED TRACE */
	#endif
m_pcEndpointUri = new char[strlen(pcEndpointUri)+1]; strcpy(m_pcEndpointUri,pcEndpointUri);	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "setEndpointUri", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
};
	void setSessionId(const char* pcSessionId);
	const char* getSessionId();
	const char* getServiceName();
	AXIS_PROTOCOL_TYPE getProtocol();
    int setProtocol(AXIS_PROTOCOL_TYPE eProtocol);
	int getSubProtocol();
	AXIS_TRANSPORT_STATUS flushOutput();
  /**
    * Set transport timeout.
    * TODO: need to implement
    *
    * @param lSeconds Timeout in seconds
    */
    void setTimeout(long lSeconds) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "setTimeout", NULL, 1,
					TRACETYPE_LONG, 0, ((void*)&lSeconds));	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "setTimeout", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
};

private:
	AttachmentHelper* pAttachmentHelper;
	void* m_pContext;

#ifndef CHUNCKED_DATA_SUPPORTED
    BufferInfo* m_pBuffers;

#endif
	
};

#endif 

