#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 * @author Sanjaya Singharage (sanjayas@opensource.lk, sanjayas@jkcsworld.com)
 * @author Susantha Kumara (susantha@opensource.lk, skumara@virtusa.com)
 */

#include "Apache2Transport.h"

#include <iostream>

#include <axis/AxisException.hpp> 

#define xxx ap_log_rerror(APLOG_MARK,APLOG_ERR, globr,"logged here");

#define yyy ap_log_rerror(APLOG_MARK, APLOG_ERR, globr,"logged here");

/* file: mod_axis2.cpp */

/* here's the content handler */

extern "C" int initialize_module (int bServer);

extern "C" int uninitialize_module ();

extern int process_request(SOAPTransport* str);

extern unsigned char chEBuf[1024];

int axis_handler_helper(request_rec* req_rec)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axis_handler_helper", NULL, 1,
					TRACETYPE_POINTER, sizeof(request_rec), ((void*)&req_rec));	  /* AUTOINSERTED TRACE */
	#endif

    int rc;
    SOAPTransport* pTransport;
	if ((!req_rec->handler) || strcmp(req_rec->handler, "axis") != 0)
    {
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (DECLINED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axis_handler_helper", NULL, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return DECLINED;
		#endif
	}

    }
    pTransport = new Apache2Transport((void*)req_rec);
    /* for SOAP 1.2 this this should be "application/soap+xml" but keep this for
     * the moment. */
    req_rec->content_type = (M_POST == req_rec->method_number) ? 
		"text/xml" : "text/html";
    // Samisa : if it is a GET request with wsdl arg then content type should be text/xml
    if (req_rec->args)
    {
        if( strcmp("wsdl", req_rec->args) == 0 )
            req_rec->content_type = "text/xml";
    }
	 /* Set up the read policy from the client.*/
    if ((rc = ap_setup_client_block(req_rec, REQUEST_CHUNKED_ERROR)) != OK)
    {
		delete pTransport;
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (rc);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axis_handler_helper", NULL, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return rc;
		#endif
	}

    }
    /* Tell the client that we are ready to receive content and check whether 
     * client will send content. control will pass to this block only if there is
     * body content in the request 
     */
    ap_should_client_block(req_rec);
    if (0 != process_request(pTransport))
    {
		delete pTransport;
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (OK);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axis_handler_helper", NULL, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return OK;
		#endif
	}

    }
	delete pTransport;
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (OK);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axis_handler_helper", NULL, 4,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return OK;
		#endif
	}
	
}

extern "C"
{ 
static apr_status_t module_exit(void *data)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "module_exit", NULL, 1,
					TRACETYPE_POINTER, 1, ((void*)&data));	  /* AUTOINSERTED TRACE */
	#endif

    uninitialize_module();
    	{
		#ifdef ENABLE_AXISTRACE
			apr_status_t traceRet = (APR_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "module_exit", NULL, 0,
					TRACETYPE_DATA, sizeof(apr_status_t), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return APR_SUCCESS;
		#endif
	}

}

static void module_init(apr_pool_t* p, server_rec* svr_rec)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "module_init", NULL, 2,
					TRACETYPE_POINTER, sizeof(apr_pool_t), ((void*)&p),
					TRACETYPE_POINTER, sizeof(server_rec), ((void*)&svr_rec));	  /* AUTOINSERTED TRACE */
	#endif

   try
   {
		apr_pool_cleanup_register(p, NULL, module_exit, apr_pool_cleanup_null);
		initialize_module(1);
   }
   catch(AxisException& e) 
   {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch(NULL, "module_init", NULL, 1,
					TRACETYPE_AXISEXCEPTION, sizeof(AxisException&), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

		std::cerr << "An error occurred while Axis was initializing: " << e.what() << std::endl;
   }
   catch(...) 
   {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch(NULL, "module_init", NULL, 2 );	  /* AUTOINSERTED TRACE */
	#endif

		std::cerr << "An unkown error occurred while Axis was initializing: " << std::endl;
   }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "module_init", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

static int axis_handler(request_rec* req_rec)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axis_handler", NULL, 1,
					TRACETYPE_POINTER, sizeof(request_rec), ((void*)&req_rec));	  /* AUTOINSERTED TRACE */
	#endif

	try
	{
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (axis_handler_helper(req_rec));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axis_handler", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return axis_handler_helper(req_rec);
		#endif
	}

	}
	catch(AxisException& e) 
	{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch(NULL, "axis_handler", NULL, 1,
					TRACETYPE_AXISEXCEPTION, sizeof(AxisException&), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

		std::cerr << "An error occurred while Axis was initializing: " << e.what() << std::endl;
	}
	catch(...) 
	{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch(NULL, "axis_handler", NULL, 2 );	  /* AUTOINSERTED TRACE */
	#endif

		std::cerr << "An unkown error occurred while Axis was initializing: " << std::endl;
	}
}

static void mod_axis_register_hooks(apr_pool_t* p)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "mod_axis_register_hooks", NULL, 1,
					TRACETYPE_POINTER, sizeof(apr_pool_t), ((void*)&p));	  /* AUTOINSERTED TRACE */
	#endif

    ap_hook_child_init(module_init, NULL, NULL, APR_HOOK_REALLY_FIRST);
    ap_hook_handler(axis_handler, NULL, NULL, APR_HOOK_LAST);    
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "mod_axis_register_hooks", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

module AP_MODULE_DECLARE_DATA axis_module = {
    STANDARD20_MODULE_STUFF,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    mod_axis_register_hooks,
};

}

