#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
 
#include <axis/OtherFaultException.hpp>

AXIS_CPP_NAMESPACE_START

/*
 * All the methods in this class do a deep copy of the data so that the fault can be
 * processed whatever state the engine is in. Doing a deep copy means this class owns
 * the storage and can delete it in its destructor making client programming simpler.
 */

#define STRINGCOPY(tgt,src)					\
{											\
	if (NULL != src && 0 != strlen(src))	\
	{										\
		tgt = new AxisChar[strlen(src)+1];	\
		strcpy(tgt, src);					\
	} else tgt = NULL;						\
}

#define STRINGREPLACE(tgt,src)				\
{											\
	if (NULL != tgt)						\
		delete [] tgt;						\
	STRINGCOPY(tgt,src);					\
}

OtherFaultException::OtherFaultException() 
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("OtherFaultException", "OtherFaultException", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	m_detail = NULL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("OtherFaultException", "OtherFaultException", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

OtherFaultException::OtherFaultException(
	const AxisChar *code, const AxisChar *string, const AxisChar *actor, const AxisChar *detail, int exceptionCode) :
	SoapFaultException(code,string,actor,exceptionCode)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("OtherFaultException", "OtherFaultException", this, 5,
					TRACETYPE_STRING, 0, ((void*)&code),
					TRACETYPE_STRING, 0, ((void*)&string),
					TRACETYPE_STRING, 0, ((void*)&actor),
					TRACETYPE_STRING, 0, ((void*)&detail),
					TRACETYPE_INT, 0, ((void*)&exceptionCode));	  /* AUTOINSERTED TRACE */
	#endif

	STRINGCOPY(m_detail,detail);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("OtherFaultException", "OtherFaultException", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

OtherFaultException::OtherFaultException(AxisException& ae) : SoapFaultException(ae)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("OtherFaultException", "OtherFaultException", this, 1,
					TRACETYPE_AXISEXCEPTION, sizeof(AxisException&), ((void*)&ae));	  /* AUTOINSERTED TRACE */
	#endif

	m_detail = NULL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("OtherFaultException", "OtherFaultException", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

OtherFaultException::OtherFaultException(const OtherFaultException& copy) : SoapFaultException(copy)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("OtherFaultException", "OtherFaultException", this, 1,
					TRACETYPE_DATA, sizeof(OtherFaultException&), ((void*)&copy));	  /* AUTOINSERTED TRACE */
	#endif

	STRINGCOPY(m_detail, copy.m_detail);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("OtherFaultException", "OtherFaultException", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

OtherFaultException& OtherFaultException::operator=(OtherFaultException copy)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("OtherFaultException", "operator=", this, 1,
					TRACETYPE_DATA, sizeof(OtherFaultException), ((void*)&copy));	  /* AUTOINSERTED TRACE */
	#endif

	SoapFaultException::operator=(copy);
	STRINGREPLACE(m_detail, copy.m_detail);
		{
		#ifdef ENABLE_AXISTRACE
			OtherFaultException& traceRet = (*this);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("OtherFaultException", "operator=", this, 0,
					TRACETYPE_DATA, sizeof(OtherFaultException&), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return *this;
		#endif
	}

}

OtherFaultException::~OtherFaultException() throw()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("OtherFaultException", "~OtherFaultException", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (NULL != m_detail) delete [] m_detail;
	m_detail = NULL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("OtherFaultException", "~OtherFaultException", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const AxisChar *OtherFaultException::getFaultDetail() const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("OtherFaultException", "getFaultDetail", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			const AxisChar* traceRet = (m_detail);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("OtherFaultException", "getFaultDetail", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_detail;
		#endif
	}

}

void OtherFaultException::setFaultDetail(const AxisChar *detail)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("OtherFaultException", "setFaultDetail", this, 1,
					TRACETYPE_STRING, 0, ((void*)&detail));	  /* AUTOINSERTED TRACE */
	#endif

	STRINGCOPY(m_detail,detail);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("OtherFaultException", "setFaultDetail", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXIS_CPP_NAMESPACE_END

