#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
 
#include <axis/SoapFaultException.hpp>

AXIS_CPP_NAMESPACE_START

/*
 * All the methods in this class do a deep copy of the data so that the fault can be
 * processed whatever state the engine is in. Doing a deep copy means this class owns
 * the storage and can delete it in its destructor making client programming simpler.
 */

#define STRINGCOPY(tgt,src)					\
{											\
	if (NULL != src && 0 != strlen(src))	\
	{										\
		tgt = new AxisChar[strlen(src)+1];	\
		strcpy(tgt, src);					\
	} else tgt = NULL;						\
}

#define STRINGREPLACE(tgt,src)				\
{											\
	if (NULL != tgt)						\
		delete [] tgt;						\
	STRINGCOPY(tgt,src);					\
}

SoapFaultException::SoapFaultException()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "SoapFaultException", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	m_code = NULL;
	m_sMessage = NULL;
	m_actor = NULL;
	m_iExceptionCode = 0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "SoapFaultException", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

SoapFaultException::SoapFaultException(
	const AxisChar *code, const AxisChar *string, const AxisChar *actor, int exceptionCode) 
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "SoapFaultException", this, 4,
					TRACETYPE_STRING, 0, ((void*)&code),
					TRACETYPE_STRING, 0, ((void*)&string),
					TRACETYPE_STRING, 0, ((void*)&actor),
					TRACETYPE_INT, 0, ((void*)&exceptionCode));	  /* AUTOINSERTED TRACE */
	#endif

	STRINGCOPY(m_code,code);
	STRINGCOPY(m_sMessage,string);
	STRINGCOPY(m_actor,actor);
	m_iExceptionCode = exceptionCode;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "SoapFaultException", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

SoapFaultException::SoapFaultException(AxisException& ae)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "SoapFaultException", this, 1,
					TRACETYPE_AXISEXCEPTION, sizeof(AxisException&), ((void*)&ae));	  /* AUTOINSERTED TRACE */
	#endif

	STRINGCOPY(m_sMessage,ae.what());
	m_iExceptionCode = ae.getExceptionCode();
	m_code = NULL;
	m_actor = NULL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "SoapFaultException", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

SoapFaultException::SoapFaultException(const SoapFaultException& copy)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "SoapFaultException", this, 1,
					TRACETYPE_DATA, sizeof(SoapFaultException&), ((void*)&copy));	  /* AUTOINSERTED TRACE */
	#endif

	STRINGCOPY(m_code	,copy.m_code);
	STRINGCOPY(m_sMessage	,copy.m_sMessage);
	STRINGCOPY(m_actor	,copy.m_actor);
	m_iExceptionCode = copy.m_iExceptionCode;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "SoapFaultException", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

SoapFaultException& SoapFaultException::operator=(SoapFaultException copy)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "operator=", this, 1,
					TRACETYPE_DATA, sizeof(SoapFaultException), ((void*)&copy));	  /* AUTOINSERTED TRACE */
	#endif

	exception::operator=(copy);
	STRINGREPLACE(m_code	,copy.m_code);
	STRINGREPLACE(m_sMessage	,copy.m_sMessage);
	STRINGREPLACE(m_actor	,copy.m_actor);
	m_iExceptionCode = copy.m_iExceptionCode;
		{
		#ifdef ENABLE_AXISTRACE
			SoapFaultException& traceRet = (*this);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "operator=", this, 0,
					TRACETYPE_DATA, sizeof(SoapFaultException&), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return *this;
		#endif
	}

}

SoapFaultException::~SoapFaultException() throw()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "~SoapFaultException", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (NULL != m_code) delete [] m_code;
	if (NULL != m_sMessage) delete [] m_sMessage;
	if (NULL != m_actor) delete [] m_actor;
	m_code = NULL;
	m_sMessage = NULL;
	m_actor = NULL;
	m_iExceptionCode = 0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "~SoapFaultException", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const AxisChar *SoapFaultException::getFaultCode() const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "getFaultCode", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			const AxisChar* traceRet = (m_code);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "getFaultCode", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_code;
		#endif
	}

}

const AxisChar *SoapFaultException::getFaultString() const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "getFaultString", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			const AxisChar* traceRet = (m_sMessage);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "getFaultString", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_sMessage;
		#endif
	}

}

const AxisChar *SoapFaultException::getFaultActor() const 
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "getFaultActor", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			const AxisChar* traceRet = (m_actor);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "getFaultActor", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_actor;
		#endif
	}

}

void SoapFaultException::setFaultCode(const AxisChar *code)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "setFaultCode", this, 1,
					TRACETYPE_STRING, 0, ((void*)&code));	  /* AUTOINSERTED TRACE */
	#endif

	STRINGCOPY(m_code,code);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "setFaultCode", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void SoapFaultException::setFaultString(const AxisChar *string)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "setFaultString", this, 1,
					TRACETYPE_STRING, 0, ((void*)&string));	  /* AUTOINSERTED TRACE */
	#endif

	STRINGCOPY(m_sMessage,string);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "setFaultString", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void SoapFaultException::setFaultActor(const AxisChar *actor)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapFaultException", "setFaultActor", this, 1,
					TRACETYPE_STRING, 0, ((void*)&actor));	  /* AUTOINSERTED TRACE */
	#endif

	STRINGCOPY(m_actor,actor);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapFaultException", "setFaultActor", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXIS_CPP_NAMESPACE_END

