#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *   @author Sanjaya Singharage
 *   @author Roshan Weerasuriya (roshan@jkcs.slt.lk, roshan@opensource.lk)
 *   @author Susantha Kumara (skumara@virtusa.com, susantha@opensource.lk)
 *
 */

#include "WSDDDeployment.h"

#include "WSDDDocument.h"

#include <axis/GDefine.hpp>

#include "../common/AxisConfig.h"

#include "../common/AxisTrace.h"

#include "AxisWsddException.h"

AXIS_CPP_NAMESPACE_START

WSDDDeployment::WSDDDeployment()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "WSDDDeployment", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_DeployedServices = NULL;
    m_sAux = "";
    m_pTransportHandlers = NULL;
    m_GlobalResponseHandlers = NULL;
    m_GlobalRequestHandlers = NULL;
    m_DeplType = DT_DEPLOYMENT;
    m_pLibNameIdMap = new map<AxisString, int>;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "WSDDDeployment", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

WSDDDeployment::~WSDDDeployment()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "~WSDDDeployment", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    WSDDHandlerList::iterator iter;
    if(m_pTransportHandlers)
    {
        delete m_pTransportHandlers;
    }
    if(m_GlobalRequestHandlers)
    {
        for(iter=m_GlobalRequestHandlers->begin();
                iter!=m_GlobalRequestHandlers->end();iter++)
        {
            delete (*iter);
        }
    }
    if(m_GlobalResponseHandlers)
    {
        for(iter=m_GlobalResponseHandlers->begin();
                iter!=m_GlobalResponseHandlers->end();iter++)
        {
            delete (*iter);
        }
    }
    WSDDServiceMap::iterator iter2;
    if(m_DeployedServices)
    {
        for(iter2=m_DeployedServices->begin()
                ;iter2!=m_DeployedServices->end();iter2++)
        {
            delete ((*iter2).second);
        }
        delete m_DeployedServices;
    }
    delete m_pLibNameIdMap;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "~WSDDDeployment", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const WSDDHandlerList* WSDDDeployment::getGlobalRequestFlowHandlers()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "getGlobalRequestFlowHandlers", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (m_GlobalRequestHandlers);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getGlobalRequestFlowHandlers", this, 0,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_GlobalRequestHandlers;
		#endif
	}

}

const WSDDHandlerList* WSDDDeployment::getGlobalResponseFlowHandlers()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "getGlobalResponseFlowHandlers", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (m_GlobalResponseHandlers);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getGlobalResponseFlowHandlers", this, 0,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_GlobalResponseHandlers;
		#endif
	}

}

int WSDDDeployment::updateWSDD(const AxisChar* sWSDD)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "updateWSDD", this, 1,
					TRACETYPE_STRING, 0, ((void*)&sWSDD));	  /* AUTOINSERTED TRACE */
	#endif

    WSDDDocument* doc = new WSDDDocument(m_pLibNameIdMap);
    if (AXIS_SUCCESS != doc->updateDeployment(sWSDD, this))
    {
        delete doc;
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "updateWSDD", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

    }
    delete doc;
    saveWSDD();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "updateWSDD", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}
        
}

int WSDDDeployment::loadWSDD( const AxisChar * sWSDD)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "loadWSDD", this, 1,
					TRACETYPE_STRING, 0, ((void*)&sWSDD));	  /* AUTOINSERTED TRACE */
	#endif

    m_sWSDDPath = std::string( sWSDD);
    WSDDDocument *	doc = new WSDDDocument( m_pLibNameIdMap);
    if( AXIS_SUCCESS != doc->getDeployment( sWSDD, this))
    {
        //printf("wsdd loading failed\n");
        delete doc;
        //return AXIS_FAIL;
        throw AxisWsddException( SERVER_WSDD_FILE_NOT_FOUND);
    }
    delete doc;
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "loadWSDD", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int WSDDDeployment::saveWSDD()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "saveWSDD", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    FILE* file;
    int Status = AXIS_FAIL;
    /* TODO we have to rename the existing server.wsdd to server.wsdd.old
     * and if anything went wrong we have to rename it back to server.wsdd
     */
    file = fopen(m_sWSDDPath.c_str(), "w");
    if(!file) 	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "saveWSDD", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

    do
    {
        if (fputs("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n", file) < 0) 
            break;
        if (fputs("<deployment xmlns=\"http://xml.apache.org/axis/wsdd/\
            \" xmlns:C=\"http://xml.apache.org/axis/wsdd/providers/C\" xmlns:\
            CPP=\"http://xml.apache.org/axis/wsdd/providers/CPP\">\n",
            file) < 0) break;
        if (fputs("\t<globalConfiguration>\n", file) < 0) break;
        WSDDHandlerList::iterator iter;
        if(m_GlobalRequestHandlers)
        {
            if (fputs("\t\t<requestFlow>\n", file) < 0) break;
            for(iter=m_GlobalRequestHandlers->begin();
            iter!=m_GlobalRequestHandlers->end(); iter++)
            {
                (*iter)->updateWSDD(file, 3);
            }
            if (fputs("\t\t</requestFlow>\n", file) < 0) break;
        }
        if(m_GlobalResponseHandlers)
        {
            if (fputs("\t\t<responseFlow>\n", file) < 0) break;
            for(iter=m_GlobalResponseHandlers->begin();
            iter!=m_GlobalResponseHandlers->end(); iter++)
            {
                (*iter)->updateWSDD(file, 3);
            }
            if (fputs("\t\t</responseFlow>\n", file) < 0) break;
        }
        if (fputs("\t</globalConfiguration>\n", file) < 0) break;
        if(m_pTransportHandlers)
        {
            m_pTransportHandlers->updateWSDD(file, 3);
        }
        WSDDServiceMap::iterator iter2;
        if(m_DeployedServices)
        {
            for(iter2=m_DeployedServices->begin();
            iter2!=m_DeployedServices->end(); iter2++)
            {
                ((*iter2).second)->updateWSDD(file, 1);
            }
        }
        
        if (fputs("</deployment>", file) < 0) break;
    } while(0);
    fflush(file);
    fclose(file);
    if (AXIS_SUCCESS != Status)
    {
        /*TODO use the previous server.wsdd file itself. Undo renaming*/
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "saveWSDD", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

const WSDDService* WSDDDeployment::getService(const AxisChar* sServiceName)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "getService", this, 1,
					TRACETYPE_STRING, 0, ((void*)&sServiceName));	  /* AUTOINSERTED TRACE */
	#endif

    WSDDServiceMap::iterator iter;
    if (!m_DeployedServices) 	{
		#ifdef ENABLE_AXISTRACE
			const WSDDService* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getService", this, 1,
					TRACETYPE_POINTER, sizeof(WSDDService), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    try
    {
    iter = m_DeployedServices->find(sServiceName);
    if (iter != m_DeployedServices->end())
    {
        	{
		#ifdef ENABLE_AXISTRACE
			const WSDDService* traceRet = ((*iter).second);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getService", this, 2,
					TRACETYPE_POINTER, sizeof(WSDDService), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*iter).second;
		#endif
	}

    }
    else
    {
        	{
		#ifdef ENABLE_AXISTRACE
			const WSDDService* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getService", this, 3,
					TRACETYPE_POINTER, sizeof(WSDDService), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

	//throw AxisWsddException_NULL(CLIENT_WSDD_SERVICE_NOT_FOUND);
    }
    }
    catch(exception& e)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("WSDDDeployment", "getService", this, 0,
					TRACETYPE_EXCEPTION, sizeof(exception&), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

        e = e;
        throw AxisWsddException(SERVER_WSDD_EXCEPTION);
    }
}

const WSDDServiceMap* WSDDDeployment::getWSDDServiceMap() const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "getWSDDServiceMap", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const WSDDServiceMap* traceRet = (m_DeployedServices);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getWSDDServiceMap", this, 0,
					TRACETYPE_POINTER, sizeof(WSDDServiceMap), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_DeployedServices;
		#endif
	}

}

const AxisChar* WSDDDeployment::getLibName(int nLibId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "getLibName", this, 1,
					TRACETYPE_INT, 0, ((void*)&nLibId));	  /* AUTOINSERTED TRACE */
	#endif

    for (map<AxisString, int>::iterator it = m_pLibNameIdMap->begin();
    it != m_pLibNameIdMap->end(); it++)
    {
        if ((*it).second == nLibId)
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const AxisChar* traceRet = ((*it).first.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getLibName", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*it).first.c_str();
		#endif
	}

        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			const AxisChar* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getLibName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

int WSDDDeployment::addService(WSDDService* pService)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "addService", this, 1,
					TRACETYPE_POINTER, sizeof(WSDDService), ((void*)&pService));	  /* AUTOINSERTED TRACE */
	#endif

    if (!m_DeployedServices) m_DeployedServices = new WSDDServiceMap;
    if (m_DeployedServices->find(pService->getServiceName()) != 
        m_DeployedServices->end())
    {
        AXISTRACE1("The service already exists and the attempt to re-deploy \
            is ignored", WARN);
    }
    else
    {
        (*m_DeployedServices)[pService->getServiceName()] = pService;
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "addService", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}        
int WSDDDeployment::addHandler(bool bGlobal, bool bRequestFlow,
                               WSDDHandler* pHandler, 
                               AXIS_PROTOCOL_TYPE protocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "addHandler", this, 4,
					TRACETYPE_BOOL, 0, ((void*)&bGlobal),
					TRACETYPE_BOOL, 0, ((void*)&bRequestFlow),
					TRACETYPE_POINTER, sizeof(WSDDHandler), ((void*)&pHandler),
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&protocol));	  /* AUTOINSERTED TRACE */
	#endif

    if (bGlobal)
    {
        if (bRequestFlow)
        {
            if(!m_GlobalRequestHandlers) m_GlobalRequestHandlers = 
                new WSDDHandlerList;
            m_GlobalRequestHandlers->push_back(pHandler);
        }
        else
        {
            if(!m_GlobalResponseHandlers) m_GlobalResponseHandlers = 
                new WSDDHandlerList;
            m_GlobalResponseHandlers->push_back(pHandler);
        }
    }
    else /* transport */
    {
        if (!m_pTransportHandlers) m_pTransportHandlers = new WSDDTransport();
        m_pTransportHandlers->addHandler(bRequestFlow, protocol, pHandler);
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "addHandler", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

const WSDDHandlerList* WSDDDeployment::getTransportRequestFlowHandlers
    (AXIS_PROTOCOL_TYPE protocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "getTransportRequestFlowHandlers", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&protocol));	  /* AUTOINSERTED TRACE */
	#endif

    if (!m_pTransportHandlers) 	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getTransportRequestFlowHandlers", this, 1,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (m_pTransportHandlers->getRequestFlowHandlers(protocol));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getTransportRequestFlowHandlers", this, 2,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pTransportHandlers->getRequestFlowHandlers(protocol);
		#endif
	}

}

const WSDDHandlerList* WSDDDeployment::getTransportResponseFlowHandlers
    (AXIS_PROTOCOL_TYPE protocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "getTransportResponseFlowHandlers", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&protocol));	  /* AUTOINSERTED TRACE */
	#endif

    if (!m_pTransportHandlers) 	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getTransportResponseFlowHandlers", this, 1,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (m_pTransportHandlers->getResponseFlowHandlers(protocol));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getTransportResponseFlowHandlers", this, 2,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pTransportHandlers->getResponseFlowHandlers(protocol);
		#endif
	}

}

int WSDDDeployment::removeService(WSDDService* pService)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "removeService", this, 1,
					TRACETYPE_POINTER, sizeof(WSDDService), ((void*)&pService));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_DeployedServices && 
        (m_DeployedServices->find(pService->getServiceName()) != 
        m_DeployedServices->end()))
    {
        WSDDService* pTheService = 
            (*m_DeployedServices)[pService->getServiceName()];
        m_pLibNameIdMap->erase(pTheService->getLibName());
        m_DeployedServices->erase(pService->getServiceName());
        delete pTheService;    
        delete pService;
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "removeService", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_NO_SUCH_SERVICE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "removeService", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_NO_SUCH_SERVICE;
		#endif
	}

}

int WSDDDeployment::removeHandler(bool bGlobal, bool bRequestFlow, 
                                  WSDDHandler* pHandler, 
                                  AXIS_PROTOCOL_TYPE protocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "removeHandler", this, 4,
					TRACETYPE_BOOL, 0, ((void*)&bGlobal),
					TRACETYPE_BOOL, 0, ((void*)&bRequestFlow),
					TRACETYPE_POINTER, sizeof(WSDDHandler), ((void*)&pHandler),
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&protocol));	  /* AUTOINSERTED TRACE */
	#endif

    m_pLibNameIdMap->erase(pHandler->getLibName());
    if (bGlobal)
    {
        WSDDHandlerList* pTempList = 
            bRequestFlow ? m_GlobalRequestHandlers : m_GlobalResponseHandlers;
        if(pTempList)
        {
            for (WSDDHandlerList::iterator itr = pTempList->begin();
                 itr != pTempList->end(); itr++)
            {
                if (strcmp((*itr)->getLibName(), pHandler->getLibName()) == 0)
                {
                    pTempList->remove(*itr);
                    delete (*itr);
                    delete pHandler;
                    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "removeHandler", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

                }
            }
        }
    }
    else /* transport */
    {
        if (!m_pTransportHandlers) 	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_NO_SUCH_HANDLER);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "removeHandler", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_NO_SUCH_HANDLER;
		#endif
	}

        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_pTransportHandlers->removeHandler(bRequestFlow, 
            protocol, pHandler));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "removeHandler", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pTransportHandlers->removeHandler(bRequestFlow, 
            protocol, pHandler);
		#endif
	}

    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_NO_SUCH_HANDLER);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "removeHandler", this, 4,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_NO_SUCH_HANDLER;
		#endif
	}
    
}

DEPLOYMENTTYPE WSDDDeployment::getDeploymentType() const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "getDeploymentType", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			DEPLOYMENTTYPE traceRet = (m_DeplType);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "getDeploymentType", this, 0,
					TRACETYPE_DATA, sizeof(DEPLOYMENTTYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_DeplType;
		#endif
	}

}

void WSDDDeployment::setDeploymentType(DEPLOYMENTTYPE nType)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDDeployment", "setDeploymentType", this, 1,
					TRACETYPE_DATA, sizeof(DEPLOYMENTTYPE), ((void*)&nType));	  /* AUTOINSERTED TRACE */
	#endif

    m_DeplType = nType;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDDeployment", "setDeploymentType", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXIS_CPP_NAMESPACE_END

