#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 * @author Sanjaya Singharage
 * @author Susantha Kumara (skumara@virtusa.com, susantha@opensource.lk)
 *
 */

#include "WSDDService.h"

#include "../common/AxisTrace.h"

AXIS_CPP_NAMESPACE_START

WSDDService::WSDDService()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "WSDDService", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_RequestHandlers = NULL;
    m_ResponseHandlers = NULL;
    m_Provider = UNKNOWN_PROVIDER;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "WSDDService", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

WSDDService::~WSDDService()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "~WSDDService", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    WSDDHandlerList::iterator iter;
    if(m_RequestHandlers)
    {
        for(iter=m_RequestHandlers->begin();iter!=m_RequestHandlers->end();
        iter++)
        {
            delete (*iter);
        }
        delete m_RequestHandlers;
    }
    if(m_ResponseHandlers)
    {
        for(iter=m_ResponseHandlers->begin();iter!=m_ResponseHandlers->end();
        iter++)
        {
            delete (*iter);
        }
        delete m_ResponseHandlers;
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "~WSDDService", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void WSDDService::setProvider(const AxisChar* sProvider)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "setProvider", this, 1,
					TRACETYPE_STRING, 0, ((void*)&sProvider));	  /* AUTOINSERTED TRACE */
	#endif

    if (0 == strcmp(sProvider, "CPP:RPC"))
    {
        m_Provider = CPP_RPC_PROVIDER;
    }
    else if (0 == strcmp(sProvider, "C:RPC"))
    {
        m_Provider = C_RPC_PROVIDER;
    }
    else if (0 == strcmp(sProvider, "CPP:DOCUMENT"))
    {
        m_Provider = CPP_DOC_PROVIDER;
    }
    else if (0 == strcmp(sProvider, "C:DOCUMENT"))
    {
        m_Provider = C_DOC_PROVIDER;
    }
    else if (0 == strcmp(sProvider, "MS:COM"))
    {
        m_Provider = COM_PROVIDER;
    }
    else
    {
        m_Provider = UNKNOWN_PROVIDER;
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "setProvider", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

PROVIDERTYPE WSDDService::getProvider() const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "getProvider", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			PROVIDERTYPE traceRet = (m_Provider);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getProvider", this, 0,
					TRACETYPE_DATA, sizeof(PROVIDERTYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_Provider;
		#endif
	}

}

const AxisChar* WSDDService::getServiceName() const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "getServiceName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const AxisChar* traceRet = (m_sName.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getServiceName", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_sName.c_str();
		#endif
	}

}

void WSDDService::addAllowedMethod(const AxisChar* sMethodName)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "addAllowedMethod", this, 1,
					TRACETYPE_STRING, 0, ((void*)&sMethodName));	  /* AUTOINSERTED TRACE */
	#endif

    m_AllowedMethods.push_back(sMethodName);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "addAllowedMethod", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

bool WSDDService::isAllowedMethod(const AxisChar* sServiceName) const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "isAllowedMethod", this, 1,
					TRACETYPE_STRING, 0, ((void*)&sServiceName));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_AllowedMethods.size() == 0) //no allowedMethods mentioned in wsdd, hence assume all are allowed
        	{
		#ifdef ENABLE_AXISTRACE
			bool traceRet = (true);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "isAllowedMethod", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return true;
		#endif
	}
 // assume that the method is allowd if allowedMethods missing
    list<AxisString>::const_iterator iter;
    iter = m_AllowedMethods.begin();
    if ((*iter) == "*")
    {
        	{
		#ifdef ENABLE_AXISTRACE
			bool traceRet = (true);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "isAllowedMethod", this, 2,
					TRACETYPE_BOOL, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return true;
		#endif
	}

    }
    for(iter = m_AllowedMethods.begin();iter != m_AllowedMethods.end();iter++)
    {
        if((*iter) == sServiceName)
        {
            	{
		#ifdef ENABLE_AXISTRACE
			bool traceRet = (true);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "isAllowedMethod", this, 3,
					TRACETYPE_BOOL, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return true;
		#endif
	}

        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			bool traceRet = (false);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "isAllowedMethod", this, 4,
					TRACETYPE_BOOL, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return false;
		#endif
	}

}

const WSDDHandlerList* WSDDService::getRequestFlowHandlers() const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "getRequestFlowHandlers", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (m_RequestHandlers);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getRequestFlowHandlers", this, 0,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_RequestHandlers;
		#endif
	}

}

const WSDDHandlerList* WSDDService::getResponseFlowHandlers() const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "getResponseFlowHandlers", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (m_ResponseHandlers);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getResponseFlowHandlers", this, 0,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_ResponseHandlers;
		#endif
	}

}

void WSDDService::addAllowedRole(const AxisChar* sRole)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "addAllowedRole", this, 1,
					TRACETYPE_STRING, 0, ((void*)&sRole));	  /* AUTOINSERTED TRACE */
	#endif

    m_AllowedRoles.push_back(sRole);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "addAllowedRole", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const list<AxisString>& WSDDService::getAllowedRoles()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "getAllowedRoles", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const list<AxisString>& traceRet = (m_AllowedRoles);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getAllowedRoles", this, 0,
					TRACETYPE_DATA, sizeof(list<AxisString>&), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_AllowedRoles;
		#endif
	}

}

void  WSDDService::addHandler(bool bRequestFlow, WSDDHandler* pHandler)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "addHandler", this, 2,
					TRACETYPE_BOOL, 0, ((void*)&bRequestFlow),
					TRACETYPE_POINTER, sizeof(WSDDHandler), ((void*)&pHandler));	  /* AUTOINSERTED TRACE */
	#endif

    if (bRequestFlow)
    {
        if (!m_RequestHandlers) m_RequestHandlers = new WSDDHandlerList;
        m_RequestHandlers->push_back(pHandler);
    }
    else
    {
        if (!m_ResponseHandlers) m_ResponseHandlers = new WSDDHandlerList;
        m_ResponseHandlers->push_back(pHandler);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "addHandler", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/*
 * This method removes the service specific handler of this service. But the entry
 * in WSDDDeployment's <LibId, LibName> is not removed because the same handler may
 * be in use by other services. 
 */

int WSDDService::removeHandler(bool bRequestFlow, WSDDHandler* pHandler)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "removeHandler", this, 2,
					TRACETYPE_BOOL, 0, ((void*)&bRequestFlow),
					TRACETYPE_POINTER, sizeof(WSDDHandler), ((void*)&pHandler));	  /* AUTOINSERTED TRACE */
	#endif

    WSDDHandlerList* pTempList = 
        bRequestFlow ? m_RequestHandlers : m_ResponseHandlers;
    if (!pTempList) 	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_NO_SUCH_HANDLER);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "removeHandler", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_NO_SUCH_HANDLER;
		#endif
	}

    for (WSDDHandlerList::iterator itr = pTempList->begin();
         itr != pTempList->end(); itr++)
    {
        if (strcmp((*itr)->getLibName(), pHandler->getLibName()) == 0)
        {
            pTempList->remove(*itr);
            delete (*itr);
            delete pHandler;
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "removeHandler", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_NO_SUCH_HANDLER);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "removeHandler", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_NO_SUCH_HANDLER;
		#endif
	}

}

const list<AxisString> WSDDService::getAllowedMethods() const
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "getAllowedMethods", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const list<AxisString> traceRet = (m_AllowedMethods);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getAllowedMethods", this, 0,
					TRACETYPE_DATA, sizeof(list<AxisString>), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_AllowedMethods;
		#endif
	}

}

int WSDDService::updateWSDD(FILE* wsddfile, int tabcount)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "updateWSDD", this, 2,
					TRACETYPE_POINTER, sizeof(FILE), ((void*)&wsddfile),
					TRACETYPE_INT, 0, ((void*)&tabcount));	  /* AUTOINSERTED TRACE */
	#endif

    m_file = wsddfile;
    printTabs(tabcount); *this << "<service name=\"" << 
        m_sName.c_str() << "\" provider=\"" << getProviderString() << 
        "\" description=\"" << m_sDescription.c_str() << "\">\n";
    if (m_Params)
    {
        map<AxisString, AxisString>::iterator itr;
        for (itr = m_Params->begin(); itr != m_Params->end(); itr++)
        {
            printTabs(tabcount+1); *this << "<parameter name=\"" << 
                (*itr).first.c_str() << "\" value=\"" << 
                (*itr).second.c_str() << "\" />";
        }
    }
    WSDDHandlerList::iterator iter;
    if(m_RequestHandlers)
    {
        *this << "\t\t<requestFlow>\n";
        for(iter=m_RequestHandlers->begin(); iter!=m_RequestHandlers->end();
            iter++)
        {
            (*iter)->updateWSDD(wsddfile, 3);
        }
        *this << "\t\t</requestFlow>\n";
    }
    if(m_ResponseHandlers)
    {
        *this << "\t\t<responseFlow>\n";
        for(iter=m_ResponseHandlers->begin(); iter!=m_ResponseHandlers->end(); 
        iter++)
        {
            (*iter)->updateWSDD(wsddfile, 3);
        }
        *this << "\t\t</responseFlow>\n";
    }
    printTabs(tabcount+1); *this <<
        "<parameter name=\"allowedMethods\" value=\"";
    /* for looop */
    for (list<AxisString>::iterator itrs = m_AllowedMethods.begin();
    itrs != m_AllowedMethods.end(); itrs++)
    {
        *this << (*itrs).c_str() << " ";
    }
    *this << "\"/>\n"; 
    printTabs(tabcount+1); *this << "<parameter name=\"className\" value=\"" 
        << m_sLibName.c_str();
    *this << "\" />\n"; 
    printTabs(tabcount); *this << "</service>\n";
    m_file = 0;
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "updateWSDD", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

const char* WSDDService::getProviderString()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDService", "getProviderString", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    switch(m_Provider)
    {
    case C_RPC_PROVIDER: 	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ("C:RPC");
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getProviderString", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return "C:RPC";
		#endif
	}

    case C_DOC_PROVIDER: 	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ("C:DOCUMENT");
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getProviderString", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return "C:DOCUMENT";
		#endif
	}

    case CPP_RPC_PROVIDER: 	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ("CPP:RPC");
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getProviderString", this, 3,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return "CPP:RPC";
		#endif
	}

    case CPP_DOC_PROVIDER: 	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ("CPP:DOCUMENT");
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getProviderString", this, 4,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return "CPP:DOCUMENT";
		#endif
	}

    case COM_PROVIDER: 	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ("COM");
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getProviderString", this, 5,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return "COM";
		#endif
	}

    default: 	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ("unknown");
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getProviderString", this, 6,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return "unknown";
		#endif
	}

    }
    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDService", "getProviderString", this, 7,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

