#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 * @author Sanjaya Singharage
 *
 */

#include "../wsdd/WSDDTransport.h"

AXIS_CPP_NAMESPACE_START

WSDDTransport::WSDDTransport()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDTransport", "WSDDTransport", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_RequestHandlers = NULL;
    m_ResponseHandlers = NULL;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "WSDDTransport", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

WSDDTransport::~WSDDTransport()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDTransport", "~WSDDTransport", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    map<AXIS_PROTOCOL_TYPE, WSDDHandlerList>::iterator iter;
    /* WSDDHandlerList * hl = NULL; */
    WSDDHandlerList::iterator iter2;
    if(m_RequestHandlers)
    {
        for(iter = m_RequestHandlers->begin(); iter !=
            m_RequestHandlers->end(); iter++)
        {
            for(iter2 = (*iter).second.begin(); iter2 != 
                (*iter).second.end(); iter2++)
            {
                delete (*iter2);
            }
        }
    }
    if(m_ResponseHandlers)
    {
        for(iter = m_ResponseHandlers->begin(); iter != 
            m_ResponseHandlers->end(); iter++)
        {
            for(iter2 = (*iter).second.begin(); iter2 != 
                (*iter).second.end();iter2++)
            {
                delete (*iter2);
            }
        }
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "~WSDDTransport", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const WSDDHandlerList* WSDDTransport::getRequestFlowHandlers
(AXIS_PROTOCOL_TYPE Protocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDTransport", "getRequestFlowHandlers", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&Protocol));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_RequestHandlers && m_RequestHandlers->find(Protocol) !=
        m_RequestHandlers->end())
        	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (&(*m_RequestHandlers)[Protocol]);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "getRequestFlowHandlers", this, 1,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return &(*m_RequestHandlers)[Protocol];
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "getRequestFlowHandlers", this, 2,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

const WSDDHandlerList* WSDDTransport::getResponseFlowHandlers
(AXIS_PROTOCOL_TYPE Protocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDTransport", "getResponseFlowHandlers", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&Protocol));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_ResponseHandlers && m_ResponseHandlers->find(Protocol) !=
        m_ResponseHandlers->end())
        	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (&(*m_ResponseHandlers)[Protocol]);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "getResponseFlowHandlers", this, 1,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return &(*m_ResponseHandlers)[Protocol];
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			const WSDDHandlerList* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "getResponseFlowHandlers", this, 2,
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

void WSDDTransport::addHandler(bool bRequestFlow, AXIS_PROTOCOL_TYPE protocol,
                               WSDDHandler* pHandler)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDTransport", "addHandler", this, 3,
					TRACETYPE_BOOL, 0, ((void*)&bRequestFlow),
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&protocol),
					TRACETYPE_POINTER, sizeof(WSDDHandler), ((void*)&pHandler));	  /* AUTOINSERTED TRACE */
	#endif

    if (bRequestFlow)
    {
        if (!m_RequestHandlers) m_RequestHandlers = new map<AXIS_PROTOCOL_TYPE,
            WSDDHandlerList>;
        (*m_RequestHandlers)[protocol].push_back(pHandler);
    }
    else
    {
        if (!m_ResponseHandlers) m_ResponseHandlers = 
            new map<AXIS_PROTOCOL_TYPE, WSDDHandlerList>;
        (*m_ResponseHandlers)[protocol].push_back(pHandler);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "addHandler", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int WSDDTransport::removeHandler(bool bRequestFlow, 
                                 AXIS_PROTOCOL_TYPE protocol,
                                 WSDDHandler* pHandler)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDTransport", "removeHandler", this, 3,
					TRACETYPE_BOOL, 0, ((void*)&bRequestFlow),
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&protocol),
					TRACETYPE_POINTER, sizeof(WSDDHandler), ((void*)&pHandler));	  /* AUTOINSERTED TRACE */
	#endif

    map<AXIS_PROTOCOL_TYPE, WSDDHandlerList>* pTempHandlers = 
        bRequestFlow ? m_RequestHandlers : m_ResponseHandlers;
    WSDDHandlerList* pList = &(*(pTempHandlers->find(protocol))).second;
    if (pList)
    {
        for (WSDDHandlerList::iterator itr = pList->begin();
             itr != pList->end(); itr++)
        {
            if (strcmp((*itr)->getLibName(), pHandler->getLibName()) == 0)
            {
                pList->remove(*itr);
                delete (*itr);
                delete pHandler;
                	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "removeHandler", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

            }
        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_NO_SUCH_HANDLER);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "removeHandler", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_NO_SUCH_HANDLER;
		#endif
	}
    
}

int WSDDTransport::updateWSDD(FILE* wsddfile, int tabcount)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("WSDDTransport", "updateWSDD", this, 2,
					TRACETYPE_POINTER, sizeof(FILE), ((void*)&wsddfile),
					TRACETYPE_INT, 0, ((void*)&tabcount));	  /* AUTOINSERTED TRACE */
	#endif

    WSDDHandlerList::iterator iter2;
    const char* trtype = 0;
    for (int type = APTHTTP1_0; type < APTOTHER; type++)
    {
        if((m_RequestHandlers && 
            (m_RequestHandlers->find((AXIS_PROTOCOL_TYPE)type) != 
            m_RequestHandlers->end())) ||
            (m_RequestHandlers && 
            (m_RequestHandlers->find((AXIS_PROTOCOL_TYPE)type) != 
            m_RequestHandlers->end())))
        {
            trtype = (APTHTTP1_0 == type || APTHTTP1_1 == type) ? 
                "http" : ((APTSMTP == type) ? "smtp" : "unsupported");
            if (fputs("\t<transport name=\"", wsddfile) < 0) 	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

            if (fputs(trtype, wsddfile) < 0) 	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

            if (fputs("\" >\n", wsddfile) < 0) 	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

            /* Write request flow handler configuration */
            if(m_RequestHandlers && 
                (m_RequestHandlers->find((AXIS_PROTOCOL_TYPE)type) != 
                m_RequestHandlers->end()))
            {
                WSDDHandlerList &list = 
                    (*m_RequestHandlers)[(AXIS_PROTOCOL_TYPE)type];
                if (fputs("\t\t<requestFlow>\n", wsddfile) < 0) 
                    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 4,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

                for(iter2 = list.begin(); iter2 != list.end(); iter2++)
                {
                    if (AXIS_SUCCESS != 
                        (*iter2)->updateWSDD(wsddfile, tabcount)) 
                        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 5,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

                }            
                if (fputs("\t\t</requestFlow>\n", wsddfile) < 0) 
                    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 6,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

            }
            /* Write response flow handler configuration */
            if(m_ResponseHandlers &&
                (m_ResponseHandlers->find((AXIS_PROTOCOL_TYPE)type) != 
                m_ResponseHandlers->end()))
            {
                WSDDHandlerList &list = 
                    (*m_ResponseHandlers)[(AXIS_PROTOCOL_TYPE)type];
                if (fputs("\t\t<responseFlow>\n", wsddfile) < 0) 
                    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 7,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

                for(iter2 = list.begin(); iter2 != list.end(); iter2++)
                {
                    if (AXIS_SUCCESS != 
                        (*iter2)->updateWSDD(wsddfile, tabcount)) 
                        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 8,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

                }            
                if (fputs("\t\t</responseFlow>\n", wsddfile) < 0) 
                    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 9,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

            }
            if (fputs("\t</transport>\n", wsddfile) < 0) 	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 10,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("WSDDTransport", "updateWSDD", this, 11,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

