#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/***************************************************************************
 *
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.                                              
 *                                                                         
 ***************************************************************************/

/*
 *    @author Susantha Kumara (susantha@opensource.lk)
 *    @author Dasarath Weerathunga
 */

#include "XmlPullParser.hpp"
 
void XmlPullParser::relocateTokens(int ioffset)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "relocateTokens", this, 1,
					TRACETYPE_INT, 0, ((void*)&ioffset));	  /* AUTOINSERTED TRACE */
	#endif

	int size= token->count();
	for (int ii= 0; ii < size; ii++)
		(*token)[ii].relocate(ioffset);	
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "relocateTokens", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XmlPullParser::shift()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "shift", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	memmove(buffer->get(), buffer->get()+offset, _next-offset);
	_next -= offset;
	last -= offset;
	offset= 0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "shift", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int XmlPullParser::read()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "read", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (_next == buffer->getSize()) {
		if (offset > 0) {
			relocateTokens(offset);
			shift();
		}
		else {
			char *p= buffer->get();
			buffer->grow();
			relocateTokens(p - buffer->get());
		}
	}
	int c= s->read(buffer->get(), _next, buffer->getSize()-_next);
	last += c;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (!c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "read", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return !c;
		#endif
	}

}

int XmlPullParser::nextCh(int eof)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "nextCh", this, 1,
					TRACETYPE_INT, 0, ((void*)&eof));	  /* AUTOINSERTED TRACE */
	#endif

	if (_next > last && read())
		if (eof)
				{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "nextCh", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

		else
			throw new XmlPullParserException();
	//Update next 
	// FIXME:
	//printf("%c", buffer->get()[_next]);
	
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (buffer->get()[_next++]);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "nextCh", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return buffer->get()[_next++];
		#endif
	}

	//update next depending on the encoding type
}
 
char *XmlPullParser::lastCh()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "lastCh", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (buffer->get()+_next-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "lastCh", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return buffer->get()+_next-1;
		#endif
	}

}

void XmlPullParser::reset()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "reset", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	offset= _next;
	token->trunc(0);
	attr->trunc(0);
	if (event == ETag || event == EmptyElemTag) {
		DEPTH &l= depth->getLast();
		if (l.count) {
			NAMESPACE *p= ns->get()+l.first;
			for (int ii= l.first; ii < l.total; ii++, p++) {
				free(p->prefix);
				free(p->uri);
			}
			ns->trunc(l.first);
		}
		depth->chip();
//		printf("\ndepth=%d\n", depth->count());
	}
	name= -1;
	_ns= -1;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "reset", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XmlPullParser::startToken()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "startToken", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	TOKEN &t= token->append();
	t.type= TOKEN::Unknown;
	t.start= lastCh();	
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "startToken", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}
	
void XmlPullParser::endToken(int type)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "endToken", this, 1,
					TRACETYPE_INT, 0, ((void*)&type));	  /* AUTOINSERTED TRACE */
	#endif

	TOKEN &t= token->getLast();
	t.type= type;
	t.end= lastCh()-1;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "endToken", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

XmlPullParser::XmlPullParser(InputStream *pStream)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "XmlPullParser", this, 1,
					TRACETYPE_POINTER, sizeof(InputStream), ((void*)&pStream));	  /* AUTOINSERTED TRACE */
	#endif

	this->s= pStream;
	state= S_1;
	offset= 0;
	last= -1;
	_next= 0;
	buffer= new Buffer<char>(1024);
	token= new Array<TOKEN>(64);
	ns= new Array<NAMESPACE>(64);
	depth= new Array<DEPTH>(16);
	attr= new Array<ATTR>(64);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "XmlPullParser", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

XmlPullParser::~XmlPullParser()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "~XmlPullParser", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	delete buffer;
	delete token;
	delete ns;
	delete depth;
	delete attr;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "~XmlPullParser", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int XmlPullParser::isS(int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "isS", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0x9 == c || 0xA == c || 0xD == c || 0x20 == c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "isS", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0x9 == c || 0xA == c || 0xD == c || 0x20 == c;
		#endif
	}

}

int XmlPullParser::parseS(int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseS", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

	while (isS(c))
		c= nextCh();
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseS", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c;
		#endif
	}

}

int XmlPullParser::parseEq(int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseEq", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

	if ('=' == parseS(c))
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (parseS(nextCh()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseEq", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return parseS(nextCh());
		#endif
	}

	else
		throw new XmlPullParserException();
}

int XmlPullParser::parseNameToken(int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseNameToken", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

	startToken();
	while (!(isS(c) || '/' == c || '?' == c || '=' == c || '>' == c)) {
		if (':' == c) {
			endToken(TOKEN::Prefix);
			c= nextCh();
			startToken();
		}
		else
			c= nextCh();
	}
	endToken(TOKEN::Name);
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseNameToken", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c;
		#endif
	}

}

int XmlPullParser::parseAttValueToken(int quote)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseAttValueToken", this, 1,
					TRACETYPE_INT, 0, ((void*)&quote));	  /* AUTOINSERTED TRACE */
	#endif

	if ('\'' == quote || '"' == quote) {
		int c= nextCh();
		startToken();
		while ('&' != c && '<' != c) {
			if (c == quote) {
				endToken(TOKEN::AttValue);
					{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (nextCh());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseAttValueToken", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return nextCh();
		#endif
	}

			}		
			c= nextCh();
		}
	}
	throw new XmlPullParserException();
}

int XmlPullParser::parseAttribute(int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseAttribute", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

	c= parseEq(parseNameToken(c));
	c= parseAttValueToken(c);
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (parseS(c));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseAttribute", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return parseS(c);
		#endif
	}

}

void XmlPullParser::parseXMLDecl()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseXMLDecl", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (nextCh() == 'x' &&
		nextCh() == 'm' &&
		nextCh() == 'l') {
		int c= parseAttribute(parseS(nextCh()));
		if (c == 'e')
			c= parseAttribute(c);
		if (c == 's')
			c= parseAttribute(c);
		if (c == '?' && nextCh() == '>') {
			event= XMLDecl;
				{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseXMLDecl", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

		}
	}
	throw new XmlPullParserException();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseXMLDecl", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XmlPullParser::parseSTagOrEmptyElemTag(int c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseSTagOrEmptyElemTag", this, 1,
					TRACETYPE_INT, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

	c= parseS(parseNameToken(c));
	event= STag;
	for (;;) {
		if (c == '/') {
			event= EmptyElemTag;
			if (nextCh() == '>')
					{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseSTagOrEmptyElemTag", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

			else
				throw new XmlPullParserException();
		}
		else
			if (c == '>')
					{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseSTagOrEmptyElemTag", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

			else
				c= parseAttribute(c);
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseSTagOrEmptyElemTag", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void XmlPullParser::parseETag()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseETag", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (parseS(parseNameToken(nextCh())) == '>')
		event= ETag;
	else
		throw new XmlPullParserException();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseETag", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int XmlPullParser::parseCharData()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parseCharData", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	int c;
	event= Content;
	startToken();
	do {
		c= nextCh(-1);
		if (-1 == c) {
			endToken(TOKEN::CharData);
				{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseCharData", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

		}
	} while ('<' != c);
	endToken(TOKEN::CharData);
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (c);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parseCharData", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return c;
		#endif
	}

}

int XmlPullParser::parse()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "parse", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	do {
		int c= nextCh(-1);
		if (-1 == c)
				{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parse", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

		switch (state) {
		case S_1:
			if ('<' == c)
				state= S_2;
			else
				state= S_0;
			break;
		case S_2:
			if ('?' == c)
				parseXMLDecl();
			else
				parseSTagOrEmptyElemTag(c);
			state= S_3;
			break;
		case S_3:
			if ('<' == c)
				state= S_4;
			else {
				c= parseCharData();
				if ('<' == c) {
					state= S_4;
						{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (event);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parse", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return event;
		#endif
	}

				}
				else if ('\0' == c)
					state= S_3;
				else
					state= S_0;
			}
			break;
		case S_4:
			if ('/' == c) {
				parseETag();
				state= S_3;
			}
			else if ('!' == c)
				state= S_0;
			else if ('?' == c)
				state= S_0;
			else {
				parseSTagOrEmptyElemTag(c);
				state= S_3;
			}
		}
		if (state == S_0)
			throw new XmlPullParserException();
	} while (state != S_3);
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (event);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "parse", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return event;
		#endif
	}

}

int XmlPullParser::next()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "next", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	reset();
	if (parse() != -1) {
		if (STag == event || EmptyElemTag == event || ETag == event) {
			TOKEN *p= token->get();
			int ii= 0;
			if (p->type == TOKEN::Prefix) {
				_ns= 0xffff;
				name= 1;
				p += 2;
				ii += 2;
			}
			else {
				_ns = -1;
				name= 0;
				p++;
				ii++;
			}
			if (ETag == event)
					{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (event);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "next", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return event;
		#endif
	}

			int count= token->count();
			while (ii < count) {
				if (p->type == TOKEN::Prefix) {
					if (!p->compare("xmlns", 5)) {
						NAMESPACE &n= ns->append();
						p++;
						n.prefix= p->toString();
						p++;
						n.uri= p->toString();
						p++;
						ii += 3;					
					}
					else {
						ATTR &a= attr->append();
						a.prefix= ii++;
						a.name= ii++;
						a.value= ii++;
						a.ns= -1;
						p += 3;
					}					
				}
				else {
					if (!p->compare("xmlns", 5)) {
						NAMESPACE &n= ns->append();
						n.prefix= NULL;
						p++;
						n.uri= p->toString();
						p++;
						ii += 2;					
					}
					else {
						ATTR &a= attr->append();
						a.prefix= -1;
						a.name= ii++;
						a.value= ii++;
						a.ns= -1;						
						p += 2;
					}
				}
			}
			if (depth->count()) {
				DEPTH &l= depth->getLast();			
				DEPTH &m= depth->append();
				m.first= l.first+l.count;
				m.total= ns->count();
				m.count= m.total-l.total;			
			}
			else {
				DEPTH &m= depth->append();
				m.first= 0;
				m.total= ns->count();
				m.count= m.total;			
			}
			_ns= resolve(_ns == -1 ? NULL : token->get());
		}
        else if (XMLDecl == event) 
        {
            //Do not return this event. Process it for internal use only
            //Get and return next event.
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (next());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "next", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return next();
		#endif
	}

        }
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (event);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "next", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return event;
		#endif
	}

	}
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "next", this, 4,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

}

int XmlPullParser::getAttributeCount()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getAttributeCount", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (STag != event && EmptyElemTag != event)
		throw new XmlPullParserException();
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (attr->count());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributeCount", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return attr->count();
		#endif
	}
	
}

char *XmlPullParser::getAttributePrefix(int ii)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getAttributePrefix", this, 1,
					TRACETYPE_INT, 0, ((void*)&ii));	  /* AUTOINSERTED TRACE */
	#endif

	if (STag != event && EmptyElemTag != event)
		throw new XmlPullParserException();
	ii= (*attr)[ii].prefix;
	if (ii == -1)
			{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributePrefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

		{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((*token)[ii].toString());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributePrefix", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*token)[ii].toString();
		#endif
	}
	
}

char *XmlPullParser::getAttributeName(int ii)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getAttributeName", this, 1,
					TRACETYPE_INT, 0, ((void*)&ii));	  /* AUTOINSERTED TRACE */
	#endif

	if (STag != event && EmptyElemTag != event)
		throw new XmlPullParserException();
	ii= (*attr)[ii].name;
	if (ii == -1)
			{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributeName", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

		{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((*token)[ii].toString());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributeName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*token)[ii].toString();
		#endif
	}
	
}

char *XmlPullParser::getAttributeValue(int ii)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getAttributeValue", this, 1,
					TRACETYPE_INT, 0, ((void*)&ii));	  /* AUTOINSERTED TRACE */
	#endif

	if (STag != event && EmptyElemTag != event)
		throw new XmlPullParserException();
	ii= (*attr)[ii].value;
	if (ii == -1)
			{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributeValue", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

		{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((*token)[ii].toString());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributeValue", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*token)[ii].toString();
		#endif
	}
	
}

char *XmlPullParser::getAttributeNamespaceUri(int ii)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getAttributeNamespaceUri", this, 1,
					TRACETYPE_INT, 0, ((void*)&ii));	  /* AUTOINSERTED TRACE */
	#endif

	if (STag != event && EmptyElemTag != event)
		throw new XmlPullParserException();
	ATTR &t= (*attr)[ii];
	if (t.ns == -1)
		t.ns= resolve(t.prefix == -1 ? NULL : token->get()+t.prefix);
    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((t.ns == -1) ? NULL : strdup((*ns)[t.ns].uri));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getAttributeNamespaceUri", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (t.ns == -1) ? NULL : strdup((*ns)[t.ns].uri);
		#endif
	}

}

int XmlPullParser::resolve(TOKEN *prefix)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "resolve", this, 1,
					TRACETYPE_POINTER, sizeof(TOKEN), ((void*)&prefix));	  /* AUTOINSERTED TRACE */
	#endif

	int ii= ns->count()-1;
	NAMESPACE *p= &ns->getLast();
	while (ii >= 0) {
		if (prefix == NULL) {
			if (p->prefix == NULL)
					{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (ii);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "resolve", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return ii;
		#endif
	}

		}
		else
			if (p->prefix && !prefix->compare(p->prefix, strlen(p->prefix)))
					{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (ii);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "resolve", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return ii;
		#endif
	}

		ii--;
		p--;
	}
//FIXME : Commented out following line because the parser fails when there are 
//        unqualified elements in xml which is IMO possible in SOAP - Susantha 
//        11/11/2004
//	throw new XmlPullParserException();
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "resolve", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}
 //not resolved
}

int XmlPullParser::getDepth()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getDepth", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (depth->count());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getDepth", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return depth->count();
		#endif
	}

}

int XmlPullParser::getNamespaceCount(int ii)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespaceCount", this, 1,
					TRACETYPE_INT, 0, ((void*)&ii));	  /* AUTOINSERTED TRACE */
	#endif

	if (ii > getDepth())
		throw new XmlPullParserException();
	else
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = ((*depth)[ii-1].total);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespaceCount", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*depth)[ii-1].total;
		#endif
	}

}

char *XmlPullParser::getNamespacePrefix(int ii)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespacePrefix", this, 1,
					TRACETYPE_INT, 0, ((void*)&ii));	  /* AUTOINSERTED TRACE */
	#endif

	if (ii > ns->count())
		throw new XmlPullParserException();
	else {
		char *p= (*ns)[ii].prefix;
			{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (p == NULL ? NULL : strdup(p));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespacePrefix", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return p == NULL ? NULL : strdup(p);
		#endif
	}
 
	}
}
	
char *XmlPullParser::getNamespaceUri(int ii)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespaceUri", this, 1,
					TRACETYPE_INT, 0, ((void*)&ii));	  /* AUTOINSERTED TRACE */
	#endif

	if (ii > ns->count())
		throw new XmlPullParserException();
	else {
		char *p= (*ns)[ii].uri;
			{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (p == NULL ? NULL : strdup(p));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespaceUri", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return p == NULL ? NULL : strdup(p);
		#endif
	}
 
	}
}

char *XmlPullParser::getName()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((*token)[name].toString());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getName", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*token)[name].toString();
		#endif
	}

}

char *XmlPullParser::getCharData()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getCharData", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = ((*token)[0].toString());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getCharData", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*token)[0].toString();
		#endif
	}

}

char *XmlPullParser::getNamespaceUri()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespaceUri", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if (_ns == -1)
		_ns= resolve(NULL);
	else
		if (_ns == 0xffff)
			_ns= resolve(token->get());
        	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (_ns == -1 ? NULL : strdup((*ns)[_ns].uri));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespaceUri", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return _ns == -1 ? NULL : strdup((*ns)[_ns].uri);
		#endif
	}

}

char* XmlPullParser::getNamespace4Prefix(const char *pcPrefix)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getNamespace4Prefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcPrefix));	  /* AUTOINSERTED TRACE */
	#endif

    for (int ix=ns->count()-1; ix >= 0; ix--)
    {
        if ((*ns)[ix].prefix && (0 == strcmp(pcPrefix, (*ns)[ix].prefix)))
        {
            	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (strdup((*ns)[ix].uri));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespace4Prefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strdup((*ns)[ix].uri);
		#endif
	}

        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getNamespace4Prefix", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

char* XmlPullParser::getPrefix4Namespace(const char *pcNs)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XmlPullParser", "getPrefix4Namespace", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcNs));	  /* AUTOINSERTED TRACE */
	#endif

    for (int ix=ns->count()-1; ix >= 0; ix--)
    {
        if ((*ns)[ix].uri && (0 == strcmp(pcNs, (*ns)[ix].uri)))
        {
            	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (strdup((*ns)[ix].prefix));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getPrefix4Namespace", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strdup((*ns)[ix].prefix);
		#endif
	}

        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XmlPullParser", "getPrefix4Namespace", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

