#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

/*
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 * @author Damitha Kumarage (damitha@opensource.lk, damitha@jkcsworld.com)
 * @author Roshan Weerasuriya (roshan@opensource.lk, roshanw@jkcsworld.com)
 * @author Samisa Abeysinghe (sabeysinghe@virtusa.com)
 *
 */

/* SoapDeSerializer.cpp: implementation of the SoapDeSerializer class. */

/*
 * Revision 1.1  2004/06/10 roshan
 * Fixed the bug AXISCPP-95 at JIRA.
 */

/*
 * Revision 1.2  2004/07/22 roshan
 * Changed code to support proper parsing of incoming SOAP Headers. The 
 *  SoapDeSerializer::getHeader() method was completely rewriten.
 */

/*
 * Revision 1.3  2004/11/27 roshan
 * Added the implementation of the setNewSoapBody() method.
 */

#ifdef WIN32

#pragma warning (disable : 4101)

#endif

#include "SoapDeSerializer.h"

#include "SoapEnvelope.h"

#include "SoapKeywordMapping.h"

#include "URIMapping.h"

#include "SoapHeader.h"

#include "SoapMethod.h"

#include "SoapBody.h"

#include "SoapFault.h"

#include "ComplexElement.h"

#include "CharacterElement.h"

#include <axis/GDefine.hpp>

#include "../common/AxisTrace.h"

#include "apr_base64.h"

#include "HexCoder.h"

#include "../engine/XMLParserFactory.h"

#include "../xml/XMLParser.h"

#include "../xml/QName.h"

#include "AxisSoapException.h"

#include "../common/AxisGenException.h"

#include "../common/AxisTrace.h"

#include <axis/Axis.hpp>

#include <list>

#include <iostream>

AxisChar * pBodyContent;

AXIS_CPP_NAMESPACE_START

#define INITIAL_ARRAY_SIZE 1
    SoapDeSerializer::SoapDeSerializer ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "SoapDeSerializer", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_pParser = XMLParserFactory::getParserObject ();
    m_pEnv = NULL;
    m_pHeader = NULL;
    m_pInputStream = 0;
    m_nStatus = AXIS_SUCCESS;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "SoapDeSerializer", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

SoapDeSerializer::~SoapDeSerializer ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "~SoapDeSerializer", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pEnv)
	delete m_pEnv;
    if (m_pHeader)
	delete m_pHeader;
    if (m_pParser)
    {
	XMLParserFactory::destroyParserObject (m_pParser);
	//delete m_pParser;
    }
	if(pBodyContent)
		delete pBodyContent;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "~SoapDeSerializer", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int
SoapDeSerializer::setInputStream (SOAPTransport * pInputStream)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "setInputStream", this, 1,
					TRACETYPE_POINTER, sizeof(SOAPTransport), ((void*)&pInputStream));	  /* AUTOINSERTED TRACE */
	#endif

    if (NULL == pInputStream)
    {
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "setInputStream", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

    }
    m_pInputStream = pInputStream;
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_pParser->setInputStream (pInputStream));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "setInputStream", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pParser->setInputStream (pInputStream);
		#endif
	}

}

SoapEnvelope *
SoapDeSerializer::getEnvelope ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getEnvelope", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    Attribute *pAttr = NULL;
    if (!m_pNode)
    {
	m_pNode = m_pParser->next ();
    }
    if (!m_pNode || (START_ELEMENT != m_pNode->m_type))
    {
		{
		#ifdef ENABLE_AXISTRACE
			SoapEnvelope* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getEnvelope", this, 1,
					TRACETYPE_POINTER, sizeof(SoapEnvelope), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    }
    if (0 == strcmp (m_pNode->m_pchNameOrValue,
		     SoapKeywordMapping::map (SOAP_VER_1_2).
		     pchWords[SKW_ENVELOPE]))
    {
	SoapEnvelope *m_pEnvl = new SoapEnvelope ();
	/* set all attributes of SoapEnvelope */
    std::list<Attribute*> attributeList0;
	pAttr = new Attribute ( attributeList0);
	pAttr->setValue (m_pNode->m_pchNamespace);
	m_pEnvl->addNamespaceDecl (pAttr);
	if (0 == strcmp (m_pNode->m_pchNamespace,
			 SoapKeywordMapping::map (SOAP_VER_1_1).
			 pchNamespaceUri))
	{
	    m_nSoapVersion = SOAP_VER_1_1;
	}
	else if (0 == strcmp (m_pNode->m_pchNamespace,
			      SoapKeywordMapping::map (SOAP_VER_1_2).
			      pchNamespaceUri))
	{
	    m_nSoapVersion = SOAP_VER_1_2;
	}
	else
	{
	    m_nSoapVersion = VERSION_LAST;
	}
	/* Set Attributes */
	for (int i = 0; m_pNode->m_pchAttributes[i]; i += 3)
	{
        std::list<Attribute*> attributeList1;
	    pAttr = new Attribute (attributeList1);
	    pAttr->setLocalName (m_pNode->m_pchAttributes[i]);
	    pAttr->setURI (m_pNode->m_pchAttributes[i + 1]);
	    pAttr->setValue (m_pNode->m_pchAttributes[i + 2]);
	    m_pEnvl->addAttribute (pAttr);
	}
	m_pNode = NULL;		/* This is to indicate that node is identified 
				 * and used */
		{
		#ifdef ENABLE_AXISTRACE
			SoapEnvelope* traceRet = (m_pEnvl);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getEnvelope", this, 2,
					TRACETYPE_POINTER, sizeof(SoapEnvelope), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pEnvl;
		#endif
	}

    }
    	{
		#ifdef ENABLE_AXISTRACE
			SoapEnvelope* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getEnvelope", this, 3,
					TRACETYPE_POINTER, sizeof(SoapEnvelope), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

/*
 * Header block parsing should be done differently for different SOAP versions.
 * Since the Header blocks may be requested in a different order than it is in 
 * the SOAP stream there must be a mechanism to manage the situation. Possibly 
 * to re-parse the stream part which has unused header blocks.
 */

IHeaderBlock *
SoapDeSerializer::getHeaderBlock (const AxisChar * pName,
				  const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getHeaderBlock", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    if (!m_pHeader)
    {
		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getHeaderBlock", this, 1,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}
		/* there has been no <Header> element so there
				 * can be no Header blocks */
    }
    	{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (m_pHeader->getHeaderBlock (pName, pNamespace, true));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getHeaderBlock", this, 2,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pHeader->getHeaderBlock (pName, pNamespace, true);
		#endif
	}

}

int
SoapDeSerializer::getHeader ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getHeader", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pHeader)
    {
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getHeader", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

    }
    m_pNode = m_pParser->next ();
    if (!m_pNode)		/* this means a SOAP error */
    {
	m_nStatus = AXIS_FAIL;
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getHeader", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

    }
    if ((START_ELEMENT == m_pNode->m_type) &&
	(0 == strcmp (m_pNode->m_pchNameOrValue,
		      SoapKeywordMapping::map (m_nSoapVersion).
		      pchWords[SKW_HEADER])))
    {
	if (m_pHeader)
	{
	    delete m_pHeader;
	}
	m_pHeader = new SoapHeader ();
	/* Set any attributes/namspaces to the SoapHeader object */
	bool blnMainLoopContStatus = true;
	while (blnMainLoopContStatus)
	{
	    m_pNode = m_pParser->next (true);
	    if (!m_pNode)
	    {
		m_nStatus = AXIS_FAIL;
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getHeader", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

	    }
	    if ((END_ELEMENT == m_pNode->m_type) &&
		(0 == strcmp (m_pNode->m_pchNameOrValue,
			      SoapKeywordMapping::map (m_nSoapVersion).
			      pchWords[SKW_HEADER])))
	    {
		m_pNode = NULL;	/* This is to indicate that node is identified
				 * and used */
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getHeader", this, 4,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

		break;
	    }
	    /* The following is done to ignore anything (eg : the toplevel 
	       whitespaces) but a start element. */
	    if (START_ELEMENT != m_pNode->m_type)
		continue;
	    HeaderBlock *pHeaderBlock = new HeaderBlock ();
	    if (m_pNode->m_pchNamespace)
	    {
		pHeaderBlock->setURI (m_pNode->m_pchNamespace);
	    }
	    pHeaderBlock->setLocalName (m_pNode->m_pchNameOrValue);
	    if ((m_pNode->m_pchAttributes[0]) != NULL)
	    {
		int iAttributeArrayIndex = 0;
		while (true)
		{
            std::list<Attribute*> attributeList2;
		    Attribute *pAttribute = new Attribute (attributeList2);
		    pAttribute->setLocalName (m_pNode->
					      m_pchAttributes
					      [iAttributeArrayIndex++]);
		    pAttribute->setURI (m_pNode->
					m_pchAttributes
					[iAttributeArrayIndex++]);
		    pAttribute->setValue (m_pNode->
					  m_pchAttributes
					  [iAttributeArrayIndex++]);
		    pHeaderBlock->addAttribute (pAttribute);
		    if (m_pNode->m_pchAttributes
			[iAttributeArrayIndex] == '\0')
		    {
			break;
		    }
		}
	    }
	    BasicNode **pNodeList = new BasicNode *[10];
	    int iListPos = 0;
	    int iLevel = 0;
	    bool bContinue = true;
	    while (bContinue)
	    {
		m_pNode = m_pParser->next (true);
		if (END_ELEMENT == m_pNode->m_type)
		{
		    if (iLevel == 0)
		    {
			//do nothing
			m_pHeader->addHeaderBlock (pHeaderBlock);
			bContinue = false;
		    }
		    else if (iLevel == 1)
		    {
			if ((pNodeList[0]) != NULL)
			{
			    //addToHeaderBlock(pNodeList[0]);
			    pHeaderBlock->addChild (pNodeList[0]);
                /* Commented by Rangika since with this part the logic is wrong
			    //m_pHeader->addHeaderBlock (pHeaderBlock);
			    //m_pParser->next (true);	//To skip the end element of the HeaderBlock
			    //bContinue = false;
                */
                iListPos--;
			    iLevel--;
			}
		    }
		    else
		    {
			//addToImmediateParent(pNodeList, iListPos);
			(pNodeList[iListPos - 2])->
			    addChild (pNodeList[iListPos - 1]);
			iListPos--;
			iLevel--;
		    }
		}
		else if (START_ELEMENT == m_pNode->m_type)
		{
		    //createBaisNode and setValue
		    BasicNode *pComplexElement = new ComplexElement ();
		    if (m_pNode->m_pchNamespace)
		    {
			pComplexElement->setURI (m_pNode->m_pchNamespace);
		    }
		    pComplexElement->setLocalName (m_pNode->m_pchNameOrValue);
		    //addtoList
		    pNodeList[iListPos] = pComplexElement;
		    iListPos++;
		    iLevel++;
		}
		else if (CHARACTER_ELEMENT == m_pNode->m_type)
		{
		    //createBasicNode and setValue
		    BasicNode *pBasicNode =
			new CharacterElement (m_pNode->m_pchNameOrValue);
		    //addToImmidiateParent
		    if (iLevel == 0)
		    {
			pHeaderBlock->addChild (pBasicNode);
		    }
		    else
		    {
			(pNodeList[iListPos - 1])->addChild (pBasicNode);
		    }
		}
	    } // end while
            
            // Samisa: clean memory
            delete [] pNodeList;
	}
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getHeader", this, 5,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

}

//Chinthana:This method adds to allow "all" & "choice" constructs to peek head next element.

const char*
SoapDeSerializer::peekNextElementName ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "peekNextElementName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		
	if (m_pParser)
	
	{
		const char* name = m_pParser->peek ();
			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (name);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "peekNextElementName", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return name;
		#endif
	}

	}
			
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "peekNextElementName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

	
}

//28/04/2005

int
SoapDeSerializer::getBody ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getBody", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (!m_pNode)
    {
	m_pNode = m_pParser->next ();
    }
    /* previous header searching may have left a node unidentified */
    if (m_pNode)
    {
	if ((START_ELEMENT == m_pNode->m_type) &&
	    (0 == strcmp (m_pNode->m_pchNameOrValue,
			  SoapKeywordMapping::map (m_nSoapVersion).
			  pchWords[SKW_BODY])))
	{
	    /* Set any attributes/namspaces to the SoapBody object */
	    m_pNode = NULL;	/* This is to indicate that node is identified 
				 * and used */
            // peek for the method name
            m_strMethodNameToInvoke = m_pParser->peek();
	    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBody", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

	}
    }
    m_nStatus = AXIS_FAIL;
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBody", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

}

int
SoapDeSerializer::checkMessageBody (const AxisChar * pName,
				    const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "checkMessageBody", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    /* check and skip the soap body tag */
    /*if (AXIS_SUCCESS != getBody ())
    {
	//return AXIS_FAIL;    
	throw AxisGenException (SERVER_UNKNOWN_ERROR);
    }*/
    if (!m_pNode)
    {
	m_pNode = m_pParser->next ();
    }
    if (!m_pNode || (START_ELEMENT != m_pNode->m_type))
    {
	//return AXIS_FAIL;
	throw AxisGenException (SERVER_UNKNOWN_ERROR);
    }
    //if (0 != strcmp(m_pNode->m_pchNameOrValue, pName)) return AXIS_FAIL;
    if (0 != strcmp (m_pNode->m_pchNameOrValue, pName))
    {
	/* A soap fault has occurred. we will deserialize it as doc literal
	 * So we set the style as doc literal. This way of doing things
	 * is not so nice. I'll rectify this asap
	 */
	setStyle (DOC_LITERAL);
	AXISTRACE1 ("AXISC_NODE_VALUE_MISMATCH_EXCEPTION", CRITICAL);
	throw AxisGenException (AXISC_NODE_VALUE_MISMATCH_EXCEPTION);
    }
    /* we can check the namespace uri too here. Should we ? */
    m_pNode = NULL;		/*This is to indicate that node is identified and used */
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "checkMessageBody", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

void *
SoapDeSerializer::checkForFault (const AxisChar * pName,
				 const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "checkForFault", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    const char *pcCmplxFaultName;
    char *pcDetail;
    char *pcFaultCode;
    char *pcFaultstring;
    char *pcFaultactor;
    if (0 == strcmp ("Fault", pName))
    {
		
	if (0 != strcmp (m_pNode->m_pchNameOrValue, pName))
	{
	    m_nStatus = AXIS_SUCCESS;
	    m_pNode = NULL;
	    AXISTRACE1 ("AXISC_NODE_VALUE_MISMATCH_EXCEPTION", CRITICAL);
	    throw AxisGenException (AXISC_NODE_VALUE_MISMATCH_EXCEPTION);
	}
	/* we can check the namespace uri too here. Should we ? */
	m_nStatus = AXIS_SUCCESS;
	m_pNode = NULL;		/*This is to indicate that node is identified and used */
	SoapFault *pFault = new SoapFault ();
	pFault->setDeSerializer (this);
	m_nStyle = getStyle ();
	/*We deserialize fault code in doc literal. */
	setStyle (DOC_LITERAL);
	pcFaultCode = getElementAsString ("faultcode", 0);
	pFault->setFaultcode (pcFaultCode == NULL ? "" : pcFaultCode);
        if ( pcFaultCode )
            delete [] pcFaultCode;
	
	pcFaultstring = getElementAsString ("faultstring", 0);
	pFault->setFaultstring (pcFaultstring == NULL ? "" : pcFaultstring);
        if ( pcFaultstring )
            delete [] pcFaultstring;
 
	pcFaultactor = getElementAsString ("faultactor", 0);
	pFault->setFaultactor (pcFaultactor == NULL ? "" : pcFaultactor);
        if ( pcFaultactor )
            delete [] pcFaultactor;
	// FJP Changed the namespace from null to a single space (an impossible
	//     value) to help method know that it is parsing a fault message.
	pcDetail = getElementAsString ("detail", " ");
	
	if (pcDetail)
	{
	    pFault->setFaultDetail (pcDetail);
			delete [] pcDetail;          
	}
	else
	{
		pcCmplxFaultName = getCmplxFaultObjectName ();
	   	pFault->setCmplxFaultObjectName (pcCmplxFaultName == NULL ? "" : pcCmplxFaultName);
        /*    if ( pcCmplxFaultName )
                delete [] (reinterpret_cast <char *> (pcCmplxFaultName) );
		*/
	}
	setStyle (m_nStyle);
		{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (pFault);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "checkForFault", this, 0,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pFault;
		#endif
	}

    }
    else
    {
	throw AxisGenException (AXISC_NODE_VALUE_MISMATCH_EXCEPTION);
    }
}

int
SoapDeSerializer::init ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "init", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_pNode = 0;
    if (m_pEnv)
    {
	delete m_pEnv;
	m_pEnv = NULL;
    }
    if (m_pHeader)
    {
	delete m_pHeader;
	m_pHeader = NULL;
    }
    m_nSoapVersion = VERSION_LAST;
    m_nStatus = AXIS_SUCCESS;
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "init", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

}

int
SoapDeSerializer::getVersion ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getVersion", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (VERSION_LAST == m_nSoapVersion)
    {
	if (m_pEnv)
	{
	    delete m_pEnv;
	}
	m_pEnv = getEnvelope ();
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nSoapVersion);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getVersion", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nSoapVersion;
		#endif
	}

}

/*
 * In rpc/encoded style the stream is as follows,
 * <abc:ArrayOfPoints xmlns:abc="http://www.opensource.lk/Points"
 *    xmlns:enc="http://www.w3.org/2001/06/soap-encoding"
 *    enc:arrayType="xsd:Point[2]">
 *    <abc:Point><x>5</x><y>6</y></abc:Point>
 *    <abc:Point><x>7</x><y>8</y></abc:Point>
 * </abc:ArrayOfPoints>
 * But in doc/literal style it is,
 *    <points><x>5</x><y>6</y></points>
 *    <points><x>7</x><y>8</y></points>
 *
 */
 
void SoapDeSerializer::deserializeLiteralComplexArray(Axis_Array * pArray, void *pDZFunct,
                void *pCreFunct, void *pDelFunct, void* pSizeFunct,
                const AxisChar * pName, const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "deserializeLiteralComplexArray", NULL, 7,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&pArray),
					TRACETYPE_POINTER, 1, ((void*)&pDZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pCreFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, 1, ((void*)&pSizeFunct),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    while(true)
    {
        const char* elementName = peekNextElementName();
        if(strcmp(elementName, pName) == 0)
        {
            pArray->addElement(getCmplxObject(pDZFunct, pCreFunct, pDelFunct, pName, pNamespace));
        }
        else
        {
            	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "deserializeLiteralComplexArray", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        }
    }
}

void SoapDeSerializer::deserializeEncodedComplexArray(Axis_Array * pArray, void *pDZFunct,
                void *pCreFunct, void *pDelFunct, void* pSizeFunct,
                const AxisChar * pName, const AxisChar * pNamespace, int size)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "deserializeEncodedComplexArray", NULL, 8,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&pArray),
					TRACETYPE_POINTER, 1, ((void*)&pDZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pCreFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, 1, ((void*)&pSizeFunct),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace),
					TRACETYPE_INT, 0, ((void*)&size));	  /* AUTOINSERTED TRACE */
	#endif

    for (int count = 0 ; count < size; count++)
    {
        const char* elementName = peekNextElementName();
        if(strcmp(elementName, pName) != 0)
        {
            pArray->addElement(getCmplxObject(pDZFunct, pCreFunct, pDelFunct, pName, pNamespace));
        }
        else
        {
            	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "deserializeEncodedComplexArray", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        }
    }
}

Axis_Array*
SoapDeSerializer::getCmplxArray ( Axis_Array* pArray, void *pDZFunct,
                void *pCreFunct, void *pDelFunct, void *pSizeFunct,
                const AxisChar * pName, const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getCmplxArray", this, 7,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&pArray),
					TRACETYPE_POINTER, 1, ((void*)&pDZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pCreFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_POINTER, 1, ((void*)&pSizeFunct),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    if (AXIS_SUCCESS != m_nStatus)
    {
        	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (pArray);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxArray", this, 1,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pArray;
		#endif
	}

        /* if anything has gone wrong
         * earlier just do nothing */
    }
    if (RPC_ENCODED == m_nStyle)
    {
        m_pNode = m_pParser->next ();
        /* just skip wrapper node with type info
         * Ex:<tns:QuoteInfoTypeArray xmlns:tns="http://www.getquote.org/test">
         */
        if (!m_pNode)
        {
            	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (pArray);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxArray", this, 2,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pArray;
		#endif
	}

        }
    
// Look for an empty array
        if (END_ELEMENT == m_pNode->m_type)
        {
            	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (pArray);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxArray", this, 3,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pArray;
		#endif
	}

        }
    
        int arraySize = getArraySize (m_pNode);
    
        if (arraySize == 0)
        {
            m_pNode = m_pParser->next ();   /* skip end element node too */
            	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (pArray);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxArray", this, 4,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pArray;
		#endif
	}

        }
        else if (arraySize > 0)
        {
            deserializeEncodedComplexArray(pArray, pDZFunct, pCreFunct, pDelFunct,
                pSizeFunct, pName, pNamespace, arraySize);
            
            if (m_nStatus != AXIS_FAIL)
            {
                	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (pArray);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxArray", this, 5,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pArray;
		#endif
	}

            }
        }
    }
    else
    {
        deserializeLiteralComplexArray(pArray, pDZFunct, pCreFunct, pDelFunct,
            pSizeFunct, pName, pNamespace);
        if (m_nStatus != AXIS_FAIL)
        {
            	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (pArray);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxArray", this, 6,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pArray;
		#endif
	}

        }
    }
    m_nStatus = AXIS_FAIL;
    m_pNode = NULL;
    	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (pArray);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxArray", this, 7,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pArray;
		#endif
	}

}

/*
 * Get Size of the single dimension array from arrayType attribute
 * Ex : enc:arrayType="xs:string[6]"
 */

int
SoapDeSerializer::getArraySize (const AnyElement * pElement)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getArraySize", this, 1,
					TRACETYPE_POINTER, sizeof(AnyElement), ((void*)&pElement));	  /* AUTOINSERTED TRACE */
	#endif

    int nSize = -1;
    /* first check whether this is a start element node */
    if (START_ELEMENT != pElement->m_type)
    {
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (nSize);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getArraySize", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return nSize;
		#endif
	}

    }
    for (int i = 0; pElement->m_pchAttributes[i]; i += 3)
    {
	if (URI_ENC == URIMapping::getURI (pElement->m_pchAttributes[i + 1])
	    && (0 == strcmp (pElement->m_pchAttributes[i],
			     SoapKeywordMapping::map (m_nSoapVersion).
			     pchWords[SKW_ARRAYTYPE])))
	{
	    QName qn;
	    qn.splitQNameString (pElement->m_pchAttributes[i + 2], '[');
	    nSize = strtol (qn.localname, &m_pEndptr, 10);
	    qn.mergeQNameString ('[');
	    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (nSize);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getArraySize", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return nSize;
		#endif
	}

	}
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (nSize);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getArraySize", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return nSize;
		#endif
	}

}

/*
 * In rpc/encoded style the stream is as follows,
 * <abc:ArrayOfPhoneNumbers xmlns:abc="http://example.org/2001/06/numbers"
 *    xmlns:enc="http://www.w3.org/2001/06/soap-encoding"
 *    enc:arrayType="xsd:string[2]">
 *    <item>+94-112-500500</item>
 *    <item>+94-112-500500</item>
 * </abc:ArrayOfPhoneNumbers>
 * But in doc/literal style it is,
 *    <phonenumbers>+94-112-500500</phonenumbers>
 *    <phonenumbers>+94-112-500500</phonenumbers> 
 *
 *
 */

void SoapDeSerializer::deserializeEncodedArray (Axis_Array* pArray, IAnySimpleType* pSimpleType, const AxisChar* pName, const AxisChar* pNamespace, int size)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "deserializeEncodedArray", NULL, 5,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&pArray),
					TRACETYPE_POINTER, sizeof(IAnySimpleType), ((void*)&pSimpleType),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace),
					TRACETYPE_INT, 0, ((void*)&size));	  /* AUTOINSERTED TRACE */
	#endif

    for (int count = 0 ; count < size; count++)
    {
        const char* elementName = peekNextElementName();
        if(strcmp(elementName, pName) != 0)
        {
            getElement(pName, pNamespace, pSimpleType, true);
            void * pValue = pSimpleType->getValue();
            pArray->addElement(pValue);
            Axis::AxisDelete(pValue, pSimpleType->getType());
        }
        else
        {
            	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "deserializeEncodedArray", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        }
    }
}

void SoapDeSerializer::deserializeLiteralArray (Axis_Array* pArray, IAnySimpleType* pSimpleType, const AxisChar* pName, const AxisChar* pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "deserializeLiteralArray", NULL, 4,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&pArray),
					TRACETYPE_POINTER, sizeof(IAnySimpleType), ((void*)&pSimpleType),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    while(true)
    {
        const char* elementName = peekNextElementName();
        if(strcmp(elementName, pName) == 0)
        {
    		getElement(pName, pNamespace, pSimpleType);
            void * pValue = pSimpleType->getValue();
            pArray->addElement(pValue);
            Axis::AxisDelete(pValue, pSimpleType->getType());
        }
        else
        {
            	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "deserializeLiteralArray", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        }
    }
}

Axis_Array*
SoapDeSerializer::getBasicArray (XSDTYPE nType,
				 const AxisChar * pName,
				 const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getBasicArray", this, 3,
					TRACETYPE_DATA, sizeof(XSDTYPE), ((void*)&nType),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Axis_Array* Array = new Axis_Array();
    Array->m_Type = nType;
    if (AXIS_SUCCESS != m_nStatus)
    {
    		{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (Array);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBasicArray", this, 1,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Array;
		#endif
	}

    }
    /* if anything has gone wrong earlier just do nothing */
    if (RPC_ENCODED == m_nStyle)
	{
		m_pNode = m_pParser->next ();
	
		/* just skip wrapper node with type info  Ex: <tns:ArrayOfPhoneNumbers
		 * xmlns:tns="http://www.getquote.org/test">
		 */
		if (!m_pNode)
		{
		    	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (Array);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBasicArray", this, 2,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Array;
		#endif
	}

		}
	
		int size = getArraySize (m_pNode);
	
		if (size == 0)
		{
			m_pNode = m_pParser->next ();	/* skip end element node too */
				{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (Array);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBasicArray", this, 3,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Array;
		#endif
	}

		}
		else if (size > 0)
        {
            IAnySimpleType* pSimpleType = AxisUtils::createSimpleTypeObject(nType);
            deserializeEncodedArray(Array, pSimpleType, pName, pNamespace, size);
            delete pSimpleType;
            
            if ( m_nStatus != AXIS_FAIL)
            {
                	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (Array);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBasicArray", this, 4,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Array;
		#endif
	}

            }        
        }
	}
    else
    {
        IAnySimpleType* pSimpleType = AxisUtils::createSimpleTypeObject(nType);
        deserializeLiteralArray(Array, pSimpleType, pName, pNamespace);
        delete pSimpleType;
        
        if ( m_nStatus != AXIS_FAIL)
        {
            	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (Array);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBasicArray", this, 5,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Array;
		#endif
	}

        }        
    }
    m_nStatus = AXIS_FAIL;
    m_pNode = NULL;
    	{
		#ifdef ENABLE_AXISTRACE
			Axis_Array* traceRet = (Array);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBasicArray", this, 6,
					TRACETYPE_POINTER, sizeof(Axis_Array), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Array;
		#endif
	}

}

/*
 * In doc/lit style pName is the name of the type instance "result"
 *  <result>
 *        <name>Widgets Inc.</name> 
 *        <symbol>WID</symbol> 
 *        <public>true</public> 
 *  </result>
 * But in rpc/encoded style pName is the name of the type "QuoteInfoType"
 *  <tns:QuoteInfoType xmlns:tns="http://www.getquote.org/test">
 *        <name xsi:type="xsd:string">Widgets Inc.</name> 
 *        <symbol xsi:type="xsd:string">WID</symbol> 
 *        <public xsi:type="xsd:boolean">true</public> 
 *  </tns:QuoteInfoType>
 *
 */

void *
SoapDeSerializer::getCmplxObject (void *pDZFunct, void *pCreFunct,
				  void *pDelFunct, const AxisChar * pName,
				  const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getCmplxObject", this, 5,
					TRACETYPE_POINTER, 1, ((void*)&pDZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pCreFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    if (AXIS_SUCCESS != m_nStatus)
		{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxObject", this, 1,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    /* if anything has gone wrong earlier just do nothing */
    if (RPC_ENCODED == m_nStyle)
    {
	m_pNode = m_pParser->next ();
	/* just skip wrapper node with type info
	 * Ex: <tns:QuoteInfoType xmlns:tns="http://www.getquote.org/test"> */
	if (!m_pNode)
	    	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxObject", this, 2,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

	/* type  can be checked here */
	TRACE_OBJECT_CREATE_FUNCT_ENTRY(pCreFunct, NULL, false, 0);
	void *pObject =
	    ((AXIS_OBJECT_CREATE_FUNCT) pCreFunct) (NULL, false, 0);
	TRACE_OBJECT_CREATE_FUNCT_EXIT(pCreFunct, pObject);
	if (pObject && pDZFunct)
	{
	    if (C_RPC_PROVIDER == getCurrentProviderType ())
	    {
		// Disable C support
		//IWrapperSoapDeSerializer_C cWSD;
		//cWSD._object = this;
		//cWSD._functions = &IWrapperSoapDeSerializer::ms_VFtable;
		//m_nStatus = ((AXIS_DESERIALIZE_FUNCT)pDZFunct)(pObject, &cWSD);
	    }
	    else
	    {
			TRACE_DESERIALIZE_FUNCT_ENTRY(pDZFunct, pObject, this);
		    m_nStatus =	((AXIS_DESERIALIZE_FUNCT) pDZFunct) (pObject, this);
			TRACE_DESERIALIZE_FUNCT_EXIT(pDZFunct, m_nStatus);
	    }
	    if (AXIS_SUCCESS == m_nStatus)
	    {
		m_pParser->next ();	/* skip end node too */
			{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (pObject);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxObject", this, 3,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pObject;
		#endif
	}

	    }
	    else
	    {
			TRACE_OBJECT_DELETE_FUNCT_ENTRY(pDelFunct, pObject, false, 0);
		    ((AXIS_OBJECT_DELETE_FUNCT) pDelFunct) (pObject, false, 0);
			TRACE_OBJECT_DELETE_FUNCT_EXIT(pDelFunct);
	    }
	}
    }
    else
    {
	if (!m_pNode)
	    /* if there is an unprocessed node that may be one left from 
	     * last array deserialization */
	    m_pNode = m_pParser->next ();
	/* wrapper node without type info  Ex: <result> */
	if (!m_pNode)
	    	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxObject", this, 4,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

	if (0 == strcmp (pName, m_pNode->m_pchNameOrValue))
	{
	    /* if this node contain attributes let them be used by the complex
	     * type's deserializer
	     */
	    if (0 != m_pNode->m_pchAttributes[0])
	    {
    		m_pCurrNode = m_pNode;
    		/*
    		 * Need to verify if the return value is NULL.
    		 */
            xsd__boolean * isNill = getAttributeAsBoolean("nil", 0);
            if (NULL != isNill)
            {
                if(true_ == *isNill)
        		{
        		    m_pParser->next ();
        		    m_pNode = NULL;
                    delete isNill;
        		    	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxObject", this, 5,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

        		} else {
        			delete isNill;
        		}	
            }
	    }
	    m_pNode = NULL;	/* node identified and used */
		TRACE_OBJECT_CREATE_FUNCT_ENTRY(pCreFunct, NULL, false, 0);
	    void *pObject = ((AXIS_OBJECT_CREATE_FUNCT) pCreFunct)(NULL, false, 0);
		TRACE_OBJECT_CREATE_FUNCT_EXIT(pCreFunct, pObject);
	    if (pObject && pDZFunct)
	    {
		if (C_DOC_PROVIDER == getCurrentProviderType ())
		{
		    // Disable C support
		    //IWrapperSoapDeSerializer_C cWSD;
		    //cWSD._object = this;
		    //cWSD._functions = &IWrapperSoapDeSerializer::ms_VFtable;
		    //m_nStatus = ((AXIS_DESERIALIZE_FUNCT)pDZFunct)
		    //    (pObject, &cWSD);
		}
		else
		{
			TRACE_DESERIALIZE_FUNCT_ENTRY(pDZFunct, pObject, this);
		    m_nStatus =	((AXIS_DESERIALIZE_FUNCT) pDZFunct) (pObject, this);
			TRACE_DESERIALIZE_FUNCT_EXIT(pDZFunct, m_nStatus);
		}
		if (AXIS_SUCCESS == m_nStatus)
		{
		    m_pParser->next ();	/* skip end node too */
		    	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (pObject);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxObject", this, 6,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pObject;
		#endif
	}

		}
		else
		{
			TRACE_OBJECT_DELETE_FUNCT_ENTRY(pDelFunct, pObject, false, 0);
		    ((AXIS_OBJECT_DELETE_FUNCT) pDelFunct) (pObject, false, 0);
			TRACE_OBJECT_DELETE_FUNCT_EXIT(pDelFunct);
		}
	    }
	}
	else
	{
	    /*
	     * TODO: Need to verify what WS-I 1.0 say
	     * about the mandatory of all the elements in the response in case of
	     * null value or none filled value. Some Web services servers work
	     * like this. This apply for all the rest of the deserializer.
	     */
	    	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxObject", this, 7,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

	}
    }
    m_nStatus = AXIS_FAIL;	/* unexpected SOAP stream */
    	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxObject", this, 8,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

const char *
SoapDeSerializer::getCmplxFaultObjectName ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getCmplxFaultObjectName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    
	if (!m_pNode)
	{
		
		// Skip the faultdetail tag
		m_pParser->next ();
	}
		
	m_nStatus = AXIS_SUCCESS;
    if (AXIS_SUCCESS != m_nStatus)
			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxFaultObjectName", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    /* if anything has gone wrong earlier just do nothing */
    if (RPC_ENCODED == m_nStyle)
    {
	
		m_pParser->next ();
		
		/* just skip wrapper node with type info
		 * Ex: <tns:QuoteInfoType xmlns:tns="http://www.getquote.org/test"> 
		 */
		if (!m_pNode)
				{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxFaultObjectName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pNode->m_pchNameOrValue);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxFaultObjectName", this, 3,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pNode->m_pchNameOrValue;
		#endif
	}

    }
    else
    {
		if (!m_pNode)
		{
		    /* if there is an unprocessed node that may be one left from 
			 * last array deserialization */
			m_pNode = m_pParser->next ();
		   	/* wrapper node without type info  Ex: <result> */
	}
	if (!m_pNode)
	    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxFaultObjectName", this, 4,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pNode->m_pchNameOrValue);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxFaultObjectName", this, 5,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pNode->m_pchNameOrValue;
		#endif
	}

    }
}

void *
SoapDeSerializer::getCmplxFaultObject (void *pDZFunct, void *pCreFunct,
				       void *pDelFunct,
				       const AxisChar * pName,
				       const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getCmplxFaultObject", this, 5,
					TRACETYPE_POINTER, 1, ((void*)&pDZFunct),
					TRACETYPE_POINTER, 1, ((void*)&pCreFunct),
					TRACETYPE_POINTER, 1, ((void*)&pDelFunct),
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    if (RPC_ENCODED == m_nStyle)
    {
	/* type  can be checked here */
	TRACE_OBJECT_CREATE_FUNCT_ENTRY(pCreFunct, NULL, false, 0);
	void *pObject =
	    ((AXIS_OBJECT_CREATE_FUNCT) pCreFunct) (NULL, false, 0);
	TRACE_OBJECT_CREATE_FUNCT_EXIT(pCreFunct, pObject);
	if (pObject && pDZFunct)
	{
	    if (C_RPC_PROVIDER == getCurrentProviderType ())
	    {
		// Disable C support
		//IWrapperSoapDeSerializer_C cWSD;
		//cWSD._object = this;
		//cWSD._functions = &IWrapperSoapDeSerializer::ms_VFtable;
		//m_nStatus = ((AXIS_DESERIALIZE_FUNCT)pDZFunct)(pObject, &cWSD);
	    }
	    else
	    {
			TRACE_DESERIALIZE_FUNCT_ENTRY(pDZFunct, pObject, this);
		    m_nStatus =	((AXIS_DESERIALIZE_FUNCT) pDZFunct) (pObject, this);
			TRACE_DESERIALIZE_FUNCT_EXIT(pDZFunct, m_nStatus);
	    }
	    if (AXIS_SUCCESS == m_nStatus)
	    {
		m_pParser->next ();	/* skip end node too */
			{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (pObject);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxFaultObject", this, 1,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pObject;
		#endif
	}

	    }
	    else
	    {
			TRACE_OBJECT_DELETE_FUNCT_ENTRY(pDelFunct, pObject, false, 0);
		    ((AXIS_OBJECT_DELETE_FUNCT) pDelFunct) (pObject, false, 0);
			TRACE_OBJECT_DELETE_FUNCT_EXIT(pDelFunct);
	    }
	}
    }
    else
    {
			
		if (0 == strcmp (pName, m_pNode->m_pchNameOrValue))
		{
	    /* if this node contain attributes let them be used by the complex
	     * type's deserializer
	     */
			if (0 != m_pNode->m_pchAttributes[0])
			{
				m_pCurrNode = m_pNode;
				xsd__boolean * isNill = getAttributeAsBoolean("nil", 0);
				if (NULL != isNill)
				{
					if(true_ == *isNill)
        			{
        				m_pParser->next ();
        				m_pNode = NULL;
						delete isNill;
        					{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxFaultObject", this, 2,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

        			} else {
        				delete isNill;
        			}
				}
			}
			m_pNode = NULL;	/* node identified and used */
			TRACE_OBJECT_CREATE_FUNCT_ENTRY(pCreFunct, NULL, false, 0);
			void *pObject = ((AXIS_OBJECT_CREATE_FUNCT) pCreFunct)(NULL, false, 0);
			TRACE_OBJECT_CREATE_FUNCT_EXIT(pCreFunct, pObject);
			if (pObject && pDZFunct)
			{
				if (C_DOC_PROVIDER == getCurrentProviderType ())
				{
				// Disable C support
				//IWrapperSoapDeSerializer_C cWSD;
				//cWSD._object = this;
				//cWSD._functions = &IWrapperSoapDeSerializer::ms_VFtable;
				//m_nStatus = ((AXIS_DESERIALIZE_FUNCT)pDZFunct)
				//    (pObject, &cWSD);
				}
				else
				{
					TRACE_DESERIALIZE_FUNCT_ENTRY(pDZFunct, pObject, this);
					m_nStatus =	((AXIS_DESERIALIZE_FUNCT) pDZFunct) (pObject, this);
					TRACE_DESERIALIZE_FUNCT_EXIT(pDZFunct, m_nStatus);
				}
				if (AXIS_SUCCESS == m_nStatus)
				{
					m_pParser->next ();	/* skip end node too */
						{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (pObject);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxFaultObject", this, 3,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pObject;
		#endif
	}

				}
				else
				{
					TRACE_OBJECT_DELETE_FUNCT_ENTRY(pDelFunct, pObject, false, 0);
					((AXIS_OBJECT_DELETE_FUNCT) pDelFunct) (pObject, false, 0);
					TRACE_OBJECT_DELETE_FUNCT_EXIT(pDelFunct);
				}
			}
		}
		else
		{
	    /*
	     * TODO: Need to verify what WS-I 1.0 say
	     * about the mandatory of all the elements in the response in case of
	     * null value or none filled value. Some Web services servers work
	     * like this. This apply for all the rest of the deserializer.
	     */
				{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxFaultObject", this, 4,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

		}
	}
    m_nStatus = AXIS_FAIL;	/* unexpected SOAP stream */
    	{
		#ifdef ENABLE_AXISTRACE
			void* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getCmplxFaultObject", this, 5,
					TRACETYPE_POINTER, 1, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

int
SoapDeSerializer::getElementForAttributes (const AxisChar * pName,
					   const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementForAttributes", NULL, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pCurrNode)
    {
	if (0 == strcmp (pName, m_pCurrNode->m_pchNameOrValue))
	    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementForAttributes", NULL, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

    }
    if (!m_pNode)
    {
	m_pNode = m_pParser->next ();
	m_pCurrNode = m_pParser->next ();
    }
    else			/* previous node not processed. So consider it */
    {
	m_pCurrNode = m_pNode;
    }
    if (0 == strcmp (pName, m_pNode->m_pchNameOrValue))
    {
	m_pNode = NULL;
    }
    else			/* error : current element is not the expected one */
    {
	m_nStatus = AXIS_FAIL;
	m_pCurrNode = NULL;
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_nStatus);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementForAttributes", NULL, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_nStatus;
		#endif
	}

}

void SoapDeSerializer::getAttribute(const AxisChar* pName, const AxisChar * pNamespace, IAnySimpleType * pSimpleType)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttribute", this, 3,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace),
					TRACETYPE_POINTER, sizeof(IAnySimpleType), ((void*)&pSimpleType));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pCurrNode)
    {
        if (START_ELEMENT == m_pCurrNode->m_type)
        {
            for (int i = 0 ; m_pCurrNode->m_pchAttributes[i]; i += 3)
            {
                if ( 0 == strcmp(m_pCurrNode->m_pchAttributes[i], pName))
                {
                    pSimpleType->deserialize(m_pCurrNode->m_pchAttributes[i+2]);
                    	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttribute", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

                }
            }
        }
        else
        {
            m_nStatus = AXIS_FAIL;
        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttribute", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

}

/*
 * Before calling any of getAttributeAs... API functions the user should move 
 * current Element to the right Element by calling GetElementForAttributes(..)
 */

xsd__int *
    SoapDeSerializer::getAttributeAsInt (const AxisChar * pName,
                                         const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsInt", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Int simpleType;
    getAttribute(pName, pNamespace, &simpleType);
		{
		#ifdef ENABLE_AXISTRACE
			xsd__int* traceRet = (simpleType.getInt());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsInt", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__int), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getInt();
		#endif
	}

}

xsd__boolean *
    SoapDeSerializer::getAttributeAsBoolean (const AxisChar * pName,
                                             const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsBoolean", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Boolean simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__boolean* traceRet = (simpleType.getBoolean());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsBoolean", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__boolean), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getBoolean();
		#endif
	}

}

xsd__unsignedInt *
    SoapDeSerializer::getAttributeAsUnsignedInt (const AxisChar * pName,
                                                 const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsUnsignedInt", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    UnsignedInt simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedInt* traceRet = (simpleType.getUnsignedInt());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsUnsignedInt", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedInt), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getUnsignedInt();
		#endif
	}

}

xsd__short *
    SoapDeSerializer::getAttributeAsShort (const AxisChar * pName,
				                           const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsShort", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Short simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__short* traceRet = (simpleType.getShort());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsShort", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__short), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getShort();
		#endif
	}

}

xsd__unsignedShort *
    SoapDeSerializer::getAttributeAsUnsignedShort (const AxisChar * pName,
                                                   const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsUnsignedShort", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    UnsignedShort simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedShort* traceRet = (simpleType.getUnsignedShort());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsUnsignedShort", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedShort), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getUnsignedShort();
		#endif
	}

}

xsd__byte *
    SoapDeSerializer::getAttributeAsByte (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsByte", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Byte simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__byte* traceRet = (simpleType.getByte());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsByte", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__byte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getByte();
		#endif
	}

}

xsd__unsignedByte *
    SoapDeSerializer::getAttributeAsUnsignedByte (const AxisChar * pName,
                                                  const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsUnsignedByte", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    UnsignedByte simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedByte* traceRet = (simpleType.getUnsignedByte());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsUnsignedByte", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedByte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getUnsignedByte();
		#endif
	}

}

xsd__long *
    SoapDeSerializer::getAttributeAsLong (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsLong", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Long simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__long* traceRet = (simpleType.getLong());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsLong", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__long), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getLong();
		#endif
	}

}

xsd__integer *
    SoapDeSerializer::getAttributeAsInteger (const AxisChar * pName,
                                             const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Integer simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__integer* traceRet = (simpleType.getInteger());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__integer), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getInteger();
		#endif
	}

}

xsd__nonNegativeInteger *
    SoapDeSerializer::getAttributeAsNonNegativeInteger (const AxisChar * pName,
                                             const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsNonNegativeInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NonNegativeInteger simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__nonNegativeInteger* traceRet = (simpleType.getNonNegativeInteger());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsNonNegativeInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__nonNegativeInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNonNegativeInteger();
		#endif
	}

}

xsd__positiveInteger *
    SoapDeSerializer::getAttributeAsPositiveInteger (const AxisChar * pName,
                                                     const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsPositiveInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    PositiveInteger simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__positiveInteger* traceRet = (simpleType.getPositiveInteger());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsPositiveInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__positiveInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getPositiveInteger();
		#endif
	}

}

xsd__nonPositiveInteger *
    SoapDeSerializer::getAttributeAsNonPositiveInteger (const AxisChar * pName,
                                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsNonPositiveInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NonPositiveInteger simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__nonPositiveInteger* traceRet = (simpleType.getNonPositiveInteger());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsNonPositiveInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__nonPositiveInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNonPositiveInteger();
		#endif
	}

}

xsd__negativeInteger *
    SoapDeSerializer::getAttributeAsNegativeInteger (const AxisChar * pName,
                                                     const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsNegativeInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NegativeInteger simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__negativeInteger* traceRet = (simpleType.getNegativeInteger());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsNegativeInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__negativeInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNegativeInteger();
		#endif
	}

}

xsd__unsignedLong *
    SoapDeSerializer::getAttributeAsUnsignedLong (const AxisChar *pName,
                                                  const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsUnsignedLong", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    UnsignedLong simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedLong* traceRet = (simpleType.getUnsignedLong());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsUnsignedLong", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedLong), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getUnsignedLong();
		#endif
	}

}

xsd__float *
    SoapDeSerializer::getAttributeAsFloat (const AxisChar * pName,
                                           const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsFloat", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Float simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__float* traceRet = (simpleType.getFloat());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsFloat", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__float), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getFloat();
		#endif
	}

}

xsd__double *
    SoapDeSerializer::getAttributeAsDouble (const AxisChar * pName,
                                            const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsDouble", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Double simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__double* traceRet = (simpleType.getDouble());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsDouble", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__double), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getDouble();
		#endif
	}

}

xsd__decimal *
    SoapDeSerializer::getAttributeAsDecimal (const AxisChar * pName,
                                             const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsDecimal", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Decimal simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__decimal* traceRet = (simpleType.getDecimal());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsDecimal", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__decimal), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getDecimal();
		#endif
	}

}

xsd__NOTATION
    SoapDeSerializer::getAttributeAsNOTATION (const AxisChar * pName,
                                              const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsNOTATION", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NOTATION simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NOTATION traceRet = (simpleType.getNOTATION());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsNOTATION", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NOTATION), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNOTATION();
		#endif
	}

}

xsd__string
    SoapDeSerializer::getAttributeAsString (const AxisChar * pName, const
                                            AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsString", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    String simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__string traceRet = (simpleType.getString());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsString", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getString();
		#endif
	}

}

xsd__normalizedString
    SoapDeSerializer::getAttributeAsNormalizedString (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsNormalizedString", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NormalizedString simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__normalizedString traceRet = (simpleType.getNormalizedString());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsNormalizedString", this, 0,
					TRACETYPE_DATA, sizeof(xsd__normalizedString), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNormalizedString();
		#endif
	}

}

xsd__token
    SoapDeSerializer::getAttributeAsToken (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsToken", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Token simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__token traceRet = (simpleType.getToken());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsToken", this, 0,
					TRACETYPE_DATA, sizeof(xsd__token), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getToken();
		#endif
	}

}

xsd__language
    SoapDeSerializer::getAttributeAsLanguage (const AxisChar * pName,
                                              const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsLanguage", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Language simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__language traceRet = (simpleType.getLanguage());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsLanguage", this, 0,
					TRACETYPE_DATA, sizeof(xsd__language), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getLanguage();
		#endif
	}

}

xsd__NMTOKEN
    SoapDeSerializer::getAttributeAsNMTOKEN (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsNMTOKEN", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NMTOKEN simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NMTOKEN traceRet = (simpleType.getNMTOKEN());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsNMTOKEN", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NMTOKEN), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNMTOKEN();
		#endif
	}

}

xsd__NMTOKENS
    SoapDeSerializer::getAttributeAsNMTOKENS (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsNMTOKENS", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NMTOKENS simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NMTOKENS traceRet = (simpleType.getNMTOKENS());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsNMTOKENS", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NMTOKENS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNMTOKENS();
		#endif
	}

}

xsd__Name
    SoapDeSerializer::getAttributeAsName (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Name simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__Name traceRet = (simpleType.getName());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__Name), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getName();
		#endif
	}

}

xsd__NCName
    SoapDeSerializer::getAttributeAsNCName (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsNCName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NCName simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NCName traceRet = (simpleType.getNCName());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsNCName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NCName), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNCName();
		#endif
	}

}

xsd__ID
    SoapDeSerializer::getAttributeAsID (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsID", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    ID simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ID traceRet = (simpleType.getID());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsID", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ID), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getID();
		#endif
	}

}

xsd__IDREF
    SoapDeSerializer::getAttributeAsIDREF (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsIDREF", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    IDREF simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__IDREF traceRet = (simpleType.getIDREF());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsIDREF", this, 0,
					TRACETYPE_DATA, sizeof(xsd__IDREF), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getIDREF();
		#endif
	}

}

xsd__IDREFS
    SoapDeSerializer::getAttributeAsIDREFS (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsIDREFS", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    IDREFS simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__IDREFS traceRet = (simpleType.getIDREFS());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsIDREFS", this, 0,
					TRACETYPE_DATA, sizeof(xsd__IDREFS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getIDREFS();
		#endif
	}

}

xsd__ENTITY
    SoapDeSerializer::getAttributeAsENTITY (const AxisChar * pName,
                                            const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsENTITY", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    ENTITY simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ENTITY traceRet = (simpleType.getENTITY());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsENTITY", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ENTITY), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getENTITY();
		#endif
	}

}

xsd__ENTITIES
    SoapDeSerializer::getAttributeAsENTITIES (const AxisChar * pName,
                                              const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsENTITIES", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    ENTITIES simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ENTITIES traceRet = (simpleType.getENTITIES());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsENTITIES", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ENTITIES), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getENTITIES();
		#endif
	}

}

xsd__anyURI
    SoapDeSerializer::getAttributeAsAnyURI (const AxisChar * pName,
                                            const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsAnyURI", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    AnyURI simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__anyURI traceRet = (simpleType.getAnyURI());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsAnyURI", this, 0,
					TRACETYPE_DATA, sizeof(xsd__anyURI), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getAnyURI();
		#endif
	}

}

xsd__QName
    SoapDeSerializer::getAttributeAsQName (const AxisChar * pName,
                                           const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsQName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    XSD_QName simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__QName traceRet = (simpleType.getQName());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsQName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__QName), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getQName();
		#endif
	}

}

xsd__hexBinary *
    SoapDeSerializer::getAttributeAsHexBinary (const AxisChar * pName,
					                           const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsHexBinary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    HexBinary simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__hexBinary* traceRet = (simpleType.getHexBinary());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsHexBinary", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__hexBinary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getHexBinary();
		#endif
	}

}

xsd__base64Binary *
    SoapDeSerializer::getAttributeAsBase64Binary (const AxisChar * pName,
                                                  const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsBase64Binary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Base64Binary simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__base64Binary* traceRet = (simpleType.getBase64Binary());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsBase64Binary", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__base64Binary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getBase64Binary();
		#endif
	}

}

xsd__dateTime *
    SoapDeSerializer::getAttributeAsDateTime (const AxisChar * pName,
                                              const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsDateTime", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    DateTime simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__dateTime* traceRet = (simpleType.getDateTime());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsDateTime", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__dateTime), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getDateTime();
		#endif
	}

}

xsd__date *
    SoapDeSerializer::getAttributeAsDate (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsDate", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Date simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__date* traceRet = (simpleType.getDate());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsDate", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__date), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getDate();
		#endif
	}

}

xsd__time *
    SoapDeSerializer::getAttributeAsTime (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsTime", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Time simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__time* traceRet = (simpleType.getTime());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsTime", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__time), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getTime();
		#endif
	}

}

xsd__gDay*
    SoapDeSerializer::getAttributeAsGDay (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsGDay", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    GDay simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gDay* traceRet = (simpleType.getGDay());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsGDay", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gDay), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getGDay();
		#endif
	}

}

xsd__gMonth*
    SoapDeSerializer::getAttributeAsGMonth (const AxisChar * pName,
                                            const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsGMonth", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

 
    GMonth simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gMonth* traceRet = (simpleType.getGMonth());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsGMonth", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gMonth), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getGMonth();
		#endif
	}

}

xsd__gMonthDay*
    SoapDeSerializer::getAttributeAsGMonthDay (const AxisChar * pName,
                                               const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsGMonthDay", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    GMonthDay simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gMonthDay* traceRet = (simpleType.getGMonthDay());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsGMonthDay", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gMonthDay), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getGMonthDay();
		#endif
	}

}

xsd__gYear*
    SoapDeSerializer::getAttributeAsGYear (const AxisChar * pName,
                                           const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsGYear", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    GYear simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gYear* traceRet = (simpleType.getGYear());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsGYear", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gYear), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getGYear();
		#endif
	}

}

xsd__gYearMonth*
    SoapDeSerializer::getAttributeAsGYearMonth (const AxisChar * pName,
                                                const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsGYearMonth", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    GYearMonth simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gYearMonth* traceRet = (simpleType.getGYearMonth());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsGYearMonth", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gYearMonth), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getGYearMonth();
		#endif
	}

}

xsd__duration *
    SoapDeSerializer::getAttributeAsDuration (const AxisChar * pName,
                                              const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttributeAsDuration", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Duration simpleType;
    getAttribute(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__duration* traceRet = (simpleType.getDuration());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttributeAsDuration", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__duration), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getDuration();
		#endif
	}

}

void SoapDeSerializer::getElement (const AxisChar * pName,
                              const AxisChar * pNamespace,
                              IAnySimpleType * pSimpleType,
							  bool isArrayElement)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElement", this, 4,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace),
					TRACETYPE_POINTER, sizeof(IAnySimpleType), ((void*)&pSimpleType),
					TRACETYPE_BOOL, 0, ((void*)&isArrayElement));	  /* AUTOINSERTED TRACE */
	#endif

    if (AXIS_SUCCESS != m_nStatus)
        	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

    if (RPC_ENCODED == m_nStyle)
    {
		bool    bNillFound = false;
		m_pNode = m_pParser->next ();
        // wrapper node with type info  Ex: <i xsi:type="xsd:int"> 
        if (!m_pNode)
		{
           	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

		}
		if (0 == strcmp (pName, m_pNode->m_pchNameOrValue))
		{
			for (int i = 0; m_pNode->m_pchAttributes[i] && !bNillFound; i += 3)
			{
				string sLocalName = m_pNode->m_pchAttributes[i];
				string sValue = m_pNode->m_pchAttributes[i + 2];
				
				if( strcmp( "nil", sLocalName.c_str()) == 0 &&
					strcmp( "true", sValue.c_str()) == 0)
				{
					bNillFound = true;
				
				}
			}
		}
        if (bNillFound || isArrayElement || (pSimpleType->getType() == getXSDType (m_pNode)))
        {
				m_pNode = m_pParser->next (true);   /* charactor node */
				if (m_pNode && (CHARACTER_ELEMENT == m_pNode->m_type))
				{
					const AxisChar* elementValue = m_pNode->m_pchNameOrValue;
										
					// FJP Added this code for fault finding.  If detail is
					//     followed by CR/LF or CR/LF then CR/LF then assume that
					//     it is not a simple object.  As added protection against
					//     false findings, the namespace has been set to an invalid
					//     value of a single space character.
					if (strlen (elementValue) < 3 && pNamespace != NULL)
					{
						if (*pNamespace == ' ')
						{
							bool bReturn = false;
        
							if (strlen (elementValue) == 0)
							{
								bReturn = true;
							}
							if (strlen (elementValue) == 1 && (*elementValue == '\n' || *elementValue == '\r'))
							{
								bReturn = true;
							}
							if (strlen (elementValue) == 2
								&& ((*elementValue == '\n' || *elementValue == '\r')
								&& (*(elementValue + 1) == '\n' || *(elementValue + 1) == '\r')))
							{
								bReturn = true;
							}
        
							if (bReturn)
							{
								m_pNode = m_pParser->next ();   /* skip end element node too */
									{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

							}
						}
					}
					pSimpleType->deserialize(elementValue);
					m_pNode = m_pParser->next ();   /* skip end element node too */
					m_pNode = NULL;
						{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

				}
				else if (m_pNode && (END_ELEMENT == m_pNode->m_type) && bNillFound  ) //xsi:nil="true"
				{
                    pSimpleType->deserialize(NULL);
					m_pNode = NULL;
						{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

				} 
				else if (END_ELEMENT == m_pNode->m_type)    // We have an empty string - Jira AXISCPP-93
				{
					pSimpleType->deserialize("");
					m_pNode = m_pParser->next();
						{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

				}
		        else
				{
					// simpleType may have xsi:nill="true" */
					//m_pNode = NULL;
					// this is important when deserializing 
					// arrays
					 
						{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

				}
        }
        else
        {
			
			/* it is an error if type is different or not present */
        }
    }
    else
    {
	 //DOC_LITERAL	
    if (!m_pNode)
        /* if there is an unprocessed node that may be one left from last 
         * array deserialization
         */
        m_pNode = m_pParser->next ();
     /* wrapper node without type info  Ex: <i> */
     if (!m_pNode)
        	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

	 if (0 == strcmp (pName, m_pNode->m_pchNameOrValue))
     {
		bool    bNillFound = false;
        for (int i = 0; m_pNode->m_pchAttributes[i] && !bNillFound; i += 3)
        {
            string sLocalName = m_pNode->m_pchAttributes[i];
            string sValue = m_pNode->m_pchAttributes[i + 2];
            if( strcmp( "nil", sLocalName.c_str()) == 0 &&
                strcmp( "true", sValue.c_str()) == 0)
            {
                bNillFound = true;
            }
        }
        m_pNode = m_pParser->next (true);   /* charactor node */
        if (m_pNode && (CHARACTER_ELEMENT == m_pNode->m_type))
        {
            
			const AxisChar* elementValue = m_pNode->m_pchNameOrValue;
           	// FJP Added this code for fault finding.  If detail is
            //     followed by CR/LF or CR/LF then CR/LF then assume that
            //     it is not a simple object.  As added protection against
            //     false findings, the namespace has been set to an invalid
            //     value of a single space character.
            if (strlen (elementValue) < 3 && pNamespace != NULL)
            {
                if (*pNamespace == ' ')
                {
                    bool bReturn = false;
					if (strlen (elementValue) == 0)
                    {
                        bReturn = true;
                    }
                    if (strlen (elementValue) == 1 && (*elementValue == '\n' || *elementValue == '\r'))
                    {
                        bReturn = true;
                    }
                    if (strlen (elementValue) == 2
                        && ((*elementValue == '\n' || *elementValue == '\r')
                        && (*(elementValue + 1) == '\n' || *(elementValue + 1) == '\r')))
                    {
                        bReturn = true;
                    }
        
                    if (bReturn)
                    {
                        m_pNode = m_pParser->next ();   /* skip end element node too */
                        	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

                    }
                }
            }
            
			pSimpleType->deserialize(elementValue);
            m_pNode = m_pParser->next ();   /* skip end element node too */
            m_pNode = NULL;
            /* this is important in doc/lit style when deserializing 
             * arrays
             */
            	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        }
        else if (m_pNode && (END_ELEMENT == m_pNode->m_type) && bNillFound ) //xsi:nil="true"
        {
            pSimpleType->deserialize(NULL);
            m_pNode = NULL;
            	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        }        
        else if (m_pNode && (END_ELEMENT == m_pNode->m_type) ) // empty tag case <tag/>
        {
            pSimpleType->deserialize("");
//            m_pNode = m_pParser->next();
            m_pNode = NULL;
            	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        }
        else
        {
            /* simpleType may have xsi:nill="true" */
//            m_pNode = NULL;
            /* this is important in doc/lit style when deserializing 
             * arrays
             */
            	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        }
     }
    else
    {
        	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        /* Not a must : m_nStatus = AXIS_FAIL; unexpected SOAP stream */
      }
    }
    m_nStatus = AXIS_FAIL;    /* unexpected SOAP stream */
    	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElement", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

}

/*
 * Deserializing Elements as values directly. 
 */

xsd__boolean *
    SoapDeSerializer::getElementAsBoolean (const AxisChar * pName,
                                           const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsBoolean", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Boolean simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__boolean* traceRet = (simpleType.getBoolean());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsBoolean", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__boolean), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getBoolean();
		#endif
	}

}

xsd__int *
    SoapDeSerializer::getElementAsInt (const AxisChar * pName,
                                       const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsInt", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Int simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__int* traceRet = (simpleType.getInt());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsInt", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__int), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getInt();
		#endif
	}

}

xsd__unsignedInt *
    SoapDeSerializer::getElementAsUnsignedInt (const AxisChar * pName,
                                               const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsUnsignedInt", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    UnsignedInt simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedInt* traceRet = (simpleType.getUnsignedInt());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsUnsignedInt", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedInt), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getUnsignedInt();
		#endif
	}

}

xsd__short *
    SoapDeSerializer::getElementAsShort (const AxisChar * pName,
                                         const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsShort", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Short simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__short* traceRet = (simpleType.getShort());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsShort", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__short), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getShort();
		#endif
	}

}

xsd__unsignedShort *
    SoapDeSerializer::getElementAsUnsignedShort (const AxisChar * pName,
                                                 const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsUnsignedShort", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    UnsignedShort simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedShort* traceRet = (simpleType.getUnsignedShort());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsUnsignedShort", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedShort), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getUnsignedShort();
		#endif
	}

}

xsd__byte *
    SoapDeSerializer::getElementAsByte (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsByte", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Byte simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__byte* traceRet = (simpleType.getByte());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsByte", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__byte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getByte();
		#endif
	}

}

xsd__unsignedByte *
    SoapDeSerializer::getElementAsUnsignedByte (const AxisChar * pName,
                                                const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsUnsignedByte", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    UnsignedByte simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedByte* traceRet = (simpleType.getUnsignedByte());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsUnsignedByte", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedByte), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getUnsignedByte();
		#endif
	}

}

xsd__long *
    SoapDeSerializer::getElementAsLong (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsLong", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Long simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__long* traceRet = (simpleType.getLong());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsLong", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__long), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getLong();
		#endif
	}

}

xsd__integer *
    SoapDeSerializer::getElementAsInteger (const AxisChar * pName,
                                           const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Integer simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__integer* traceRet = (simpleType.getInteger());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__integer), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getInteger();
		#endif
	}

}

xsd__unsignedLong *
    SoapDeSerializer::getElementAsUnsignedLong (const AxisChar * pName,
                                                const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsUnsignedLong", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    UnsignedLong simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__unsignedLong* traceRet = (simpleType.getUnsignedLong());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsUnsignedLong", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__unsignedLong), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getUnsignedLong();
		#endif
	}

}

xsd__float *
    SoapDeSerializer::getElementAsFloat (const AxisChar * pName,
                                         const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsFloat", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Float simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__float* traceRet = (simpleType.getFloat());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsFloat", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__float), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getFloat();
		#endif
	}

}

xsd__double *
    SoapDeSerializer::getElementAsDouble (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsDouble", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Double simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__double* traceRet = (simpleType.getDouble());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsDouble", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__double), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getDouble();
		#endif
	}

}

xsd__decimal *
    SoapDeSerializer::getElementAsDecimal (const AxisChar * pName,
                                           const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsDecimal", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Decimal simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__decimal* traceRet = (simpleType.getDecimal());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsDecimal", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__decimal), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getDecimal();
		#endif
	}

}

xsd__string
    SoapDeSerializer::getElementAsString (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsString", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    String simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__string traceRet = (simpleType.getString());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsString", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getString();
		#endif
	}

}

xsd__anyURI
    SoapDeSerializer::getElementAsAnyURI (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsAnyURI", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    AnyURI simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__anyURI traceRet = (simpleType.getAnyURI());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsAnyURI", this, 0,
					TRACETYPE_DATA, sizeof(xsd__anyURI), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getAnyURI();
		#endif
	}

}

xsd__QName
    SoapDeSerializer::getElementAsQName (const AxisChar * pName,
                                         const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsQName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    XSD_QName simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__QName traceRet = (simpleType.getQName());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsQName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__QName), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getQName();
		#endif
	}

}

xsd__hexBinary *
    SoapDeSerializer::getElementAsHexBinary (const AxisChar * pName,
                                             const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsHexBinary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    HexBinary simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__hexBinary* traceRet = (simpleType.getHexBinary());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsHexBinary", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__hexBinary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getHexBinary();
		#endif
	}

}

xsd__base64Binary *
    SoapDeSerializer::getElementAsBase64Binary (const AxisChar * pName,
                                                const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsBase64Binary", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Base64Binary simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__base64Binary* traceRet = (simpleType.getBase64Binary());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsBase64Binary", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__base64Binary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getBase64Binary();
		#endif
	}

}

xsd__dateTime *
    SoapDeSerializer::getElementAsDateTime (const AxisChar * pName,
                                            const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsDateTime", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    DateTime simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__dateTime* traceRet = (simpleType.getDateTime());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsDateTime", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__dateTime), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getDateTime();
		#endif
	}

}

xsd__date *
    SoapDeSerializer::getElementAsDate (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsDate", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Date simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__date* traceRet = (simpleType.getDate());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsDate", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__date), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getDate();
		#endif
	}

}

xsd__time *
    SoapDeSerializer::getElementAsTime (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsTime", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Time simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__time* traceRet = (simpleType.getTime());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsTime", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__time), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getTime();
		#endif
	}

}

xsd__duration *
    SoapDeSerializer::getElementAsDuration (const AxisChar * pName,
                                            const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsDuration", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Duration simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__duration* traceRet = (simpleType.getDuration());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsDuration", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__duration), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getDuration();
		#endif
	}

}

xsd__gYearMonth *
    SoapDeSerializer::getElementAsGYearMonth (const AxisChar * pName,
                                              const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsGYearMonth", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    GYearMonth simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gYearMonth* traceRet = (simpleType.getGYearMonth());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsGYearMonth", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gYearMonth), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getGYearMonth();
		#endif
	}

}

xsd__gYear *
    SoapDeSerializer::getElementAsGYear (const AxisChar * pName,
                                         const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsGYear", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    GYear simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gYear* traceRet = (simpleType.getGYear());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsGYear", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gYear), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getGYear();
		#endif
	}

}

xsd__gMonthDay *
    SoapDeSerializer::getElementAsGMonthDay (const AxisChar * pName,
                                             const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsGMonthDay", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    GMonthDay simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gMonthDay* traceRet = (simpleType.getGMonthDay());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsGMonthDay", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gMonthDay), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getGMonthDay();
		#endif
	}

}

xsd__gDay *
    SoapDeSerializer::getElementAsGDay (const AxisChar * pName,
                                        const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsGDay", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    GDay simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gDay* traceRet = (simpleType.getGDay());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsGDay", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gDay), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getGDay();
		#endif
	}

}

xsd__gMonth *
    SoapDeSerializer::getElementAsGMonth (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsGMonth", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    GMonth simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__gMonth* traceRet = (simpleType.getGMonth());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsGMonth", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__gMonth), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getGMonth();
		#endif
	}

}

xsd__nonPositiveInteger *
    SoapDeSerializer::getElementAsNonPositiveInteger (const AxisChar * pName,
                                                      const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsNonPositiveInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NonPositiveInteger simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__nonPositiveInteger* traceRet = (simpleType.getNonPositiveInteger());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsNonPositiveInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__nonPositiveInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNonPositiveInteger();
		#endif
	}

}

xsd__negativeInteger *
    SoapDeSerializer::getElementAsNegativeInteger (const AxisChar * pName,
                                                   const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsNegativeInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NegativeInteger simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__negativeInteger* traceRet = (simpleType.getNegativeInteger());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsNegativeInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__negativeInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNegativeInteger();
		#endif
	}

}

xsd__nonNegativeInteger *
    SoapDeSerializer::getElementAsNonNegativeInteger (const AxisChar * pName,
                                                      const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsNonNegativeInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NonNegativeInteger simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__nonNegativeInteger* traceRet = (simpleType.getNonNegativeInteger());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsNonNegativeInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__nonNegativeInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNonNegativeInteger();
		#endif
	}

}

xsd__positiveInteger *
    SoapDeSerializer::getElementAsPositiveInteger (const AxisChar * pName,
                                            const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsPositiveInteger", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    PositiveInteger simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__positiveInteger* traceRet = (simpleType.getPositiveInteger());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsPositiveInteger", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__positiveInteger), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getPositiveInteger();
		#endif
	}

}

xsd__normalizedString
    SoapDeSerializer::getElementAsNormalizedString (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsNormalizedString", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NormalizedString simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__normalizedString traceRet = (simpleType.getNormalizedString());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsNormalizedString", this, 0,
					TRACETYPE_DATA, sizeof(xsd__normalizedString), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNormalizedString();
		#endif
	}

}

xsd__token
    SoapDeSerializer::getElementAsToken (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsToken", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Token simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__token traceRet = (simpleType.getToken());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsToken", this, 0,
					TRACETYPE_DATA, sizeof(xsd__token), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getToken();
		#endif
	}

}

xsd__language
    SoapDeSerializer::getElementAsLanguage (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsLanguage", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Language simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__language traceRet = (simpleType.getLanguage());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsLanguage", this, 0,
					TRACETYPE_DATA, sizeof(xsd__language), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getLanguage();
		#endif
	}

}

xsd__Name
    SoapDeSerializer::getElementAsName (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    Name simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__Name traceRet = (simpleType.getName());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__Name), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getName();
		#endif
	}

}

xsd__NCName
    SoapDeSerializer::getElementAsNCName (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsNCName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NCName simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NCName traceRet = (simpleType.getNCName());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsNCName", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NCName), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNCName();
		#endif
	}

}

xsd__ID
    SoapDeSerializer::getElementAsID (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsID", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    ID simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ID traceRet = (simpleType.getID());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsID", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ID), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getID();
		#endif
	}

}

xsd__IDREF
    SoapDeSerializer::getElementAsIDREF (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsIDREF", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    IDREF simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__IDREF traceRet = (simpleType.getIDREF());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsIDREF", this, 0,
					TRACETYPE_DATA, sizeof(xsd__IDREF), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getIDREF();
		#endif
	}

}

xsd__IDREFS
    SoapDeSerializer::getElementAsIDREFS (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsIDREFS", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    IDREFS simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__IDREFS traceRet = (simpleType.getIDREFS());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsIDREFS", this, 0,
					TRACETYPE_DATA, sizeof(xsd__IDREFS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getIDREFS();
		#endif
	}

}

xsd__ENTITY
    SoapDeSerializer::getElementAsENTITY (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsENTITY", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    ENTITY simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ENTITY traceRet = (simpleType.getENTITY());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsENTITY", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ENTITY), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getENTITY();
		#endif
	}

}

xsd__ENTITIES
    SoapDeSerializer::getElementAsENTITIES (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsENTITIES", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    ENTITIES simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__ENTITIES traceRet = (simpleType.getENTITIES());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsENTITIES", this, 0,
					TRACETYPE_DATA, sizeof(xsd__ENTITIES), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getENTITIES();
		#endif
	}

}

xsd__NMTOKEN
    SoapDeSerializer::getElementAsNMTOKEN (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsNMTOKEN", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NMTOKEN simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NMTOKEN traceRet = (simpleType.getNMTOKEN());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsNMTOKEN", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NMTOKEN), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNMTOKEN();
		#endif
	}

}

xsd__NMTOKENS
    SoapDeSerializer::getElementAsNMTOKENS (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsNMTOKENS", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NMTOKENS simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NMTOKENS traceRet = (simpleType.getNMTOKENS());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsNMTOKENS", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NMTOKENS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNMTOKENS();
		#endif
	}

}

xsd__NOTATION
    SoapDeSerializer::getElementAsNOTATION (const AxisChar * pName,
                                          const AxisChar * pNamespace)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getElementAsNOTATION", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pName),
					TRACETYPE_STRING, 0, ((void*)&pNamespace));	  /* AUTOINSERTED TRACE */
	#endif

    NOTATION simpleType;
    getElement(pName, pNamespace, &simpleType);
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__NOTATION traceRet = (simpleType.getNOTATION());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getElementAsNOTATION", this, 0,
					TRACETYPE_DATA, sizeof(xsd__NOTATION), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return simpleType.getNOTATION();
		#endif
	}

}

xsd__base64Binary
    SoapDeSerializer::decodeFromBase64Binary (const AxisChar * pValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "decodeFromBase64Binary", NULL, 1,
					TRACETYPE_STRING, 0, ((void*)&pValue));	  /* AUTOINSERTED TRACE */
	#endif

    xsd__base64Binary value;
	xsd__int size = apr_base64_decode_len (pValue);
	xsd__unsignedByte * pTemp = new xsd__unsignedByte[size + 1];
    size = apr_base64_decode_binary (pTemp, pValue);
    pTemp[size] = 0; // Null terminate so it could used as a string
	value.set(pTemp, size);
	delete [] pTemp;
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__base64Binary traceRet = (value);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "decodeFromBase64Binary", NULL, 0,
					TRACETYPE_DATA, sizeof(xsd__base64Binary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return value;
		#endif
	}

}

xsd__hexBinary SoapDeSerializer::decodeFromHexBinary (const AxisChar * pValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "decodeFromHexBinary", NULL, 1,
					TRACETYPE_STRING, 0, ((void*)&pValue));	  /* AUTOINSERTED TRACE */
	#endif

    xsd__hexBinary value;
    xsd__int size = strlen (pValue) / 2;
	xsd__unsignedByte * pTemp = new xsd__unsignedByte[size + 1];
    Hex_Decode (pTemp, pValue);
	pTemp[size] = 0; // Null terminate so it could be used as a string
	value.set(pTemp, size);
    delete [] pTemp;
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__hexBinary traceRet = (value);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "decodeFromHexBinary", NULL, 0,
					TRACETYPE_DATA, sizeof(xsd__hexBinary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return value;
		#endif
	}

}

const xsd__string
SoapDeSerializer::getFaultAsXMLString()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getFaultAsXMLString", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (AXIS_SUCCESS != m_nStatus || RPC_ENCODED == m_nStyle)
        	{
		#ifdef ENABLE_AXISTRACE
			const xsd__string traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getFaultAsXMLString", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    AnyType *any = getAnyObject();
    int len = 1, i; // Add 1 for the null terminator
    for (i=0; i<any->_size; i++)
        if (any->_array[i])
            len += strlen(any->_array[i]);
    xsd__string ret = new char[len];
    memset(ret,0,len);
    for (i=0; i<any->_size; i++) if (any->_array[i]) 
    {
        strcat(ret,any->_array[i]);
        delete [] any->_array[i];
    }
    delete any->_array;
    delete any;
    	{
		#ifdef ENABLE_AXISTRACE
			const xsd__string traceRet = (ret);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getFaultAsXMLString", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return ret;
		#endif
	}

}

/*
 * This method checks the attribute list of pElement node for the xsi:type 
 * attribute. If it is found then finds what the type is. Otherwise returns 
 * XSD_UNKNOWN.
 */

XSDTYPE SoapDeSerializer::getXSDType (const AnyElement * pElement)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getXSDType", this, 1,
					TRACETYPE_POINTER, sizeof(AnyElement), ((void*)&pElement));	  /* AUTOINSERTED TRACE */
	#endif

    /* first check whether this is a start element node */
    if (START_ELEMENT != pElement->m_type)
    {
        	{
		#ifdef ENABLE_AXISTRACE
			XSDTYPE traceRet = (XSD_UNKNOWN);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getXSDType", this, 1,
					TRACETYPE_DATA, sizeof(XSDTYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return XSD_UNKNOWN;
		#endif
	}

    }
    for (int i = 0; pElement->m_pchAttributes[i]; i += 3)
    {
        if (URI_XSI == URIMapping::getURI (pElement->m_pchAttributes[i + 1]))
        {
            ::QName qn;
            XSDTYPE type = XSD_UNKNOWN;
            qn.splitQNameString (pElement->m_pchAttributes[i + 2], ':');
            const char * pNS = m_pParser->getNS4Prefix (qn.uri);
            if (pNS)
            {
                if (URI_XSD == URIMapping::getURI (pNS) || URI_ENC == URIMapping::getURI (pNS))
                {
                    type = TypeMapping::map (qn.localname);
                }
            }
            qn.mergeQNameString (':');
            	{
		#ifdef ENABLE_AXISTRACE
			XSDTYPE traceRet = (type);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getXSDType", this, 2,
					TRACETYPE_DATA, sizeof(XSDTYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return type;
		#endif
	}

        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			XSDTYPE traceRet = (XSD_UNKNOWN);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getXSDType", this, 3,
					TRACETYPE_DATA, sizeof(XSDTYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return XSD_UNKNOWN;
		#endif
	}

}

/*
 * Used by the Axis Engine to get any left header blocks in the deserializer 
 * even after the completion of message path. Then those headers will be added
 * to the serializer because they are probably headers targetted to nex soap 
 * processors.
 */

HeaderBlock *
SoapDeSerializer::getHeaderBlock ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getHeaderBlock", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (!m_pHeader)
		{
		#ifdef ENABLE_AXISTRACE
			HeaderBlock* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getHeaderBlock", this, 1,
					TRACETYPE_POINTER, sizeof(HeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	{
		#ifdef ENABLE_AXISTRACE
			HeaderBlock* traceRet = ((HeaderBlock *) m_pHeader->getHeaderBlock (true));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getHeaderBlock", this, 2,
					TRACETYPE_POINTER, sizeof(HeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (HeaderBlock *) m_pHeader->getHeaderBlock (true);
		#endif
	}

}

/*
 * Used probably by a handler to add a header block to the Deserializer.
 * Probably to be used by a subsequent handler in the request message path
 */

int AXISCALL
SoapDeSerializer::addHeaderBlock (IHeaderBlock * pBlk)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "addHeaderBlock", this, 1,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&pBlk));	  /* AUTOINSERTED TRACE */
	#endif

    if (!m_pHeader)
	m_pHeader = new SoapHeader ();
    m_pHeader->addHeaderBlock ((HeaderBlock *) pBlk);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "addHeaderBlock", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

xsd__hexBinary SoapDeSerializer::getBodyAsHexBinary ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getBodyAsHexBinary", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    /* TODO */
    xsd__hexBinary hb;
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__hexBinary traceRet = (hb);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBodyAsHexBinary", this, 0,
					TRACETYPE_DATA, sizeof(xsd__hexBinary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return hb;
		#endif
	}

}

xsd__base64Binary SoapDeSerializer::getBodyAsBase64Binary ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getBodyAsBase64Binary", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    /* TODO */
    //AxisChar *
	//pBodyContent = (AxisChar *) malloc (1000);
	//Chinthana:Remove malloc
	pBodyContent = new AxisChar[1000];
    pBodyContent[0] = '\0';
    m_pNode = m_pParser->next ();
    if ((START_ELEMENT == m_pNode->m_type) &&
	(0 == strcmp (m_pNode->m_pchNameOrValue,
		      SoapKeywordMapping::map (m_nSoapVersion).
		      pchWords[SKW_BODY])))
    {
	/* This is done to skip the BODY element declaration */
	m_pNode = m_pParser->next ();
    }
    while (!((END_ELEMENT == m_pNode->m_type) &&
	     (0 == strcmp (m_pNode->m_pchNameOrValue,
			   SoapKeywordMapping::map (m_nSoapVersion).
			   pchWords[SKW_BODY]))))
    {
	strcat (pBodyContent, (AnyElemntUtils::toString (m_pNode)).c_str ());
    }
    	{
		#ifdef ENABLE_AXISTRACE
			xsd__base64Binary traceRet = (decodeFromBase64Binary (pBodyContent));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBodyAsBase64Binary", this, 0,
					TRACETYPE_DATA, sizeof(xsd__base64Binary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return decodeFromBase64Binary (pBodyContent);
		#endif
	}

}

AxisChar *
SoapDeSerializer::getBodyAsChar ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getBodyAsChar", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    //AxisChar *pBodyContent = (AxisChar *) malloc (1000);
	//Chinthana:Remove malloc
	pBodyContent = new AxisChar[1000];
    pBodyContent[0] = '\0';
    m_pNode = m_pParser->next ();
    if ((START_ELEMENT == m_pNode->m_type) &&
	(0 == strcmp (m_pNode->m_pchNameOrValue,
		      SoapKeywordMapping::map (m_nSoapVersion).
		      pchWords[SKW_BODY])))
    {
	/* This is done to skip the BODY element declaration */
	m_pNode = m_pParser->next ();
    }
    while (!((END_ELEMENT == m_pNode->m_type) &&
	     (0 == strcmp (m_pNode->m_pchNameOrValue,
			   SoapKeywordMapping::map (m_nSoapVersion).
			   pchWords[SKW_BODY]))))
    {
	strcat (pBodyContent, (AnyElemntUtils::toString (m_pNode)).c_str ());
    }
    	{
		#ifdef ENABLE_AXISTRACE
			AxisChar* traceRet = (pBodyContent);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBodyAsChar", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pBodyContent;
		#endif
	}

}

int
SoapDeSerializer::setNewSoapBody (AxisChar * pNewSoapBody)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "setNewSoapBody", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pNewSoapBody));	  /* AUTOINSERTED TRACE */
	#endif

    DeSerializerMemBufInputStream stream (pNewSoapBody);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_pParser->setInputStream (&stream));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "setNewSoapBody", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pParser->setInputStream (&stream);
		#endif
	}

}

bool SoapDeSerializer::isAnyMustUnderstandHeadersLeft ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "isAnyMustUnderstandHeadersLeft", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    /* TODO */
    	{
		#ifdef ENABLE_AXISTRACE
			bool traceRet = (false);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "isAnyMustUnderstandHeadersLeft", this, 0,
					TRACETYPE_BOOL, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return false;
		#endif
	}

}

int
SoapDeSerializer::flushInputStream ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "flushInputStream", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    int nChars = 100;
    char pBuffer[100];
    if (!m_pInputStream)
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "flushInputStream", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

    while (TRANSPORT_IN_PROGRESS ==
	   m_pInputStream->getBytes (pBuffer, &nChars));
	
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "flushInputStream", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

AnyType *
SoapDeSerializer::getAnyObject ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAnyObject", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    AnyType *pAny = new AnyType ();
    pAny->_array = 0;
    pAny->_size = 0;
    int tagCount = 0;
    int lstSize = 0;
    bool bContinue = false;
    AxisString xmlStr = "";
    AxisString nsDecls = "";
    list < AxisString > lstXML;
    if (!m_pNode)
	{
    	m_pNode = m_pParser->anyNext();
	}
    while ((END_ELEMENT != m_pNode->m_type) || (tagCount >= 0) || bContinue)
    {
        // Continue if processing start prefix,
        // as we haven't yet found the true start of the tag
        if (START_PREFIX == m_pNode->m_type)
        {
            bContinue = true;
        }
        else
        {
            bContinue = false;
        }
        
        // Increment counter if entering new tag
        if (START_ELEMENT == m_pNode->m_type && START_END_ELEMENT != m_pNode->m_type2)
        {
            tagCount++;
        }
        // Decrement counter if exiting tag
        else if (END_ELEMENT == m_pNode->m_type)
        {
            tagCount--;
        }
        
    	if (START_PREFIX == m_pNode->m_type)
    	{
    	    nsDecls += " xmlns";
    	    /* why dont parser return null if there is no
    	     * prefix. Expat return null but not xerces.
    	     * TODO : will have to remove following strcmp s onece Xerces is 
    	     * corrected
    	     */
    	    if (m_pNode->m_pchNameOrValue
    		    && (strcmp (m_pNode->m_pchNameOrValue, "") != 0))
    	    {
        		nsDecls += ":";
        		nsDecls += m_pNode->m_pchNameOrValue;
    	    }
    	    nsDecls += "=\"";
    	    nsDecls += m_pNode->m_pchNamespace;
    	    nsDecls += "\"";
    	}
    	else if (CHARACTER_ELEMENT != m_pNode->m_type)
    	{
    	    serializeTag (xmlStr, m_pNode, nsDecls);
    	    nsDecls = "";
    	}
    	else
    	{
    	    xmlStr += m_pNode->m_pchNameOrValue;
    	}
    
    	if ( !bContinue && tagCount == 0 && (!xmlStr.empty ()))	/* copying the First level element into the list */
    	{
    	    lstXML.push_back (xmlStr);
    	    xmlStr = "";
            m_pNode = NULL;
            break;
    	}
    
    	m_pNode = m_pParser->anyNext ();
    
            if (!m_pNode) // Samisa: there is something wrong in the XSD Any XML strean
            {	      // Store whatever we have by now and break
                if (!xmlStr.empty ())
                {
                    lstXML.push_back (xmlStr);
                    xmlStr = "";
                }
                break;
            }
    }
    lstSize = lstXML.size ();
    pAny->_array = new char *[lstSize];
    pAny->_size = 0;
    list < AxisString >::iterator i;	/* Iterator for traversing the list */
    for (i = lstXML.begin (); i != lstXML.end (); i++)
    {
    	const char *s = (*i).c_str ();
    	pAny->_array[pAny->_size] = new char[strlen (s) + 1];
    	strcpy (pAny->_array[pAny->_size], s);
    	pAny->_size++;
    }
    	{
		#ifdef ENABLE_AXISTRACE
			AnyType* traceRet = (pAny);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAnyObject", this, 0,
					TRACETYPE_POINTER, sizeof(AnyType), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pAny;
		#endif
	}

}

void
SoapDeSerializer::serializeTag (AxisString & xmlStr, const AnyElement * node,
				AxisString & nsDecls)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "serializeTag", this, 3,
					TRACETYPE_STLSTRING, 0, ((void*)&xmlStr),
					TRACETYPE_POINTER, sizeof(AnyElement), ((void*)&node),
					TRACETYPE_STLSTRING, 0, ((void*)&nsDecls));	  /* AUTOINSERTED TRACE */
	#endif

    /*
       Note that if this is an end tag and since m_pchNameOrValue doesn't give
       the "/" sign. So we have to add that sign as well in to the end tag
     */
    const XML_Ch *pchPrefix = 0;
    if (START_ELEMENT == node->m_type)
    {
    	xmlStr += "<";
    	/* why dont parser set null if there is no
    	 * namespace. Expat set null but not xerces.
    	 * TODO : will have to remove following strcmp s onece Xerces is 
    	 * corrected
    	 */
    	if (node->m_pchNamespace && (strcmp (node->m_pchNamespace, "") != 0))
    	{
    	    pchPrefix = m_pParser->getPrefix4NS (node->m_pchNamespace);
    	    /* why dont parser return null if there is no
    	     * prefix. Expat does but not xerces.
    	     * TODO : will have to remove following strcmp s onece Xerces is 
    	     * corrected
    	     */
    	    if (pchPrefix && (strcmp (pchPrefix, "") != 0))
    	    {
        		xmlStr += pchPrefix;
    	   	   xmlStr += ":";
    	    }
    	}
    
    	xmlStr += node->m_pchNameOrValue;
        if (!nsDecls.empty ())
        {
            xmlStr += nsDecls.c_str ();
        }
    	if (node->m_pchAttributes)
    	{
    	    int j;
    
    	    /* structure of the m_pchAttributes[] array is,
    	     * sequence of (local_name, namespace_uri, value)
    	     */
    
    	    for (j = 0; j < 300; j += 3)	/* MAX_NO_OF_ATTRIBUTES = 100 */
    	    {
        		if (node->m_pchAttributes[j])
        		{
        		    if (node->m_pchAttributes[j + 1])
        		    {
            			pchPrefix = m_pParser->getPrefix4NS (node->
    						     m_pchAttributes[j + 1]);
                    }
                    else
                    {
                        pchPrefix = NULL;
                    }
        		    /* why dont parser return null if there is no
        		     * prefix. Expat does but not xerces.
        		     * TODO : will have to remove following strcmp s onece Xerces is 
        		     * corrected
        		     */
        		    if (pchPrefix && (strcmp (pchPrefix, "") != 0))
    	       	    {
    	           		xmlStr += " ";
        			    xmlStr += pchPrefix;
            			xmlStr += ":";
        		    }
                    else
                    {
                        // if there is no prefix then we need to add a space
                        xmlStr +=" ";
                    }
        		    xmlStr += node->m_pchAttributes[j];
        		    xmlStr += "=\"";
        		    xmlStr += node->m_pchAttributes[j + 2];
        		    xmlStr += "\"";
                }
        	   	else
                {
        		    break;
                }
            }
        }
        if (START_END_ELEMENT == node->m_type2)
        {
            xmlStr += "/>";
        }
        else
        {
            xmlStr += ">";
        }
    }
    else if (END_ELEMENT == node->m_type)
    {
    	xmlStr += "</";
    	/* if (node->m_pchNamespace) why dont parser set null if there is no
    	 * namespace. Expat set null but not xerces.
    	 * TODO : will have to remove following strcmp s onece Xerces is 
    	 * corrected
    	 */
    	if (node->m_pchNamespace && (strcmp (node->m_pchNamespace, "") != 0))
    	{
    	    pchPrefix = m_pParser->getPrefix4NS (node->m_pchNamespace);
    	    /* why dont parser return null if there is no
    	     * prefix. Expat does but not xerces.
    	     * TODO : will have to remove following strcmp s onece Xerces is 
    	     * corrected
    	     */
    	    if (pchPrefix && (strcmp (pchPrefix, "") != 0))
    	    {
    			xmlStr += pchPrefix;
    			xmlStr += ":";
    	    }
			else
			{
// This code is required because the namespace for the closing tag may have
// been deleted before it can be checked (m_pParser->getPrefix4NS).  If it has
// been deleted, then the code needs to look at the opening tag and use that
// namespace for the closing tag.
// This is because:-
// [2511] m_pNode = m_pParser->anyNext() calls
//   XercesHandler::endPrefixMapping() and this deletes the namespace before it
//                                     can be looked up by m_pParser->getPrefix4NS!
// Check if NameOrValue is the same as the tag name at the beginning of the XML
// string.  If it is, check if it has a namespace.  If it has, then add the
// same namespace to the XML string.  This test needs to be done because the
// namespace is being deleted before it can be checked.
//
// There has got to be a better way of doing this, but it was not obvious at
// the time!
				const char *	pszXML = xmlStr.c_str();
				char *			pNSEnd = strchr( pszXML, ':');
				char *			pTagEnd = strchr( pszXML, '>');
				if( pNSEnd && (pNSEnd < pTagEnd))
				{
					int				iNSStart = 1;
					int				iNSEnd = (int) (strchr( pszXML, ':') - pszXML) - iNSStart;
					string			sNamespace = xmlStr.substr( iNSStart, iNSEnd);
					int				iTagEnd = 0;
					char *			pSpace = strchr( pszXML, ' ');
					char *			pBrace = strchr( pszXML, '>');
					if( pSpace == NULL)
					{
						iTagEnd = pSpace - pszXML;
					}
					else if( pBrace == NULL)
					{
						iTagEnd = pBrace - pszXML;
					}
					else if( pBrace < pSpace)
					{
						iTagEnd = pBrace - pszXML;
					}
					else if( pSpace <= pBrace)
					{
						iTagEnd = pSpace - pszXML;
					}
					string	sTag = xmlStr.substr( iNSEnd + iNSStart + 1, iTagEnd - (iNSEnd + iNSStart + 1));
					if( !sTag.compare( node->m_pchNameOrValue))
					{
	    				xmlStr += sNamespace;
						xmlStr += ":";
					}
				}
			}
    	}
        xmlStr += node->m_pchNameOrValue;
        if (START_END_ELEMENT == node->m_type2)
        {
            xmlStr += "/>";
        }
        else
        {
            xmlStr += ">";
        }
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "serializeTag", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
SoapDeSerializer::getChardataAs (void *pValue, XSDTYPE type)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getChardataAs", this, 2,
					TRACETYPE_POINTER, 1, ((void*)&pValue),
					TRACETYPE_DATA, sizeof(XSDTYPE), ((void*)&type));	  /* AUTOINSERTED TRACE */
	#endif

    if (!m_pNode)
	   m_pNode = m_pParser->next (true);	/* charactor node */
    if (m_pNode && (CHARACTER_ELEMENT == m_pNode->m_type))
    {
        IAnySimpleType* pSimpleType = AxisUtils::createSimpleTypeObject(type);
        pSimpleType->deserialize(m_pNode->m_pchNameOrValue);
        pValue = pSimpleType->getValue();
        delete pSimpleType;
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getChardataAs", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

LONGLONG
SoapDeSerializer::strtoll (const char *pValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "strtoll", NULL, 1,
					TRACETYPE_STRING, 0, ((void*)&pValue));	  /* AUTOINSERTED TRACE */
	#endif

    LONGLONG llRetVal = 0;
    LONGLONG llPowerOf10 = 1;
    int iLength = strlen (pValue);
    int iCountDownTo = 0;
    bool bMinus = false;
    if (*pValue == '-')
    {
	bMinus = true;
	iCountDownTo = 1;
    }
    if (iLength > 0)
    {
	iLength--;
    }
    for (int iCount = iLength; iCount >= iCountDownTo; iCount--)
    {
	llRetVal += (LONGLONG) (pValue[iCount] - '0') * llPowerOf10;
	llPowerOf10 *= (LONGLONG) 10;
    }
    if (bMinus)
    {
	llRetVal = -llRetVal;
    }
    	{
		#ifdef ENABLE_AXISTRACE
			LONGLONG traceRet = (llRetVal);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "strtoll", NULL, 0,
					TRACETYPE_DATA, sizeof(LONGLONG), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return llRetVal;
		#endif
	}

}

/* This function is never called. */

AXIS_TRANSPORT_STATUS
SoapDeSerializer::DeSerializerMemBufInputStream::
sendBytes (const char *pcSendBuffer, const void *pBufferid)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "sendBytes", NULL, 2,
					TRACETYPE_STRING, 0, ((void*)&pcSendBuffer),
					TRACETYPE_POINTER, 1, ((void*)&pBufferid));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "sendBytes", NULL, 0,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

}

AXIS_TRANSPORT_STATUS
SoapDeSerializer::DeSerializerMemBufInputStream::getBytes (char *pcBuffer,
							   int *piRetSize)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getBytes", NULL, 2,
					TRACETYPE_STRING, 0, ((void*)&pcBuffer),
					TRACETYPE_POINTER, sizeof(int), ((void*)&piRetSize));	  /* AUTOINSERTED TRACE */
	#endif

    if (!m_pcDeSeriaMemBuffer)
		{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FAILED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBytes", NULL, 1,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FAILED;
		#endif
	}

    int
	nBufLen = strlen (m_pcDeSeriaMemBuffer);
    if (0 == nBufLen)
		{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBytes", NULL, 2,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

    nBufLen = ((*piRetSize - 1) < nBufLen) ? (*piRetSize - 1) : nBufLen;
    strncpy (pcBuffer, m_pcDeSeriaMemBuffer, nBufLen);
    pcBuffer[nBufLen] = 0;
    m_pcDeSeriaMemBuffer += nBufLen;
    	{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_IN_PROGRESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getBytes", NULL, 3,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_IN_PROGRESS;
		#endif
	}

}

ISoapAttachment* SoapDeSerializer::getAttachment(const char* pcAttachmentid)	
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAttachment", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcAttachmentid));	  /* AUTOINSERTED TRACE */
	#endif
    		
    	{
		#ifdef ENABLE_AXISTRACE
			ISoapAttachment* traceRet = (m_pInputStream->getAttachment(pcAttachmentid));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAttachment", this, 0,
					TRACETYPE_POINTER, sizeof(ISoapAttachment), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pInputStream->getAttachment(pcAttachmentid);
		#endif
	}
	 	
}

ISoapAttachment** SoapDeSerializer::getAllAttachments(int *pAttchArraySize)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getAllAttachments", this, 1,
					TRACETYPE_POINTER, sizeof(int), ((void*)&pAttchArraySize));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			ISoapAttachment** traceRet = (m_pInputStream->getAllAttachments(pAttchArraySize));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getAllAttachments", this, 0,
					TRACETYPE_POINTER, sizeof(ISoapAttachment*), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pInputStream->getAllAttachments(pAttchArraySize);
		#endif
	}

}

const char* SoapDeSerializer::getMethodNameToInvoke()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "getMethodNameToInvoke", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_strMethodNameToInvoke.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "getMethodNameToInvoke", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_strMethodNameToInvoke.c_str();
		#endif
	}

}

void SoapDeSerializer::setMethodNameToInvoke(const char* methodName)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapDeSerializer", "setMethodNameToInvoke", this, 1,
					TRACETYPE_STRING, 0, ((void*)&methodName));	  /* AUTOINSERTED TRACE */
	#endif

    m_strMethodNameToInvoke = methodName;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapDeSerializer", "setMethodNameToInvoke", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXIS_CPP_NAMESPACE_END

